////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Assistant.cpp"
//
//   CAssistant
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Assistant.h"
#include <map>
#include "./Mosaic.h"
#include <process.h> // _beginthread
#include "./WinDefAdd.h"

#define ID_TIMER_REDRAW 123

////////////////////////////////////////////////////////////////////////////////
//                            types & constants
////////////////////////////////////////////////////////////////////////////////
typedef std::multimap<const CCell*, const CCell*> MM_cpTB;
#define WORK_ABORT   false
#define WORK_SUCCESS true

typedef unsigned (__stdcall *PTHREAD_START) (void *); //   
#define chBEGINTHREADEX(psa, cbStack, pfnStartAddr,   /*   */ \
   pvParam, fdwCreate, pdwThreadId)                 \
      ((HANDLE)_beginthreadex(                      \
         (void *)        (psa),                     \
         (unsigned)      (cbStack),                 \
         (PTHREAD_START) (pfnStartAddr),            \
         (void *)        (pvParam),                 \
         (unsigned)      (fdwCreate),               \
         (unsigned *)    (pdwThreadId)))

////////////////////////////////////////////////////////////////////////////////
//                                   other
////////////////////////////////////////////////////////////////////////////////
inline bool Insert(MM_cpTB&, const CCell*, const CCell*);
inline void Delete(CCell::SET_cpBase&, const CCell*);

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////

WORK_RESULT CAssistant::FindCell(OUT CClickData &m_click) {
   if (!m_SetForOpen.empty()) {
      m_click.m_CoordCell = (*m_SetForOpen.begin())->GetCoord();
      m_click.m_bIsLeft = true;
   } else {
      if (!m_SetForFlag.empty()) {
         m_click.m_CoordCell = (*m_SetForFlag.begin())->GetCoord();
         m_click.m_bIsLeft = true;
      } else {
         if (!m_SetForAnalyse.empty()) {
            //::SetCursor(LoadCursor(::GetModuleHandle(NULL), IDC_WAIT));
            if (!Analyse(m_click)) //   
               return WORK_ABORT;
            //::SetCursor(LoadCursor(::GetModuleHandle(NULL), IDC_ARROW));
/**
            LOG_PUT(g_pLog, Put, LL_DEBUG) _T("coord = %i"), m_click.m_CoordCell);
            if (m_click.m_CoordCell != INCORRECT_COORD) {
               if (m_click.m_bIsLeft)
                  LOG_PUT(g_pLog, Put, LL_DEBUG) _T("coord for left  clik"));
               else
                  LOG_PUT(g_pLog, Put, LL_DEBUG) _T("coord for right clik"));
            }
/**/
         }
      }
   }
   return WORK_SUCCESS;
}

//const CCell *cellAnalyseSave;
//const CCell *cellInfluenceSave;

WORK_RESULT CAssistant::Analyse(OUT CClickData &m_click) {
   const int iNumberNeighbor = m_Mosaic.GetNeighborNumber();
/*
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
*/
   MM_cpTB mmPair; //  

   for (CCell::SET_cpBase::const_iterator cellAnalyse=m_SetForAnalyse.begin(); cellAnalyse!=m_SetForAnalyse.end(); cellAnalyse++) { //     m_SetForAnalyse
      //  cellAnalyse 

      CCell::SET_cpBase setInfluence, setCnF; //    setInfluence  setCnF (Close & noFlag)

      //  cellAnalyse (   )  
      //        setCnF
      for (int k=0; k<iNumberNeighbor; k++) {
         const COORD &coordNeighbor = (*cellAnalyse)->GetNeighborCoord(k);
         if (coordNeighbor == INCORRECT_COORD) continue;
         const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
         if ((cell->Cell_GetStatus() == CCell::_Close) &&
             (cell->Cell_GetClose () != CCell::_Flag))
         {
            setCnF.insert(cell);
         }
      }

      //    . setCnF.
      //    (  cellAnalyse)    setInfluence.
      for (CCell::SET_cpBase::const_iterator p=setCnF.begin(); p!=setCnF.end(); p++) {
         for (int k=0; k<iNumberNeighbor; k++) {
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if ((cell->Cell_GetStatus() == CCell::_Open) &&
                (cell != (*cellAnalyse)))
            {
               setInfluence.insert(cell);
            }
         }
      }

      if (!setInfluence.empty()) {
         for (CCell::SET_cpBase::const_iterator cellInfluence=setInfluence.begin(); cellInfluence!=setInfluence.end(); cellInfluence++) { //     setInfluence
            //  cellInfluence 

            // ,      .   -  .
            if (!Insert(mmPair, *cellAnalyse, *cellInfluence))
               continue;

            //      cellAnalyse      cellInfluence.
            if (!Analyse2Cell(*cellAnalyse, *cellInfluence, m_click)) // ,    ... :(
               return WORK_ABORT; //   
            if (m_click.m_CoordCell == INCORRECT_COORD) continue;
            return WORK_SUCCESS;
         }
      }
   }
   //if (m_click.m_CoordCell == INCORRECT_COORD) {
   //   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("coord cellAnalyse   = [%i,%i]"), cellAnalyseSave  ->GetCoord().X, cellAnalyseSave  ->GetCoord().Y);
   //   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("coord cellInfluence = [%i,%i]"), cellInfluenceSave->GetCoord().X, cellInfluenceSave->GetCoord().Y);
   //}
   //    ,  
   return WORK_SUCCESS;
}

#ifdef _DEBUG
CString toBinary(int    iVal, //    
                 size_t iLen  //   
) {
   ATLASSERT(iLen <= sizeof(int)<<3);
   //iLen = min(iLen, sizeof(int)<<3);
   CString strRes(_T('0'), (int)iLen); //     !
   for (int i=0; i<iLen; i++) {
      if (iVal & (1<<i)) {
         strRes.SetAt(strRes.GetLength()-i-1, _T('1'));
      }
   }
   return strRes;
}
typedef std::vector<int> VEC_Int;
CString toStr(const VEC_Int &VecPossibleCombinations, size_t iLenBinary) {
   CString strRes;
   for (int i=0; i<VecPossibleCombinations.size(); i++) {
      if (!strRes.IsEmpty()) {
         strRes += _T(", ");
      }
      //strRes += ::Format(_T("%i"), VecPossibleCombinations[i]);
      strRes += toBinary(VecPossibleCombinations[i], iLenBinary);
   }
   return strRes;
}
#endif

WORK_RESULT CAssistant::Analyse2Cell(const CCell *cellAnalyse, const CCell *cellInfluence, OUT CClickData &m_click) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Analyse2Cell:   [%i, %i](val %i)      [%i, %i](val %i) "), int(cellAnalyse->GetCoord().X), int(cellAnalyse->GetCoord().Y), (int)cellAnalyse->Cell_GetOpen(), int(cellInfluence->GetCoord().X), int(cellInfluence->GetCoord().Y), (int)cellInfluence->Cell_GetOpen());

   CCell::SET_cpBase SetCnF;
   const int iNumberNeighbor = m_Mosaic.GetCell(0,0)->GetNeighborNumber();
   //  cellAnalyse (   )  
   //        SetCnF
   int k=0;
   for (; k<iNumberNeighbor; k++) {
      const COORD &coordNeighbor = cellAnalyse->GetNeighborCoord(k);
      if (coordNeighbor == INCORRECT_COORD) continue;
      const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
      if ((cell->Cell_GetStatus() == CCell::_Close) &&
          (cell->Cell_GetClose () != CCell::_Flag))
      {
         SetCnF.insert(cell);
      }
   }
   //  cellInfluence (   )  
   //        SetCnF
   for (k=0; k<iNumberNeighbor; k++) {
      const COORD &coordNeighbor = cellInfluence->GetNeighborCoord(k);
      if (coordNeighbor == INCORRECT_COORD) continue;
      const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
      if ((cell->Cell_GetStatus() == CCell::_Close) &&
          (cell->Cell_GetClose () != CCell::_Flag))
      {
         SetCnF.insert(cell);
      }
   }

   if (SetCnF.size() > ((sizeof(int))<<3)) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Analyse2Cell: !!! -       Integer'e.   .   ... "));
      m_click.m_CoordCell = INCORRECT_COORD;
      return WORK_SUCCESS;
   }

   //  SetCnF -        
   //  . ..    SetCnF 
   //    (  )  ,   -
   //    iMaxCombination  2     SetCnF.
   //        integer'e...
   const int iMaxCombination = 1<<SetCnF.size(); // pow(2, SetCnF.size());
   VEC_Int VecPossibleCombinations; // 0..2^n
   for (k=0; k<iMaxCombination; k++) {
      if (m_Sync.IsNeedAbortWorkStage()) {
         return WORK_ABORT; //   
      }
      //   ...
      if (TestCombination(k, cellAnalyse, cellInfluence, SetCnF)) {
         //...    -  .
         VecPossibleCombinations.push_back(k);
      }
   }

   if (!VecPossibleCombinations.empty()) {
      // 1.              ,
      //      ,   (     SetCnF)  .
      //     ,        CClickData.
      //
      //      ...        ,
      //      ,         c.
      // ..        .
      {
         int i = 0; //  ( )    SetCnF
         const int iSize = VecPossibleCombinations.size();
         //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Analyse2Cell:       VecPossibleCombinations.size() = %i (%s)"), iSize, (LPCTSTR)toStr(VecPossibleCombinations, SetCnF.size()));
         for (CCell::SET_cpBase::const_iterator q=SetCnF.begin(); q!=SetCnF.end(); i++, q++) {
            int iMask = 1<<i; // pow(2,i); -  .          __ (VecPossibleCombinations)
            int count = 0; //  
            for (VEC_Int::const_iterator p=VecPossibleCombinations.begin(); p!=VecPossibleCombinations.end(); p++) {
               if (!(iMask & (*p))) {
                  count++;
                  if (count == iSize) break;
               }
            }
            const float fProbability = (float)count/iSize;
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Analyse2Cell:           [%i, %i] = %i/%i=%f "), int((*q)->GetCoord().X), int((*q)->GetCoord().Y), count, iSize, fProbability);
            if(m_click.m_fProbability < fProbability) {
               //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Analyse2Cell:         [%i, %i]    %f "), int((*q)->GetCoord().X), int((*q)->GetCoord().Y), fProbability);
               m_click.m_fProbability = fProbability;
               m_click.m_PrbltCell = (*q)->GetCoord();

               if (count == iSize) break; // 100%

               //cellAnalyseSave   = cellAnalyse; // for debug
               //cellInfluenceSave = cellInfluence; // for debug
            }
         }
         if(m_click.m_fProbability + 0.001 >= 1.0) // 100%
         {
            m_click.m_CoordCell = m_click.m_PrbltCell;
            m_click.m_bIsLeft = true;
            return WORK_SUCCESS;
         }
         /**
         int resultTable = 0;
         for (VEC_Int::const_iterator p=VecPossibleCombinations.begin(); p!=VecPossibleCombinations.end(); p++) {
            resultTable |= *p;
         }
         for (CCell::SET_cpBase::const_iterator q=SetCnF.begin(); q!=SetCnF.end(); q++) {
            if (!(resultTable&1)) {
               m_click.m_CoordCell = (*q)->GetCoord();
               m_click.m_bIsLeft = true;
               return WORK_SUCCESS;
            }
            resultTable = resultTable>>1;
         }
         /**/
      }
      // 2.              ,
      //      ,   (     SetCnF)  .
      //     ,        CClickData.
      {
         int iMask = iMaxCombination-1; //  
         for (VEC_Int::const_iterator p=VecPossibleCombinations.begin(); p!=VecPossibleCombinations.end(); p++) {
            iMask &= *p;
         }
         for (CCell::SET_cpBase::const_iterator q=SetCnF.begin(); q!=SetCnF.end(); q++) {
            if (iMask & 1) {
               m_click.m_CoordCell = (*q)->GetCoord();
               m_click.m_bIsLeft = false;
               return WORK_SUCCESS;
            }
            iMask = iMask>>1;
         }
      }
   }
   return WORK_SUCCESS;
}


inline bool CAssistant::TestCombination(
   const int iCombination,
   const CCell *cellAnalyse,
   const CCell *cellInfluence,
   const CCell::SET_cpBase &SetCnF) const
{
   const int iNumberNeighbor = m_Mosaic.GetCell(0,0)->GetNeighborNumber();
   int i=1;
   for (CCell::SET_cpBase::const_iterator p=SetCnF.begin(); p!=SetCnF.end(); p++, i=i<<1) {
      (*p)->SetPresumeFlag(!!(i & iCombination)); //  
   }

   int numberFlag = 0;
   int k=0;
   for (; k<iNumberNeighbor; k++) {
      const COORD &coordNeighbor = cellAnalyse->GetNeighborCoord(k);
      if (coordNeighbor == INCORRECT_COORD) continue;
      if (m_Mosaic.GetCell(coordNeighbor)->Cell_GetStatus() == CCell::_Close) {
         if (m_Mosaic.GetCell(coordNeighbor)->Cell_GetClose() == CCell::_Flag) {
            numberFlag++;
         } else {
            if (m_Mosaic.GetCell(coordNeighbor)->GetPresumeFlag()) {
               numberFlag++;
            }
         }
      }
   }
   if (cellAnalyse->Cell_GetOpen() != (CCell::EOpen)numberFlag) return false;

   numberFlag = 0;
   for (k=0; k<iNumberNeighbor; k++) {
      const COORD &coordNeighbor = cellInfluence->GetNeighborCoord(k);
      if (coordNeighbor == INCORRECT_COORD) continue;
      if (m_Mosaic.GetCell(coordNeighbor)->Cell_GetStatus() == CCell::_Close) {
         if (m_Mosaic.GetCell(coordNeighbor)->Cell_GetClose() == CCell::_Flag) {
            numberFlag++;
         } else {
            if (m_Mosaic.GetCell(coordNeighbor)->GetPresumeFlag()) {
               numberFlag++;
            }
         }
      }
   }
   if (cellInfluence->Cell_GetOpen() != (CCell::EOpen)numberFlag) return false;

   return true;
}

void CAssistant::InitForNewGame() {
   m_SetForOpen    .clear();
   m_SetForFlag    .clear();
   m_SetForAnalyse .clear();
   m_SetCloseNoFlag.clear();
   for (int i = 0; i < m_Mosaic.GetSize().cx; i++)
      for (int j = 0; j < m_Mosaic.GetSize().cy; j++)
         m_SetCloseNoFlag.insert(m_Mosaic.GetCell(i,j));
   m_SM.Reset();

   if (m_Cursor.m_hWnd == NULL) {
      m_Cursor.Create(m_Mosaic.m_hWnd,
#ifdef _WTL_VER
         CWindow::rcDefault
#else
         RECTEX(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0)
#endif
      );
   }
}

void CAssistant::ClickEnd(const CCell::CClickReportContext &ClickReportContext) {
   const int iNumberNeighbor = m_Mosaic.GetCell(0,0)->GetNeighborNumber();

   //   m_SetOpenNil
   if (!ClickReportContext.m_SetOpenNil.empty())
      for (CCell::SET_cpBase::const_iterator p=ClickReportContext.m_SetOpenNil.begin(); p!=ClickReportContext.m_SetOpenNil.end(); p++)
         Delete(m_SetCloseNoFlag, *p);

   //   m_SetOpen
   if (!ClickReportContext.m_SetOpen.empty()) {
      for (CCell::SET_cpBase::const_iterator p =ClickReportContext.m_SetOpen.begin();
                                              p!=ClickReportContext.m_SetOpen.end(); p++)
      {
         for (int k=0; k<iNumberNeighbor; k++) {
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("p=0x%08X; k=%d"), (*p), k);
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if ((cell->Cell_GetStatus() == CCell::_Close) ||
                (cell->Cell_GetOpen  () == CCell::_Nil  )) continue;
            m_SetForOpen   .insert(cell);
            m_SetForFlag   .insert(cell);
            m_SetForAnalyse.insert(cell);
         }
         m_SetForOpen   .insert(*p);
         m_SetForFlag   .insert(*p);
         m_SetForAnalyse.insert(*p);
         Delete(m_SetCloseNoFlag, *p);
      }
   }

   //   m_SetFlag
   if (!ClickReportContext.m_SetFlag.empty()) {
      for (CCell::SET_cpBase::const_iterator p=ClickReportContext.m_SetFlag.begin(); p!=ClickReportContext.m_SetFlag.end(); p++) {
         for (int k=0; k<iNumberNeighbor; k++) {
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if ((cell->Cell_GetStatus() == CCell::_Close) ||
                (cell->Cell_GetOpen  () == CCell::_Nil  )) continue;
            m_SetForOpen   .insert(cell);
            m_SetForFlag   .insert(cell);
            m_SetForAnalyse.insert(cell);
         }
         if ((*p)->Cell_GetClose() == CCell::_Flag)
            Delete(m_SetCloseNoFlag, *p);
         else
            m_SetCloseNoFlag.insert(*p); //    ...
      }
   }

   //   m_SetForFlag
   while (!m_SetForFlag.empty()) {
      bool isBreak = true;
      for (CCell::SET_cpBase::const_iterator p=m_SetForFlag.begin(); p!=m_SetForFlag.end(); p++) {
         int iCloseF = 0; // -    
         int iCloseN = 0; // -    
         for (int k=0; k<iNumberNeighbor; k++) {
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if (cell->Cell_GetStatus() == CCell::_Close)
               if (cell->Cell_GetClose() == CCell::_Flag)
                    iCloseF++;
               else iCloseN++;
         }

         if ((!iCloseN) ||                                               // 2.  ,       ;
             ((*p)->Cell_GetOpen() != (CCell::EOpen)(iCloseF+iCloseN))) // 1.  ,       -  .
         {
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetForFlag del cell=[%i,%i]"), (*p)->GetCoord().X, (*p)->GetCoord().Y);
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
            m_SetForFlag.erase(*p);
            isBreak = false;
            break; // loop for
         }
      }
      if (isBreak) break; // loop while
   }

   //   m_SetForOpen
   while (!m_SetForOpen.empty()) {
      bool isBreak = true;
      for (CCell::SET_cpBase::const_iterator p=m_SetForOpen.begin(); p!=m_SetForOpen.end(); p++) {
         int iCloseF = 0; // -    
         int iCloseN = 0; // -    
         for (int k=0; k<iNumberNeighbor; k++) {
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if (cell->Cell_GetStatus() == CCell::_Close)
               if (cell->Cell_GetClose() == CCell::_Flag)
                    iCloseF++;
               else iCloseN++;
         }
         if ((!iCloseN) ||                                     // 1.  ,       
             ((*p)->Cell_GetOpen() != (CCell::EOpen)iCloseF)) // 2.  ,      -    
         {
            //g_Log.Pur(_T("m_SetForOpen del cell=[%i,%i]"), (*p)->GetCoord().X, (*p)->GetCoord().Y);
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
            m_SetForOpen.erase(*p);
            isBreak = false;
            break; // loop for
         }
      }
      if (isBreak) break; // loop while
   }

   //   m_SetForAnalyse
   while (!m_SetForAnalyse.empty()) {
      bool isBreak = true;
      for (CCell::SET_cpBase::const_iterator p=m_SetForAnalyse.begin(); p!=m_SetForAnalyse.end(); p++) {
         int iCloseF = 0; // -    
         int iCloseN = 0; // -    
         for (int k=0; k<iNumberNeighbor; k++) {
            const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
            if (coordNeighbor == INCORRECT_COORD) continue;
            const CCell* cell = m_Mosaic.GetCell(coordNeighbor);
            if (cell->Cell_GetStatus() == CCell::_Close)
               if (cell->Cell_GetClose() == CCell::_Flag)
                    iCloseF++;
               else iCloseN++;
         }
         if ((!iCloseN) ||                                              // 1.  ,       
             ((*p)->Cell_GetOpen() == (CCell::EOpen) iCloseF) ||       // 2.  ,     -    
             ((*p)->Cell_GetOpen() == (CCell::EOpen)(iCloseF+iCloseN))) // 3.  ,      -  .
         {
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetForAnalyse del cell=[%i,%i]"), (*p)->GetCoord().X, (*p)->GetCoord().Y);
            //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
            m_SetForAnalyse.erase(*p);
            isBreak = false;
            break; // loop for
         }
      }
      if (isBreak) break; // loop while
   }

   //   :
   // 1.  m_SetForFlag -             .
   // 2.  m_SetForOpen -             .
   // 3.  m_SetForAnalyse -             (    ).
   // 4.  m_SetCloseNoFlag -      .
   //         

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetForOpen    size = %i"), m_SetForOpen   .size());
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetForFlag    size = %i"), m_SetForFlag   .size());
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetForAnalyse size = %i"), m_SetForAnalyse.size());
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("------------------"));
}

WORK_RESULT CAssistant::CSequentialMove::AddNextVectors(const int iDepth, const int iBitMask4SM) {
   if (m_Assistant.Sync().IsNeedAbortWorkStage()) {
      return WORK_ABORT; //   
   }
   if (!AnalyseVector(iBitMask4SM)) { //  "" ,   k,   
      return WORK_ABORT;
   }
   const int iSize = m_Assistant.m_SetCloseNoFlag.size();
   int depth = 0;
   int i=0;
   for (i=iSize-1; i>=0; i--) {
      if (iBitMask4SM & (1<<i)) depth++;
      if (depth == iDepth) break;
   }
   for (int j=i+1; j<iSize; j++) {
      if (iBitMask4SM & (1<<j)) break;
      int iBitMask4SM_Next = iBitMask4SM;
      iBitMask4SM_Next = iBitMask4SM_Next | (1<<j); //j'    '1'
      iBitMask4SM_Next = iBitMask4SM_Next & (~(1<<i)); //i'    '0'
      if (iDepth <= m_iMineNoFlag) {
         if (!AddNextVectors(iDepth+1, iBitMask4SM_Next)) {
            return WORK_ABORT;
         }
      }
   }
   return WORK_SUCCESS;
}

WORK_RESULT CAssistant::CSequentialMove::AnalyseVector(const int iBitMask4SM) {
   const int iSize = m_Assistant.m_SetCloseNoFlag.size();
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSequentialMove::AnalyseVector: iBitMask4SM=%s "), (LPCTSTR)toBinary(iBitMask4SM, iSize));
   const int iNumberNeighbor = m_Assistant.m_Mosaic.GetCell(0,0)->GetNeighborNumber();
   bool bResult = true;
   for (CCell::SET_cpBase::const_iterator p=m_Assistant.m_SetForAnalyse.begin(); p!=m_Assistant.m_SetForAnalyse.end(); p++) {
      if (m_Assistant.Sync().IsNeedAbortWorkStage()) {
         LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::AnalyseVector:  . "));
         return WORK_ABORT; //   
      }
      int iCloseF = 0; // -    
      int iCloseN = 0; // -     
      for (int k=0; k<iNumberNeighbor; k++) {
         const COORD &coordNeighbor = (*p)->GetNeighborCoord(k);
         if (coordNeighbor == INCORRECT_COORD) continue;
         const CCell* cell = m_Assistant.m_Mosaic.GetCell(coordNeighbor);
         if (cell->Cell_GetStatus() == CCell::_Close) {
            if (cell->Cell_GetClose() == CCell::_Flag) {
               iCloseF++;
            } else {
               int j=0;
               for (CCell::SET_cpBase::const_iterator I=m_Assistant.m_SetCloseNoFlag.begin(); I!=m_Assistant.m_SetCloseNoFlag.end(); I++, j++) {
                  if (cell == *I){
                     if (iBitMask4SM & (1<<j)) iCloseN++;
                     break;
                  }
               }
            }
         }
      }
      if ((*p)->Cell_GetOpen() != (CCell::EOpen)(iCloseF+iCloseN)){
         bResult = false;
         break;
      }
   }
   if (bResult) {
      m_VecSM.push_back(iBitMask4SM);
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::AnalyseVector:  iBitMask4SM=%s       "), (LPCTSTR)toBinary(iBitMask4SM, iSize));
   } else {
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::AnalyseVector:  iBitMask4SM=%s .    "), (LPCTSTR)toBinary(iBitMask4SM, iSize));
   }
   return WORK_SUCCESS;
}

WORK_RESULT CAssistant::CSequentialMove::CanBegin() { //      
                                                      //    -    
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSequentialMove::CanBegin: "));

   if (m_bSequentialMove) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin:       "));
      return WORK_SUCCESS;
   }

   const int iSize = m_Assistant.m_SetCloseNoFlag.size();

   //      
   if (iSize > ((sizeof(int))<<3)) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin: -   (%i)     Integer'e "), iSize);
      return WORK_SUCCESS;
   }

   if (m_Assistant.m_Mosaic.GetGameStatus() != eGSPlay) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin:        - Mosaic.GetGameStatus() != eGSPlay "));
      return WORK_SUCCESS;
   }

   int iMaxFlags = toINT(0.03*m_Assistant.m_Mosaic.GetSize().cx*m_Assistant.m_Mosaic.GetSize().cy);
   switch (m_Assistant.m_Mosaic.GetSkillLevel()) {
   case eSkillLevelBeginner    : iMaxFlags = 2; break;
   case eSkillLevelAmateur     : iMaxFlags = 4; break;
   case eSkillLevelProfessional: iMaxFlags = 6; break;
   case eSkillLevelCrazy       : iMaxFlags = 9; break;
   }

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("m_SetCloseNoFlag.size() = %i"), iSize);

   m_iMineNoFlag = 0; // -   
   for (CCell::SET_cpBase::const_iterator p=m_Assistant.m_SetCloseNoFlag.begin(); p!=m_Assistant.m_SetCloseNoFlag.end(); p++)
      if ((*p)->Cell_GetOpen() == CCell::_Mine)
         m_iMineNoFlag++;
   if (m_iMineNoFlag > iMaxFlags) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin:        -       (%i).  %i  . "), m_iMineNoFlag, iMaxFlags);
      return WORK_SUCCESS;
   }

   //    
   //        

   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::CSequentialMove::CanBegin: m_SetCloseNoFlag.size() = %i "), iSize);

   //     
   if (!AddNextVectors(1, (1<<m_iMineNoFlag)-1 /*       */)) {
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin:   "));
      return WORK_ABORT;
   }
   m_bSequentialMove = true; //     
   m_iRow = m_iCol = 0;
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::CanBegin:      "));
   return WORK_SUCCESS;//WORK_ABORT;//
}

void CAssistant::CSequentialMove::Work(CClickData &m_click) {
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSequentialMove::Work: "));
   const int iSize = m_Assistant.m_SetCloseNoFlag.size();
   CCell::EClose oldClose;

   m_click.m_bIsLeft = false;
   if (m_VecSM.size() <= m_iRow) {
      LOG_PUT(g_pLog, Put, LL_FATAL) _T("  CAssistant::CSequentialMove::Work:   !"));
   } else {
      do {
         m_click.m_Close = ((m_VecSM[m_iRow]) & (1<<m_iCol)) ? CCell::_Unknown : CCell::_Clear;
         {
            int k=0;
            for (CCell::SET_cpBase::const_iterator p=m_Assistant.m_SetCloseNoFlag.begin(); p!=m_Assistant.m_SetCloseNoFlag.end(); p++, k++)
               if (k==m_iCol) {
                  m_click.m_CoordCell = (*p)->GetCoord();
                  oldClose = (*p)->Cell_GetClose();
                  break;
               }
         }
         if (++m_iCol >= m_Assistant.m_SetCloseNoFlag.size()) {
            m_iCol = 0;
            if (++m_iRow >= m_VecSM.size()) //   ...
               m_bSequentialMove = false;
         }
      } while((m_click.m_Close == oldClose) //       
              && m_bSequentialMove);
   }
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSequentialMove::Work: "));
}

////////////////////////////////////////////////////////////////////////////////
//                            other implementation
////////////////////////////////////////////////////////////////////////////////
inline bool Insert(MM_cpTB& mm, const CCell *cellAnalyse, const CCell *cellInfluence) {
   typedef MM_cpTB::iterator I;
   std::pair<I,I> b = mm.equal_range(cellAnalyse);
   for (I i=b.first; i!=b.second; i++) {
      if ((*i).second == cellInfluence) return false; //  
   }
   typedef MM_cpTB::value_type VT;
   const VT vt1(cellAnalyse  , cellInfluence);
   const VT vt2(cellInfluence, cellAnalyse  );
   mm.insert(vt1);
   mm.insert(vt2);

 //mm.insert(std::make_pair(cellAnalyse  , cellInfluence));
 //mm.insert(std::make_pair(cellInfluence, cellAnalyse  ));
/*
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("mm.insert: [%i,%i] [%i,%i]"), cellAnalyse->GetCoord().X, cellAnalyse->GetCoord().Y, cellInfluence->GetCoord().X, cellInfluence->GetCoord().Y);
 //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("mm.size = %i"), mm.size());
*/
   return true;
}

inline void Delete(CCell::SET_cpBase& set, const CCell* cell4Del) {
   if (!set.empty()) {
      CCell::SET_cpBase::const_iterator I = set.find(cell4Del);
      if (I != set.end()) {
         set.erase(*I);
      }
   }
}

/*
#define IMAGE_JOB_SET   SendMessage(m_BtnPause.GetHandle(), BM_SETIMAGE, IMAGE_ICON, (LPARAM)GetImageBtnPause(3))
#define IMAGE_JOB_RESET SendMessage(m_BtnPause.GetHandle(), BM_SETIMAGE, IMAGE_ICON, \
                           m_Mosaic.GetGameStatusIsPlay() ?                        \
                              m_Mosaic.GetPause() ?                       \
                              (LPARAM)GetImageBtnPause(1) :      \
                              (LPARAM)GetImageBtnPause(0) :      \
                           (LPARAM)GetImageBtnPause(2))
}
/**/

void CAssistant::Work(bool bSuggestClick) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::Work: "));

   if ((m_Mosaic.GetGameStatus() == eGSEnd) &&
        m_Mosaic.GetAssistantInfo().m_bAutoStart)
   {
      m_Mosaic.GameNew(true);
   }
   if ((m_Mosaic.GetGameStatus() == eGSPlay) &&
        m_Mosaic.GetPause() &&
        m_Mosaic.GetAssistantInfo().m_bIgnorePause)
   {
      m_Mosaic.SetPause(false);
   }

   //static COORD lastClick = INCORRECT_COORD;
   //if (lastClick != INCORRECT_COORD) IMAGE_JOB_SET;
   m_click = CClickData();
   m_click.m_bSuggestClick = bSuggestClick;
   switch (m_Mosaic.GetGameStatus()) {
   default:
      break;
   case eGSReady:
      //       ,     
      m_click.m_CoordCell.X = rand(m_Mosaic.GetSize().cx-1);
      m_click.m_CoordCell.Y = rand(m_Mosaic.GetSize().cy-1);
      break;
   case eGSPlay:
      FORWARD_WM_MOSAIC_CHANGEGAMESTATUS(::GetParent(m_Mosaic.m_hWnd), -1, ::PostMessage);
      if (!bSuggestClick && m_SM.IsProcessed()) { //    ?
         m_SM.Work(m_click);                      // , .   
      } else {
         if (!FindCell(m_click)) { //    
            //   
         } else {
            if (m_click.m_CoordCell != INCORRECT_COORD) {
               //     
            } else {
               //    -      ,    -         
               if (bSuggestClick) {
                  ::BeepSpeaker(); //     ,      (/),
                                   // ..     100%    .
                  // @TODO:   -      
               } else {
                  if (!m_SM.CanBegin()) {      //      
                     //   
                  } else {
                     if (m_SM.IsProcessed()) { //    ?
                        m_SM.Work(m_click);    // ,  -   
                     } else {
                        //     -       
                        if (m_Mosaic.GetAssistantInfo().m_bStopWork /*|| bSuggestClick*/) {
                           //      
                           //      (  100% )
                        } else {
                           if (m_click.m_fProbability >= 0.5) { //   50%
                              m_click.m_CoordCell = m_click.m_PrbltCell;
                              LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Work:   [%i, %i] c  %f     "), int(m_click.m_PrbltCell.X), int(m_click.m_PrbltCell.Y), m_click.m_fProbability);
                           } else {
                              do {
                                 m_click.m_CoordCell.X = rand(m_Mosaic.GetSize().cx-1);
                                 m_click.m_CoordCell.Y = rand(m_Mosaic.GetSize().cy-1);
                              } while ((m_Mosaic.GetCell(m_click.m_CoordCell)->Cell_GetStatus() == CCell::_Open) ||
                                       (m_Mosaic.GetCell(m_click.m_CoordCell)->Cell_GetClose()  == CCell::_Flag));
                              LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Work:    [%i, %i] "), int(m_click.m_CoordCell.X), int(m_click.m_CoordCell.Y));
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   } // switch

   if (m_click.m_CoordCell != INCORRECT_COORD) {
      ::SetEvent(m_Sync.m_hEventHaveClickData);
      //OnClick();
   }

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::Work: "));
}

void CAssistant::OnClick() {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::OnClick: "));
   if (m_click.m_CoordCell != INCORRECT_COORD) {
      //     /
      //   ,     
      m_Mosaic.m_PlayInfo |= ePIPlayerAssistant; //           
                                                 //        OnClick 

      //lastClick = m_click.m_CoordCell;
      //IMAGE_JOB_RESET;
      const POINT pointCenter = m_Mosaic.GetCell(m_click.m_CoordCell)->GetCenterPixel();
      { // set cursor to new position
         if (!::IsWindowVisible(m_Cursor.m_hWnd)) {
            ::ShowWindow(m_Cursor.m_hWnd, SW_SHOWNOACTIVATE);
         }
         POINT pointCursor = ::ClientToScreen(m_Mosaic.m_hWnd, pointCenter);
         ::MoveWindow(m_Cursor.m_hWnd, pointCursor.x, pointCursor.y, 100, 100, TRUE);
      }
      if (m_click.m_bSuggestClick) {
         LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::OnClick:    -     [%i, %i] "), int(m_click.m_CoordCell.X), int(m_click.m_CoordCell.Y));
      } else {
         if (m_click.m_bIsLeft) {
            LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::OnClick:      [%i, %i] "), int(m_click.m_CoordCell.X), int(m_click.m_CoordCell.Y));
            m_Mosaic.OnLButtonDown_F(m_click.m_CoordCell);
            m_Mosaic.OnLButtonUp_F  (m_click.m_CoordCell, true);
         } else {
            LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::OnClick:      [%i, %i] "), int(m_click.m_CoordCell.X), int(m_click.m_CoordCell.Y));
            m_Mosaic.OnRButtonDown_F(m_click.m_CoordCell, true, m_click.m_Close);
         }
      }
   }
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::OnClick: "));
}

DWORD CAssistant::Thread() {
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::Thread: "));
   bool bLoop = true;
   do {
      const HANDLE pHandles[2]= {m_Sync.m_hEventCanWork, m_Sync.m_hEventDestroyThread};

      bLoop = false;
      switch (::WaitForMultipleObjects(2, pHandles, FALSE, INFINITE)) {
      case WAIT_OBJECT_0:
         //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Thread: WaitForMultipleObjects: WAIT_OBJECT_0:    . "));
         ::ResetEvent(m_Sync.m_hEventWorkAborted);
         Work(WAIT_OBJECT_0 == ::WaitForSingleObject(m_Sync.m_hEventSuggestLeftClick, 0));
         ::SetEvent(m_Sync.m_hEventWorkAborted);
         bLoop = true;
         break;
      case WAIT_OBJECT_0+1:
         LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CAssistant::Thread: WaitForMultipleObjects: WAIT_OBJECT_1:  . "));
         break;
      }
   } while(bLoop);

   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::Thread: "));
   return 0;
}

CAssistant::CAssistant(CMosaic& mosaic) :
   m_Mosaic(mosaic),
#pragma warning(disable:4355) // 'this' : used in base member initializer list
   m_SM    (*this),
   m_Sync  (*this)
#pragma warning(default:4355)
{
   DWORD dwThreadID = 0;
   m_Sync.m_hThread = 
#ifndef _MINI_EXE_
      chBEGINTHREADEX(NULL, 0, ThreadStatic, this, 0, &dwThreadID);
#else
      ::CreateThread(NULL, 0, ThreadStatic, this, 0, &dwThreadID);
#endif
}

CAssistant::~CAssistant() {
   m_Sync.AbortThread();
}

////////////////////////////////////////////////////////////////////////////////
//                               CSync
////////////////////////////////////////////////////////////////////////////////
CAssistant::CSync::CSync(CAssistant &Assistant) :
   m_Assistant(Assistant),
   m_hThread  (NULL)
{
   m_hEventDestroyThread    = ::CreateEvent(NULL, FALSE, FALSE, NULL);
   m_hEventCanWork          = ::CreateEvent(NULL, FALSE, FALSE, NULL);
   m_hEventWorkAbort        = ::CreateEvent(NULL,  TRUE, FALSE, NULL);
   m_hEventWorkAborted      = ::CreateEvent(NULL,  TRUE,  TRUE, NULL);
   m_hEventSuggestLeftClick = ::CreateEvent(NULL, FALSE, FALSE, NULL);
   m_hEventHaveClickData    = ::CreateEvent(NULL, FALSE, FALSE, NULL);
}

CAssistant::CSync::~CSync() {
   ::CloseHandle(m_hEventCanWork);          m_hEventCanWork       = NULL;
   ::CloseHandle(m_hEventDestroyThread);    m_hEventDestroyThread = NULL;
   ::CloseHandle(m_hEventWorkAbort);        m_hEventWorkAbort     = NULL;
   ::CloseHandle(m_hEventWorkAborted);      m_hEventWorkAborted   = NULL;
   ::CloseHandle(m_hEventSuggestLeftClick); m_hEventWorkAborted   = NULL;
   ::CloseHandle(m_hEventHaveClickData);    m_hEventHaveClickData = NULL;
}

bool CAssistant::CSync::HaveClickData() {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSync::HaveClickData: "));

   bool bRes = (WAIT_OBJECT_0 == ::WaitForSingleObject(m_hEventHaveClickData, 0));

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSync::HaveClickData: return %s "), bRes ? _T("true") : _T("false"));

   return bRes;
}

void CAssistant::CSync::StartWorkStage() {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSync::StartWorkStage: "));

   ::ResetEvent(m_hEventWorkAbort); // ,      
   ::SetEvent(m_hEventCanWork);

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSync::StartWorkStage: "));
}

bool CAssistant::CSync::IsNeedAbortWorkStage() {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSync::IsNeedAbortWorkStage: "));

   bool bRes = (WAIT_OBJECT_0 == ::WaitForSingleObject(m_hEventWorkAbort, 0));

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSync::IsNeedAbortWorkStage: return %s "), bRes ? _T("true") : _T("false"));

   return bRes;
}

void CAssistant::CSync::AbortAndWaitCurrentWorkStage(DWORD dwWaitTime) {
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSync::AbortAndWaitCurrentWorkStage: "));

   ::ShowWindow(m_Assistant.m_Cursor.m_hWnd, SW_HIDE);

   // 1.   ,   ...
   ::SetEvent(m_hEventWorkAbort);

   // 2.    ,   ...
   ::WaitForSingleObject(m_hEventWorkAborted, dwWaitTime);

   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSync::AbortAndWaitCurrentWorkStage: "));
}

void CAssistant::CSync::AbortThread() {
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("> CAssistant::CSync::AbortThread: "));

   // 1.       
   ::SetEvent(m_hEventDestroyThread);

   // 2.   ,   ...
   AbortAndWaitCurrentWorkStage(1000);

   // 3.    
   ::WaitForSingleObject(m_hThread, 2000);

   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("< CAssistant::CSync::AbortThread: "));
}
