////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "ButtonEx.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include <TmSchema.h>
#include "../Lib/CommonLib.h"
#include "ButtonEx.h"
#include "../DynLoadLib.h"

#ifndef _WTL_VER

///////////////////////////////////////////////////////////////////////////////
//                         implementation class
///////////////////////////////////////////////////////////////////////////////
CButtonEx::CButtonEx(HWND hWndBttn /*= NULL*/) :
   m_hWnd(NULL),
   m_pfnSuperWindowProc(NULL),
   m_fMouseOver(false),
   m_bBoldFontWhenMouseOver(true),
   m_bDrawingFocus(true)
{
   if (hWndBttn) {
      SubclassWindow(hWndBttn);
   }
}

BOOL CButtonEx::SubclassWindow(HWND hWndBttn) {
   m_hWnd = hWndBttn;
   m_pfnSuperWindowProc = (WNDPROC)::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)CButtonEx::WndProcThis);
   if (!m_pfnSuperWindowProc) {
      m_pfnSuperWindowProc = ::DefWindowProc;
   }
   ::SetWindowUserData(m_hWnd, (LONG_PTR)this);
   return !!m_hWnd;
}

HWND CButtonEx::UnsubclassWindow(BOOL bForce /*= FALSE*/) {
   WNDCLASSEX wc = {sizeof(WNDCLASSEX), 0, NULL, 0,0, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
   ::GetClassInfoEx(::GetCurrentInstance(), ::GetClassName(m_hWnd), &wc);

   WNDPROC pActiveProc = (WNDPROC)::GetWindowLongPtr(m_hWnd, GWLP_WNDPROC);

   HWND hWnd = NULL;
   if (bForce || (wc.lpfnWndProc != pActiveProc)) {
      if (!::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)m_pfnSuperWindowProc))
         if (!::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)wc.lpfnWndProc))
            return NULL;

      m_pfnSuperWindowProc = ::DefWindowProc;
      hWnd = m_hWnd;
      m_hWnd = NULL;
   }
   return hWnd;
}

/*static*/ LRESULT CALLBACK CButtonEx::WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   CButtonEx *This = (CButtonEx*)::GetWindowUserData(hWnd);
   if (This) {
      return This->WndProc(hWnd, uMsg, wParam, lParam);
   }
   return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CButtonEx::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   switch (uMsg){
   case WM_ERASEBKGND: return TRUE;
   case WM_PAINT:
      {
         PAINTSTRUCT paint;
         HDC hDC = ::BeginPaint(m_hWnd, &paint);
         DoPaint(hDC);
         ::EndPaint(m_hWnd, &paint);
         return 0;
      }
   case WM_PRINTCLIENT   : DoPaint((HDC)wParam); return 0;
   case WM_MOUSEMOVE     : OnMouseMove((int)(short)LOWORD(lParam), (int)(short)HIWORD(lParam), (UINT)(wParam)); break;
   case WM_MOUSELEAVE    : return OnMouseLeave();
   case WM_SETFOCUS      :
   case WM_KILLFOCUS     :
   case WM_CAPTURECHANGED: 
   case WM_ENABLE        :
   case BM_SETSTATE      :
   case BM_SETCHECK      :
   case WM_SETTEXT       :
   case WM_UPDATEUISTATE :
      {
         ::SendMessage(m_hWnd, WM_SETREDRAW, FALSE, 0); //  
         LRESULT lResult = ::CallWindowProc(m_pfnSuperWindowProc, m_hWnd, uMsg, wParam, lParam);
         ::SendMessage(m_hWnd, WM_SETREDRAW, TRUE, 0); //  
         if (::IsWindowVisible(m_hWnd)) {
            ::InvalidateRect(m_hWnd, NULL, FALSE);
            ::UpdateWindow(m_hWnd);
         }
         return lResult;
      }
   case WM_DESTROY    :
      m_fMouseOver = false;
      m_hWnd = NULL;
      break;
   }
   return ::CallWindowProc(m_pfnSuperWindowProc, hWnd, uMsg, wParam, lParam);
}

void CButtonEx::DrawBkgrnd(HDC hCDC, const RECT &rect) {
   HBRUSH hBrushNew = ::CreateSolidBrush(::GetSysColor(COLOR_BTNFACE));
   HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
   ::PatBlt(hCDC, rect.left,rect.top, rect.right-rect.left, rect.bottom-rect.top, PATCOPY);
   ::SelectObject(hCDC, hBrushOld);
   ::DeleteObject(hBrushNew);
}

void CButtonEx::DoPaint(HDC hDC) {
   const SIZE sizeWin = ::GetWindowSize(m_hWnd);

   HDC hCDC = ::CreateCompatibleDC(hDC);
   HBITMAP hBmpNew  = ::CreateBitmap(sizeWin);
   HGDIOBJ hBmpOld  = ::SelectObject(hCDC, hBmpNew);

   DWORD dwStyle = (DWORD)::GetWindowLongPtr(m_hWnd, GWL_STYLE);
   int iState = Button_GetState(m_hWnd); // defined in <WindowsX.h>

#ifdef _DEBUG
   if (!(((dwStyle & BS_TYPEMASK)==BS_PUSHBUTTON   ) ||
         ((dwStyle & BS_TYPEMASK)==BS_DEFPUSHBUTTON) ||
          (dwStyle & BS_PUSHLIKE))
      //||(dwStyle & BS_FLAT)
      )
   {
      _ASSERT(("  !", 0));
   }
#endif

   BOOL bChecked = !!(iState & BST_CHECKED);
   BOOL bPushed  = !!(iState & BST_PUSHED);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  State [%s];  Style=[%s] "), (LPCTSTR)CLogger::ButtonState(iState), (LPCTSTR)CLogger::WindowStyle(m_hWnd));

   RECTEX rect = ::GetClientRect(m_hWnd);
   CWinXPTheme::PF_OpenThemeData pfOpenThemeData = CWinXPTheme::OpenThemeData();
   HTHEME hTheme = pfOpenThemeData ? pfOpenThemeData(m_hWnd, L"button") : NULL;
   int iStateId = 0; // 0 or PBS_NORMAL ()

   // 1. Draw border
   if (!hTheme) {
      BOOL bDown = (bChecked || bPushed || (iState & BST_INDETERMINATE));
      ::DrawFrameControl(hCDC, &rect, DFC_BUTTON, bDown ? (DFCS_PUSHED|DFCS_BUTTONPUSH) : DFCS_BUTTONPUSH);
      /** /
      rect = RECTEX(1, 1, sizeWin.cx-1, sizeWin.cy-1);
      /**/
      if (bPushed) {
         rect.left++; rect.top++;
      } else {
         rect.right--; rect.bottom--;
      }
      ::DrawFrameControl(hCDC, &rect, DFC_BUTTON, bDown ? (DFCS_PUSHED|DFCS_BUTTONPUSH) : DFCS_BUTTONPUSH);
      rect = RECTEX(1+!!bPushed, 1+!!bPushed, sizeWin.cx-1-!bPushed, sizeWin.cy-1-!bPushed);
      /**/
   } else {
      if (bPushed || (bChecked && !m_fMouseOver))
         iStateId = PBS_PRESSED;
      else if (!::IsWindowEnabled(m_hWnd))
         iStateId = PBS_DISABLED;
      else if (m_fMouseOver)
         iStateId = PBS_HOT;
      else if (((dwStyle & BS_TYPEMASK) == BS_DEFPUSHBUTTON  ) ||
             //((dwStyle & BS_TYPEMASK) == BS_PUSHBUTTON     ) ||
               ((dwStyle & BS_TYPEMASK) == BS_AUTOCHECKBOX   ) ||
             //((dwStyle & BS_TYPEMASK) == BS_CHECKBOX       ) ||
             //((dwStyle & BS_TYPEMASK) == BS_AUTO3STATE     ) ||
               ((dwStyle & BS_TYPEMASK) == BS_3STATE         ) ||
             //((dwStyle & BS_TYPEMASK) == BS_RADIOBUTTON    ) ||
               ((dwStyle & BS_TYPEMASK) == BS_AUTORADIOBUTTON))
         iStateId = PBS_DEFAULTED;
      else
         iStateId = PBS_NORMAL;

      CWinXPTheme::PF_DrawThemeBackground pfDrawThemeBackground = CWinXPTheme::DrawThemeBackground();
      if(pfDrawThemeBackground) {
         DrawBkgrnd(hCDC, rect);
         pfDrawThemeBackground(hTheme, hCDC, BP_PUSHBUTTON, iStateId, &rect, 0);
      }
   }

   // 2. Draw background
   if (hTheme) {
      CWinXPTheme::PF_GetThemeBackgroundContentRect pfGetThemeBackgroundContentRect = CWinXPTheme::GetThemeBackgroundContentRect();
      if(pfGetThemeBackgroundContentRect) {
         HRESULT hRes = pfGetThemeBackgroundContentRect(hTheme, hCDC, BP_PUSHBUTTON, iStateId, &rect, &rect);
         if (SUCCEEDED(hRes)) {
            /**/
            {
               RECT rc = rect;
               if ((iStateId == PBS_HOT) || (iStateId == PBS_DEFAULTED)) {
                  rc.left++; rc.top++; rc.right--; rc.bottom--;
               }
               DrawBkgrnd(hCDC, rc);
            }
            /** /
            CWinXPTheme::PF_DrawThemeParentBackground pfDrawThemeParentBackground = CWinXPTheme::DrawThemeParentBackground();
            if (pfDrawThemeParentBackground) {
               RECT rc = rect;
               if ((iStateId == PBS_HOT) || (iStateId == PBS_DEFAULTED)) {
                  rc.left++; rc.top++; rc.right--; rc.bottom--;
               }
               pfDrawThemeParentBackground(m_hWnd, hCDC, &rc);
            }
            /**/
         }
      }
   } else {
      DrawBkgrnd(hCDC, rect);
   }

   // 3. Draw contents
   // 3.1. draw custom
   BOOL bDraw = DrawCustom(hCDC, rect);
   // 3.2. draw bitamp
   if (!bDraw && (dwStyle & BS_BITMAP)) {
      HBITMAP hBmp = (HBITMAP)::SendMessage(m_hWnd, BM_GETIMAGE, (WPARAM)IMAGE_BITMAP, 0L);
      if (hBmp) {
         RECTEX rc(rect);
         //rc = FindInnerRect((RECTEX)::SizeBitmap(hBmp), rect);

         HDC hCDC2 = ::CreateCompatibleDC(hCDC);
         HGDIOBJ hGObj = ::SelectObject(hCDC2, hBmp);
         ::SetStretchBltMode(hCDC, COLORONCOLOR);
         if (!m_fMouseOver) {
            rc.left++; rc.top++; rc.right--; rc.bottom--;
         }

         bDraw = ::StretchBlt(hCDC, rc, hCDC2, SizeBitmap(hBmp), SRCCOPY);
         ::SelectObject(hCDC2, hGObj);
         ::DeleteDC(hCDC2);
      }
   }
   // 3.3. draw icon
   if (!bDraw && (dwStyle & BS_ICON)) {
      HICON hIcon = (HICON)::SendMessage(m_hWnd, BM_GETIMAGE, (WPARAM)IMAGE_ICON, 0L);
      if (hIcon) {
         RECTEX rc = FindInnerRect((RECTEX)::SizeIcon(hIcon), rect);
         if (!m_fMouseOver) {
            rc.left++; rc.top++; rc.right--; rc.bottom--;
         }
         bDraw = ::DrawIconEx(hCDC,
            rc.left, rc.top,
            hIcon,
            rc.width(), rc.height(),
            0, NULL, DI_NORMAL);
      }
   }
   // 3.4. draw text
   if (!bDraw) {
      ::SetBkMode(hCDC, TRANSPARENT);

      HFONT hFontBttn = (HFONT)::SendMessage(m_hWnd, WM_GETFONT, 0, 0);
      if (hFontBttn && (::GetObjectType(hFontBttn) != OBJ_FONT))
         hFontBttn = NULL;
      HFONT hFontOld = NULL;
      HFONT hFontNew = NULL;
      if (m_fMouseOver && m_bBoldFontWhenMouseOver) {
         LOGFONT logFont;
         if (sizeof(LOGFONT) == ::GetObject(hFontBttn, sizeof(LOGFONT), &logFont)) {
            logFont.lfWeight = FW_BOLD;
            hFontNew = ::CreateFontIndirect(&logFont);
            hFontOld = (HFONT)::SelectObject(hCDC, hFontNew);
         }
      } else {
         hFontOld = hFontBttn ? (HFONT)::SelectObject(hCDC, hFontBttn) : NULL;
      }
      CString strText = ::GetWindowText(m_hWnd);
      if (::IsWindowEnabled(m_hWnd)) {
         ::SetTextColor(hCDC, hTheme ? 0 : ::GetSysColor(COLOR_BTNTEXT));
         RECTEX rc = rect;
         if (!hTheme && bPushed)
            rc.move(1,1);
         bDraw = !!::DrawText(hCDC, strText, strText.GetLength(), &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
      } else {
         if (!hTheme) {
            ::SetTextColor(hCDC, 0xFFFFFF);
            bDraw = !!::DrawText(hCDC, strText, strText.GetLength(), &RECTEX(rect).move(1,1), DT_CENTER | DT_VCENTER | DT_SINGLELINE);
         }
         ::SetTextColor(hCDC, ::GetSysColor(COLOR_GRAYTEXT));
         bDraw &= !!::DrawText(hCDC, strText, strText.GetLength(), &rect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
      }
      if (hFontOld)
         ::SelectObject(hCDC, hFontOld);
      if (hFontNew)
         ::DeleteObject(hFontNew);
   }
   if (m_bDrawingFocus)
      if (iState & BST_FOCUS) { //if (m_hWnd == ::GetFocus()) { //
         if (!hTheme)
            ::InflateRect(&rect, -::GetSystemMetrics(SM_CXEDGE), -::GetSystemMetrics(SM_CYEDGE));
         ::DrawFocusRect(hCDC, &rect);
      }

   ::BitBlt(hDC, sizeWin, hCDC, SRCCOPY);

   ::SelectObject(hCDC, hBmpOld);
   ::DeleteObject(hBmpNew);

   ::DeleteDC(hCDC); hCDC = NULL;

   CWinXPTheme::PF_CloseThemeData pfCloseThemeData = CWinXPTheme::CloseThemeData();
   if (pfCloseThemeData && hTheme) {
      pfCloseThemeData(hTheme); hTheme = NULL;
   }
}

void CButtonEx::OnMouseMove(int x, int y, UINT keyFlags) {
   if(::GetCapture() == m_hWnd) {
      ::InvalidateRect(m_hWnd, NULL, FALSE);
      ::UpdateWindow(m_hWnd);
   } else if(!m_fMouseOver)
	   {
		   m_fMouseOver = true;
		   ::InvalidateRect(m_hWnd, NULL, FALSE);
		   ::UpdateWindow(m_hWnd);
		   StartTrackMouseLeave();
	   }
   return;
}

LRESULT CButtonEx::OnMouseLeave() {
   if (m_fMouseOver) {
      m_fMouseOver = false;
      ::InvalidateRect(m_hWnd, NULL, FALSE);
      ::UpdateWindow(m_hWnd);
   }
   return 0;
}

BOOL CButtonEx::StartTrackMouseLeave() {
	TRACKMOUSEEVENT tme;
	tme.cbSize = sizeof(tme);
	tme.dwFlags = TME_LEAVE;
	tme.hwndTrack = m_hWnd;
	return _TrackMouseEvent(&tme);
}

#endif // _WTL_VER
