////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Table.cpp"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Table.h"

#ifndef _WTL_VER

CTable::CTable(HWND hWnd):
   m_hWnd       (NULL),
   m_iDefH      (27), //   ,      
   m_iDefW      (100),
   m_CurrentCell(INVALID_CELL()),
   m_bSelect    (false),
   m_iStaticCol (0),
   m_iStaticRow (1)
{
   static WNDCLASS wc = {
      CS_DBLCLKS,                        // UINT    style
      CTable::WndProcThis,               // WNDPROC lpfnWndProc
      0,                                 // int     cbClsExtra
      0,                                 // int     cbWndExtra
      ::GetModuleHandle(NULL),           // HANDLE  hInstance
      (HICON)0,                          // HICON   hIcon
      ::LoadCursor(NULL, IDC_ARROW),     // HCURSOR hCursor
      ::GetSysColorBrush(COLOR_BTNFACE), // HBRUSH  hbrBackground
      NULL,                              // LPCTSTR lpszMenuName
      CTable::GetWindowClass()           // LPCTSTR lpszClassName
   };
   static ATOM atom = ::RegisterClass(&wc);

   if (hWnd) {
      SubclassWindow(hWnd);
   }
}

BOOL CTable::SubclassWindow(HWND hWnd) {
   m_hWnd = hWnd;
   ::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LPARAM)CTable::WndProcThis);
   ::SetWindowUserData(m_hWnd, (LONG_PTR)this);
   return !!m_hWnd;
}

CTable::~CTable() {
   for (SHORT i=0; i<m_ColW.size(); i++)
      for (SHORT j=0; j<m_RowH.size(); j++)
         delete m_Data[i][j];
}

/* static */ LRESULT CALLBACK CTable::WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   CTable *This = NULL;
   if (uMsg == WM_CREATE) {
      CREATESTRUCT *pCS = (CREATESTRUCT*)lParam;
      This = pCS ? (CTable*)pCS->lpCreateParams : NULL;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }            
   } else {
      This = (CTable*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->WndProc(hWnd, uMsg, wParam, lParam);
   }

   return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

HWND CTable::Create(
   HWND hWndParent, RECT &rcPos, LPCTSTR szWindowName,
   DWORD dwStyle, DWORD dwExStyle, UINT nID)
{
   m_hWnd = ::CreateWindowEx(
      WS_EX_CLIENTEDGE,
      CTable::GetWindowClass(), szWindowName, // NULL,
      WS_GROUP | WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL | WS_HSCROLL,
      rcPos.left, rcPos.top, rcPos.right-rcPos.left, rcPos.bottom-rcPos.top, //CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      hWndParent, (HMENU)nID, ::GetModuleHandle(NULL), this);
   return m_hWnd;
}

BOOL CTable::OnCreate(HWND, LPCREATESTRUCT lpCreateStruct) {
   SetRowNumber(11);
   SetColNumber(6);

   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   SCROLLINFO scInfoHorz = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0, Width(m_ColW.size())+1,
      min(rcClient.right-rcClient.left, Width(m_ColW.size())),
      0, 0
   };
   ::SetScrollInfo(m_hWnd, SB_HORZ, &scInfoHorz, TRUE);
   SCROLLINFO scInfoVert = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0, Height(m_RowH.size())+1,
      min(rcClient.bottom-rcClient.top, Height(m_RowH.size())),
      0, 0
   };
   ::SetScrollInfo(m_hWnd, SB_VERT, &scInfoVert, TRUE);
   return TRUE;
}

void CTable::SetColNumber(SHORT nCol) {
   if (nCol < 0) return;
   SHORT nColOld = m_ColW.size();
   if (nCol == nColOld) return;

   { //   -  
      for (SHORT i=nCol; i<nColOld; i++)
         for (SHORT j=0; j<m_RowH.size(); j++) {
            delete m_Data[i][j]; m_Data[i][j] = NULL;
         }
   }
   UINT nWidthMaxOld = Width(nColOld);

   { //   
      m_ColW.resize(nCol);
      m_Data.resize(nCol);
      for (SHORT i=nColOld; i<nCol; i++) { //   
         m_ColW[i] = m_iDefW;
         m_Data[i].resize(m_RowH.size());
      }
   }
   { //   -  
      for (SHORT i=nColOld; i<nCol; i++)
         for (SHORT j=0; j<m_RowH.size(); j++)
            m_Data[i][j] = new CTableData;
   }

   if (m_CurrentCell.X > nCol-1) SetCurrentCell(nCol-1, m_CurrentCell.Y, m_bSelect);

   RecountScrollInfo();
   RECT rcInvalidate = {-::GetScrollPos(m_hWnd, SB_HORZ)+min(nWidthMaxOld, Width(m_ColW.size())),
                        -::GetScrollPos(m_hWnd, SB_VERT),
                        -::GetScrollPos(m_hWnd, SB_HORZ)+max(nWidthMaxOld, Width(m_ColW.size()))+1,
                        -::GetScrollPos(m_hWnd, SB_VERT)+Height(m_RowH.size())+1};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

void CTable::SetRowNumber(SHORT nRow) {
   if (nRow < 0) return;
   SHORT nRowOld = m_RowH.size();
   if (nRow == nRowOld) return;

   { //   -  
      for (SHORT i=0; i<m_ColW.size(); i++)
         for (SHORT  j=nRow; j<nRowOld; j++) {
            delete m_Data[i][j]; m_Data[i][j] = NULL;
         }
   }
   UINT nHeightMaxOld = Height(nRowOld);

   { //   
      m_RowH.resize(nRow);
      for (SHORT i=0; i<m_ColW.size(); i++)
         m_Data[i].resize(nRow);
      for (SHORT j=nRowOld; j<nRow; j++)
         m_RowH[j] = m_iDefH;
   }
   { //   -  
      for (SHORT i=0; i<m_ColW.size(); i++)
         for (SHORT j=nRowOld; j<nRow; j++)
            m_Data[i][j] = new CTableData;
   }

   if (m_CurrentCell.Y > nRow-1) SetCurrentCell(m_CurrentCell.X, nRow-1, m_bSelect);

   RecountScrollInfo();
   RECT rcInvalidate = {-::GetScrollPos(m_hWnd, SB_HORZ),
                        -::GetScrollPos(m_hWnd, SB_VERT)+min(nHeightMaxOld, Height(m_RowH.size())),
                        -::GetScrollPos(m_hWnd, SB_HORZ)+Width(m_ColW.size())+1,
                        -::GetScrollPos(m_hWnd, SB_VERT)+max(nHeightMaxOld, Height(m_RowH.size()))+1};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

bool CTable::SetColWidth(SHORT indexCol, UINT width) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size())) return false;
   if (m_ColW[indexCol] == width) return false;

   UINT nWidthMaxOld = Width(m_ColW.size());
   m_ColW[indexCol] = width;
   RecountScrollInfo();
   RECT rcInvalidate = {-::GetScrollPos(m_hWnd, SB_HORZ)+Width(indexCol),
                        -::GetScrollPos(m_hWnd, SB_VERT),
                        -::GetScrollPos(m_hWnd, SB_HORZ)+max(nWidthMaxOld, Width(m_ColW.size()))+1,
                        -::GetScrollPos(m_hWnd, SB_VERT)+Height(m_RowH.size())+1};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
   return true;
}

UINT CTable::GetColWidth(SHORT indexCol) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size())) return -1;
   return m_ColW[indexCol];
}

bool CTable::SetRowHeight(SHORT indexRow, UINT height) {
   if ((indexRow < 0) || (indexRow >= m_RowH.size())) return false;
   if (m_RowH[indexRow] == height) return false;

   UINT nHeightMaxOld = Height(m_RowH.size());
   m_RowH[indexRow] = height;
   RecountScrollInfo();
   RECT rcInvalidate = {-::GetScrollPos(m_hWnd, SB_HORZ),
                        -::GetScrollPos(m_hWnd, SB_VERT)+Height(indexRow),
                        -::GetScrollPos(m_hWnd, SB_HORZ)+Width(m_ColW.size())+1,
                        -::GetScrollPos(m_hWnd, SB_VERT)+max(nHeightMaxOld, Height(m_RowH.size()))+1};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
   return true;
}

UINT CTable::GetRowHeight(SHORT indexRow) {
   if ((indexRow < 0) || (indexRow >= m_RowH.size())) return -1;
   return m_RowH[indexRow];
}

UINT CTable::Width(SHORT col) const { //   col 
   UINT result = 0;
   for (SHORT i=0; i<min(m_ColW.size(), col); i++)
      result += m_ColW[i];
   return result;
}

UINT CTable::Height(SHORT row) const { //   row 
   UINT result = 0;
   for (SHORT j=0; j<min(m_RowH.size(), row); j++)
      result += m_RowH[j];
   return result;
}

void CTable::SetText(SHORT indexCol, SHORT indexRow, LPCTSTR str) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   m_Data[indexCol][indexRow]->m_strText = str;
   UINT x = (indexCol < m_iStaticCol) ? 0 : ::GetScrollPos(m_hWnd, SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : ::GetScrollPos(m_hWnd, SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

void CTable::SetFormatText(SHORT indexCol, SHORT indexRow, UINT m_uFormatText) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   m_Data[indexCol][indexRow]->m_uFormatText = m_uFormatText;
   UINT x = (indexCol < m_iStaticCol) ? 0 : ::GetScrollPos(m_hWnd, SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : ::GetScrollPos(m_hWnd, SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

void CTable::SetImage(SHORT indexCol, SHORT indexRow, const CImageSerialize *pImage) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;

#define m_pImage (m_Data[indexCol][indexRow]->m_pImage)
   if (m_pImage) {
      delete m_pImage; m_pImage = NULL;
   }
   if (pImage && !pImage->m_strPath.IsEmpty()) {
      CImage img(*pImage);
      if (img.Valid())
         m_pImage = new CImage(*pImage);
   }
#undef m_pImage
   UINT x = (indexCol < m_iStaticCol) ? 0 : ::GetScrollPos(m_hWnd, SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : ::GetScrollPos(m_hWnd, SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

void CTable::SetCurrentCell(SHORT indexCol, SHORT indexRow, bool isSelect) {
   if ((indexCol < m_iStaticCol) || (indexCol >= m_ColW.size()) ||
       (indexRow < m_iStaticRow) || (indexRow >= m_RowH.size())) return;
   COORD newCell = {indexCol, indexRow};
   if ((m_CurrentCell == newCell) && (m_bSelect == isSelect)) return;

   RECT rcInvalidateOld = {Width(m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                           Width(m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   m_bSelect = isSelect;
   m_CurrentCell = newCell;
   RECT rcInvalidateNew = {Width(m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                           Width(m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   RECT rcInvalidate = {min(rcInvalidateOld.left  ,rcInvalidateNew.left  )-::GetScrollPos(m_hWnd, SB_HORZ),
                        min(rcInvalidateOld.top   ,rcInvalidateNew.top   )-::GetScrollPos(m_hWnd, SB_VERT),
                        max(rcInvalidateOld.right ,rcInvalidateNew.right )-::GetScrollPos(m_hWnd, SB_HORZ),
                        max(rcInvalidateOld.bottom,rcInvalidateNew.bottom)-::GetScrollPos(m_hWnd, SB_VERT)};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
   SetVisibleCell(indexCol, indexRow);
}

LPCTSTR CTable::GetCurrentText() const {
   if (m_CurrentCell == INVALID_CELL()) {
      return NULL;
   }
   return m_Data[m_CurrentCell.X][m_CurrentCell.Y]->m_strText;
}

void CTable::UnselectCell() {
   if (m_CurrentCell == INVALID_CELL()) return;
   RECT rcInvalidate = {Width(m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                        Width(m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   m_CurrentCell = INVALID_CELL();
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

COORD CTable::FindCell(int xWin, int yWin) const {
   UINT x = xWin+::GetScrollPos(m_hWnd, SB_HORZ);
   UINT y = yWin+::GetScrollPos(m_hWnd, SB_VERT);
   COORD resultCell = INVALID_CELL();
   UINT result = 0;
   SHORT i = 0;
   for (; i<m_ColW.size(); i++) {
      if ((x >= result) && (x < result+m_ColW[i])) {
         resultCell.X = i;
         break;
      }
      result += m_ColW[i];
   }
   result = 0;
   for (i=0; i<m_RowH.size(); i++) {
      if ((y >= result) && (y < result+m_RowH[i])) {
         resultCell.Y = i;
         break;
      }
      result += m_RowH[i];
   }
   return ((resultCell.X == INVALID_CELL().X) ||
           (resultCell.Y == INVALID_CELL().Y)) ?
           INVALID_CELL() :
           resultCell;
}

void CTable::RecountScrollInfo() {
   RECT rcClient = GetClientRect(m_hWnd);
   UINT x = Width (m_ColW.size());
   UINT y = Height(m_RowH.size());
   UINT h = ::GetScrollPos(m_hWnd, SB_HORZ);
   UINT v = ::GetScrollPos(m_hWnd, SB_VERT);
   SCROLLINFO scInfoHorz = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0,  x-1,
      min(x,     rcClient.right),
      min(x, min(rcClient.right, h)),
   };
   ::SetScrollInfo(m_hWnd, SB_HORZ, &scInfoHorz, TRUE);
   static bool oldShowSBHorz = (rcClient.right < x);
          bool newShowSBHorz = (rcClient.right < x);
   if (oldShowSBHorz != newShowSBHorz) {
      oldShowSBHorz = newShowSBHorz;
      ::ShowScrollBar(m_hWnd, SB_HORZ, newShowSBHorz); //      ,   ...
      RecountScrollInfo();                             // ...     
   }
   //::EnableScrollBar(m_hWnd, SB_HORZ, (rcClient.right < x) ? ESB_ENABLE_BOTH : ESB_DISABLE_BOTH);
   SCROLLINFO scInfoVert = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0,  y-1,
      min(y,     rcClient.bottom),
      min(y, min(rcClient.bottom, v)),
   };
   ::SetScrollInfo(m_hWnd, SB_VERT, &scInfoVert, TRUE);
   static bool oldShowSBVert = (rcClient.bottom < y);
          bool newShowSBVert = (rcClient.bottom < y);
   if (oldShowSBVert != newShowSBVert) {
      oldShowSBVert = newShowSBVert;
      ::ShowScrollBar(m_hWnd, SB_VERT, newShowSBVert); //      ,   ...
      RecountScrollInfo();                         // ...     
   }
   //::EnableScrollBar(m_hWnd, SB_VERT, (rcClient.bottom < y) ? ESB_ENABLE_BOTH : ESB_DISABLE_BOTH);
}

bool CTable::VisibleCell(SHORT indexCol, SHORT indexRow) const { // a  ? (  ?)
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return false; //  
   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   rcClient.left += Width (m_iStaticCol);
   rcClient.top  += Height(m_iStaticRow);
   if ((rcClient.left   <= Width (indexCol  )-::GetScrollPos(m_hWnd, SB_HORZ)) &&
       (rcClient.right  >= Width (indexCol+1)-::GetScrollPos(m_hWnd, SB_HORZ)) &&
       (rcClient.top    <= Height(indexRow  )-::GetScrollPos(m_hWnd, SB_VERT)) &&
       (rcClient.bottom >= Height(indexRow+1)-::GetScrollPos(m_hWnd, SB_VERT)))
      return true;
   else
      return false;
}

void CTable::SetVisibleCell(SHORT indexCol, SHORT indexRow) { //    ()  
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   if (VisibleCell(indexCol, indexRow)) {
      return; //      
   }
   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   if(::GetScrollPos(m_hWnd, SB_HORZ)<(Width (indexCol+1)-rcClient.right))
      ::SetScrollPos(m_hWnd, SB_HORZ,  Width (indexCol+1)-rcClient.right,  TRUE);
   if(::GetScrollPos(m_hWnd, SB_VERT)<(Height(indexRow+1)-rcClient.bottom))
      ::SetScrollPos(m_hWnd, SB_VERT,  Height(indexRow+1)-rcClient.bottom, TRUE);
   if(::GetScrollPos(m_hWnd, SB_HORZ)>(Width (indexCol  )-Width (m_iStaticCol)))
      ::SetScrollPos(m_hWnd, SB_HORZ,  Width (indexCol  )-Width (m_iStaticCol), TRUE);
   if(::GetScrollPos(m_hWnd, SB_VERT)>(Height(indexRow  )-Height(m_iStaticRow)))
      ::SetScrollPos(m_hWnd, SB_VERT,  Height(indexRow  )-Height(m_iStaticRow), TRUE);

   ::InvalidateRect(m_hWnd, NULL, FALSE);
}

////////////////////////////////////////////////////////////////////////////////
//                          
////////////////////////////////////////////////////////////////////////////////
void CTable::DrawCell(HDC hCDC, const COORD &coordCell, const RECT &rcCell) {
   ::MoveToEx(hCDC, rcCell.left , rcCell.bottom-1, NULL);
   ::LineTo  (hCDC, rcCell.left , rcCell.top);
   ::LineTo  (hCDC, rcCell.right, rcCell.top);
   RECT rectText = {rcCell.left+2+4, rcCell.top+2, rcCell.right-1, rcCell.bottom-1};
   if (m_Data[coordCell.X][coordCell.Y]->m_pImage) {
      int m = min(rectText.right-rectText.left, rectText.bottom-rectText.top);
      RECT rectImage = {rectText.left, rectText.top, rectText.left+m, rectText.top+m};
      m_Data[coordCell.X][coordCell.Y]->m_pImage->Draw(hCDC, rectImage);
      rectText.left += m+4;
   }
   if (!m_Data[coordCell.X][coordCell.Y]->m_strText.IsEmpty()) {
      COLORREF colorTextOld;
      if ((coordCell == m_CurrentCell) && m_bSelect) {
         colorTextOld = ::SetTextColor(hCDC, ::GetSysColor(COLOR_HIGHLIGHTTEXT));
      }
      ::DrawText(hCDC, m_Data[coordCell.X][coordCell.Y]->m_strText, -1, &rectText, m_Data[coordCell.X][coordCell.Y]->m_uFormatText);
      if ((coordCell == m_CurrentCell) && m_bSelect) {
         ::SetTextColor(hCDC, colorTextOld);
      }
   }
}

void CTable::DrawCellField(HDC hCDC, const COORD &coordCell, const RECT &rcCell) {
   if (coordCell == m_CurrentCell) {
      if (m_bSelect) {
         HBRUSH hBrushNew = ::GetSysColorBrush(COLOR_HIGHLIGHT);
         HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
         ::Rectangle(hCDC, rcCell.left, rcCell.top, rcCell.right+1, rcCell.bottom+1);
         ::SelectObject(hCDC, hBrushOld);
      } else {
         HPEN hPenNew = ::CreatePen(PS_DOT, 1, GetLineColor());
         HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);
         ::MoveToEx(hCDC, rcCell.left +1, rcCell.top   +1, NULL);
         ::LineTo  (hCDC, rcCell.left +1, rcCell.bottom-1);
         ::LineTo  (hCDC, rcCell.right-1, rcCell.bottom-1);
         ::LineTo  (hCDC, rcCell.right-1, rcCell.top   +1);
         ::LineTo  (hCDC, rcCell.left +1, rcCell.top   +1);
         ::SelectObject(hCDC, hPenOld);
         ::DeleteObject(hPenNew);
      }
   }
   DrawCell(hCDC, coordCell, rcCell);
}

void CTable::DrawCellCaption(HDC hCDC, const COORD &coordCell, const RECT &rcCell) {
   ::Rectangle(hCDC, rcCell.left, rcCell.top, rcCell.right+1, rcCell.bottom+1);
   {
      HGDIOBJ hPenOld = ::SelectObject(hCDC, ::GetStockObject(WHITE_PEN));
      ::MoveToEx(hCDC, rcCell.left +1, rcCell.bottom-1, NULL);
      ::LineTo  (hCDC, rcCell.left +1, rcCell.top   +1);
      ::LineTo  (hCDC, rcCell.right-1, rcCell.top   +1);
      ::SelectObject(hCDC, ::GetStockObject(BLACK_PEN));
      ::LineTo  (hCDC, rcCell.right-1, rcCell.bottom-1);
      ::LineTo  (hCDC, rcCell.left +1, rcCell.bottom-1);
      ::SelectObject(hCDC, hPenOld);
   }
   DrawCell(hCDC, coordCell, rcCell);
}

void CTable::OnPaint(HWND) {
   PAINTSTRUCT PaintStruct;
   HDC hDC = ::BeginPaint(m_hWnd, &PaintStruct); // original DC
   HDC hCDC = ::CreateCompatibleDC(hDC);
   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   HBITMAP hBmpNew = ::CreateCompatibleBitmap(hDC, rcClient.right, rcClient.bottom);
   HGDIOBJ hBmpOld = ::SelectObject(hCDC, hBmpNew);
   UINT x = ::GetScrollPos(m_hWnd, SB_HORZ);
   UINT y = ::GetScrollPos(m_hWnd, SB_VERT);
   { // 
      HBRUSH  hBrushNew = ::GetSysColorBrush(COLOR_WINDOW);
      HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
      ::PatBlt(hCDC, 0,0, rcClient.right, rcClient.bottom, PATCOPY);
      ::SelectObject(hCDC, hBrushOld);
   }
   { //    ...
      HPEN    hPenNew = ::CreatePen(PS_SOLID, 1, GetLineColor());
      HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);
      int iBkModeOld = ::SetBkMode(hCDC, TRANSPARENT); //     
      LOGFONT Font = {8,0,0,0,
                      FW_NORMAL,//FW_BOLD,//
                      0,0,0,
                      DEFAULT_CHARSET,
                      OUT_DEFAULT_PRECIS,
                      CLIP_DEFAULT_PRECIS,
                      DEFAULT_QUALITY,
                      DEFAULT_PITCH | FF_DONTCARE,
                      _T("MS Sans Serif")};// _T("Times New Roman")
      HFONT hFont = ::CreateFontIndirect(&Font);
      HGDIOBJ hFontOld = ::SelectObject(hCDC, hFont);
      { // ...  a 
         for (SHORT i=m_iStaticCol; i<m_ColW.size(); i++)
            for (SHORT j=m_iStaticRow; j<m_RowH.size(); j++) {
               COORD coordCell = {i,j};
               RECT rcCell = {Width(i)-x,Height(j)-y,Width(i+1)-x,Height(j+1)-y};
               DrawCellField(hCDC, coordCell, rcCell);
         }
      }
      { // ...  a 
         HBRUSH  hBrushNew = ::CreateSolidBrush(GetHeaderColor());
         HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
         { //  
            for (SHORT i=m_iStaticCol; i<m_ColW.size(); i++)
               for (SHORT j=0; j<min(m_iStaticRow, m_RowH.size()); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i)-x,Height(j),Width(i+1)-x,Height(j+1)};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         { //  
            for (SHORT i=0; i<min(m_iStaticCol, m_ColW.size()); i++)
               for (SHORT j=m_iStaticRow; j<m_RowH.size(); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i),Height(j)-y,Width(i+1),Height(j+1)-y};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         { //   
            for (SHORT i=0; i<min(m_iStaticCol, m_ColW.size()); i++)
               for (SHORT j=0; j<min(m_iStaticRow, m_RowH.size()); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i),Height(j),Width(i+1),Height(j+1)};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         ::SelectObject(hCDC, hBrushOld);
         ::DeleteObject(hBrushNew);
      }
      ::MoveToEx(hCDC, Width(m_ColW.size())-x, 0                   -y, NULL);
      ::LineTo  (hCDC, Width(m_ColW.size())-x, Height(m_RowH.size()-y));
      ::LineTo  (hCDC, -1                  -x, Height(m_RowH.size()-y));
      ::SelectObject(hCDC, hPenOld);
      ::DeleteObject(hPenNew);
      ::SetBkMode(hCDC, iBkModeOld);
      ::SelectObject(hCDC, hFontOld);
      ::DeleteObject(hFont);
   }
   ::BitBlt(hDC,  PaintStruct.rcPaint.left,  PaintStruct.rcPaint.top,
                  PaintStruct.rcPaint.right -PaintStruct.rcPaint.left,
                  PaintStruct.rcPaint.bottom-PaintStruct.rcPaint.top,
            hCDC, PaintStruct.rcPaint.left, PaintStruct.rcPaint.top, SRCCOPY);
   ::SelectObject(hCDC, hBmpOld);
   ::DeleteObject(hBmpNew);
   ::DeleteDC(hCDC);
   ::EndPaint(m_hWnd, &PaintStruct);
   DrawHScroll();
   DrawVScroll();
}

void CTable::OnMouseMove(HWND, int x, int y, UINT keyFlags) {
   if (!(keyFlags & MK_LBUTTON)) return;
   COORD cell = FindCell(x, y);
   if (cell == INVALID_CELL()) return;
   SetCurrentCell(cell.X, cell.Y, false);
}

void CTable::OnLButtonDown(HWND, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
   ::SetFocus(m_hWnd);
   ::SetCapture(m_hWnd);
   COORD cell = FindCell(x, y);
   if (cell == INVALID_CELL()) return;

   SetCurrentCell(cell.X, cell.Y, !!fDoubleClick);
   if (fDoubleClick)
      FORWARD_WM_COMMAND(::GetParent(m_hWnd), ::GetDlgCtrlID(m_hWnd), m_hWnd, CTable::NC_SELECT_CELL, ::PostMessage);
}

void CTable::OnLButtonUp(HWND, int x, int y, UINT keyFlags) {
   ::ReleaseCapture();
}

void CTable::OnRButtonDown(HWND, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
   ::SetFocus(m_hWnd);
}

void CTable::OnSize(HWND, UINT state, int cx, int cy) {
   RecountScrollInfo();
}

void CTable::OnHScroll(HWND, HWND hWndScrollBar, UINT nScrollCode, int nPos) {
   SCROLLINFO scInfo = {sizeof(SCROLLINFO), SIF_RANGE | SIF_PAGE | SIF_POS};
   ::GetScrollInfo(m_hWnd, SB_HORZ, &scInfo);
   switch (nScrollCode) {
   case SB_TOP          : scInfo.nPos = scInfo.nMin; break;
   case SB_BOTTOM       : scInfo.nPos = scInfo.nMax; break;
   case SB_LINELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage/10); break;
   case SB_LINERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage/10); break;
   case SB_PAGELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage); break;
   case SB_PAGERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage); break;
   case SB_THUMBPOSITION:
   case SB_THUMBTRACK   : scInfo.nPos = nPos; break;
   case SB_ENDSCROLL    : break;
   }
   scInfo.fMask = SIF_POS;
   ::SetScrollInfo(m_hWnd, SB_HORZ, &scInfo, TRUE);
   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   RECT rcInvalidate = {Width(m_iStaticCol), 0, rcClient.right, rcClient.bottom};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

void CTable::OnVScroll(HWND, HWND hWndScrollBar, UINT nScrollCode, int nPos) {
   SCROLLINFO scInfo = {sizeof(SCROLLINFO), SIF_RANGE | SIF_PAGE | SIF_POS};
   ::GetScrollInfo(m_hWnd, SB_VERT, &scInfo);
   switch (nScrollCode) {
   case SB_TOP          : scInfo.nPos = scInfo.nMin; break;
   case SB_BOTTOM       : scInfo.nPos = scInfo.nMax; break;
   case SB_LINELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage/10); break;
   case SB_LINERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage/10); break;
   case SB_PAGELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage); break;
   case SB_PAGERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage); break;
   case SB_THUMBPOSITION:
   case SB_THUMBTRACK   : scInfo.nPos = nPos; break;
   case SB_ENDSCROLL    : break;
   }
   scInfo.fMask = SIF_POS;
   ::SetScrollInfo(m_hWnd, SB_VERT, &scInfo, TRUE);
   RECT rcClient; ::GetClientRect(m_hWnd, &rcClient);
   RECT rcInvalidate = {0, Height(m_iStaticRow), rcClient.right, rcClient.bottom};
   ::InvalidateRect(m_hWnd, &rcInvalidate, FALSE);
}

BOOL CTable::OnEraseBkgnd(HWND, HDC hdc) {
   return TRUE;
}

void CTable::OnKey(HWND, UINT nVirtKey, BOOL fDown, int cRepeat, UINT flags) {
   //g_Log.Put(CLogger::LL_DEBUG, "vk = 0x%08X", nVirtKey);

   if (fDown) return;
   switch (nVirtKey) {
   case VK_LEFT  : SetCurrentCell(m_CurrentCell.X-1, m_CurrentCell.Y  , false); break;
   case VK_UP    : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y-1, false); break;
   case VK_RIGHT : SetCurrentCell(m_CurrentCell.X+1, m_CurrentCell.Y  , false); break;
   case VK_DOWN  : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y+1, false); break;
   case VK_HOME  : SetCurrentCell(m_CurrentCell.X  , m_iStaticRow     , false); break;
   case VK_END   : SetCurrentCell(m_CurrentCell.X  , m_RowH.size()  -1, false); break;
   case VK_PRIOR : SetCurrentCell(m_CurrentCell.X  , max(m_iStaticRow   , m_CurrentCell.Y-7), false); break;
   case VK_NEXT  : SetCurrentCell(m_CurrentCell.X  , min(m_RowH.size()-1, m_CurrentCell.Y+7), false); break;
   case VK_SPACE : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y, !m_bSelect); break;
   case VK_DELETE: FORWARD_WM_COMMAND(::GetParent(m_hWnd), ::GetDlgCtrlID(m_hWnd), m_hWnd, CTable::NC_DELETE_CELL, ::PostMessage); break;
   }
}

UINT CTable::OnGetDlgCode(HWND, LPMSG lpMsg) {
   //g_Log.Put(CLogger::LL_DEBUG, "msg WM_GETDLGCODE: keyState = 0x%08X", keyState);
   if (GetKeyState(VK_SPACE) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_SPACE, count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_UP   ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_UP   , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_DOWN ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_DOWN , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_LEFT ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_LEFT , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_RIGHT) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_RIGHT, count = 0, 0, SendMessage);
   }

   return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

CTable::CTable(HWND hWnd) :
   m_iDefH      (27), //   ,      
   m_iDefW      (100),
   m_CurrentCell(INVALID_CELL()),
   m_bSelect    (false),
   m_iStaticCol (0),
   m_iStaticRow (1)
{
   m_hWnd = hWnd;
}

CTable::~CTable() {
   for (SHORT i=0; i<m_ColW.size(); i++)
      for (SHORT j=0; j<m_RowH.size(); j++)
         delete m_Data[i][j];
}

int CTable::OnCreate(LPCREATESTRUCT lpCreateStruct) {
   SetRowNumber(11);
   SetColNumber(6);

   RECT rcClient; GetClientRect(&rcClient);
   SCROLLINFO scInfoHorz = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0, Width(m_ColW.size())+1,
      min(rcClient.right-rcClient.left, Width(m_ColW.size())),
      0, 0
   };
   SetScrollInfo(SB_HORZ, &scInfoHorz, TRUE);
   SCROLLINFO scInfoVert = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0, Height(m_RowH.size())+1,
      min(rcClient.bottom-rcClient.top, Height(m_RowH.size())),
      0, 0
   };
   SetScrollInfo(SB_VERT, &scInfoVert, TRUE);
   return 0;
}

void CTable::SetColNumber(SHORT nCol) {
   if (nCol < 0) return;
   SHORT nColOld = m_ColW.size();
   if (nCol == nColOld) return;

   { //   -  
      for (SHORT i=nCol; i<nColOld; i++)
         for (SHORT j=0; j<m_RowH.size(); j++) {
            delete m_Data[i][j]; m_Data[i][j] = NULL;
         }
   }
   UINT nWidthMaxOld = Width(nColOld);

   { //   
      m_ColW.resize(nCol);
      m_Data.resize(nCol);
      for (SHORT i=nColOld; i<nCol; i++) { //   
         m_ColW[i] = m_iDefW;
         m_Data[i].resize(m_RowH.size());
      }
   }
   { //   -  
      for (SHORT i=nColOld; i<nCol; i++)
         for (SHORT j=0; j<m_RowH.size(); j++)
            m_Data[i][j] = new CTableData;
   }

   if (m_CurrentCell.X > nCol-1) SetCurrentCell(nCol-1, m_CurrentCell.Y, m_bSelect);

   RecountScrollInfo();
   RECT rcInvalidate = {-GetScrollPos(SB_HORZ)+min(nWidthMaxOld, Width(m_ColW.size())),
                        -GetScrollPos(SB_VERT),
                        -GetScrollPos(SB_HORZ)+max(nWidthMaxOld, Width(m_ColW.size()))+1,
                        -GetScrollPos(SB_VERT)+Height(m_RowH.size())+1};
   InvalidateRect(&rcInvalidate, FALSE);
}

void CTable::SetRowNumber(SHORT nRow) {
   if (nRow < 0) return;
   SHORT nRowOld = m_RowH.size();
   if (nRow == nRowOld) return;

   { //   -  
      for (SHORT i=0; i<m_ColW.size(); i++)
         for (SHORT  j=nRow; j<nRowOld; j++) {
            delete m_Data[i][j]; m_Data[i][j] = NULL;
         }
   }
   UINT nHeightMaxOld = Height(nRowOld);

   { //   
      m_RowH.resize(nRow);
      for (SHORT i=0; i<m_ColW.size(); i++)
         m_Data[i].resize(nRow);
      for (SHORT j=nRowOld; j<nRow; j++)
         m_RowH[j] = m_iDefH;
   }
   { //   -  
      for (SHORT i=0; i<m_ColW.size(); i++)
         for (SHORT j=nRowOld; j<nRow; j++)
            m_Data[i][j] = new CTableData;
   }

   if (m_CurrentCell.Y > nRow-1) SetCurrentCell(m_CurrentCell.X, nRow-1, m_bSelect);

   RecountScrollInfo();
   RECT rcInvalidate = {-GetScrollPos(SB_HORZ),
                        -GetScrollPos(SB_VERT)+min(nHeightMaxOld, Height(m_RowH.size())),
                        -GetScrollPos(SB_HORZ)+Width(m_ColW.size())+1,
                        -GetScrollPos(SB_VERT)+max(nHeightMaxOld, Height(m_RowH.size()))+1};
   InvalidateRect(&rcInvalidate, FALSE);
}

bool CTable::SetColWidth(SHORT indexCol, UINT width) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size())) return false;
   if (m_ColW[indexCol] == width) return false;

   UINT nWidthMaxOld = Width(m_ColW.size());
   m_ColW[indexCol] = width;
   RecountScrollInfo();
   RECT rcInvalidate = {-GetScrollPos(SB_HORZ)+Width(indexCol),
                        -GetScrollPos(SB_VERT),
                        -GetScrollPos(SB_HORZ)+max(nWidthMaxOld, Width(m_ColW.size()))+1,
                        -GetScrollPos(SB_VERT)+Height(m_RowH.size())+1};
   InvalidateRect(&rcInvalidate, FALSE);
   return true;
}

UINT CTable::GetColWidth(SHORT indexCol) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size())) return -1;
   return m_ColW[indexCol];
}

bool CTable::SetRowHeight(SHORT indexRow, UINT height) {
   if ((indexRow < 0) || (indexRow >= m_RowH.size())) return false;
   if (m_RowH[indexRow] == height) return false;

   UINT nHeightMaxOld = Height(m_RowH.size());
   m_RowH[indexRow] = height;
   RecountScrollInfo();
   RECT rcInvalidate = {-GetScrollPos(SB_HORZ),
                        -GetScrollPos(SB_VERT)+Height(indexRow),
                        -GetScrollPos(SB_HORZ)+Width(m_ColW.size())+1,
                        -GetScrollPos(SB_VERT)+max(nHeightMaxOld, Height(m_RowH.size()))+1};
   InvalidateRect(&rcInvalidate, FALSE);
   return true;
}

UINT CTable::GetRowHeight(SHORT indexRow) {
   if ((indexRow < 0) || (indexRow >= m_RowH.size())) return -1;
   return m_RowH[indexRow];
}

UINT CTable::Width(SHORT col) const { //   col 
   UINT result = 0;
   for (SHORT i=0; i<min(m_ColW.size(), col); i++)
      result += m_ColW[i];
   return result;
}

UINT CTable::Height(SHORT row) const { //   row 
   UINT result = 0;
   for (SHORT j=0; j<min(m_RowH.size(), row); j++)
      result += m_RowH[j];
   return result;
}

void CTable::SetText(SHORT indexCol, SHORT indexRow, LPCTSTR str) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   m_Data[indexCol][indexRow]->m_strText = str;
   UINT x = (indexCol < m_iStaticCol) ? 0 : GetScrollPos(SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : GetScrollPos(SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   InvalidateRect(&rcInvalidate, FALSE);
}

void CTable::SetFormatText(SHORT indexCol, SHORT indexRow, UINT m_uFormatText) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   m_Data[indexCol][indexRow]->m_uFormatText = m_uFormatText;
   UINT x = (indexCol < m_iStaticCol) ? 0 : GetScrollPos(SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : GetScrollPos(SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   InvalidateRect(&rcInvalidate, FALSE);
}

void CTable::SetImage(SHORT indexCol, SHORT indexRow, const CImageSerialize *pImage) {
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;

#define m_pImage (m_Data[indexCol][indexRow]->m_pImage)
   if (m_pImage) {
      delete m_pImage; m_pImage = NULL;
   }
   if (pImage && !pImage->m_strPath.IsEmpty()) {
      CImage img(*pImage);
      if (img.Valid())
         m_pImage = new CImage(*pImage);
   }
#undef m_pImage
   UINT x = (indexCol < m_iStaticCol) ? 0 : GetScrollPos(SB_HORZ);
   UINT y = (indexRow < m_iStaticRow) ? 0 : GetScrollPos(SB_VERT);
   RECT rcInvalidate = {Width(indexCol  )-x, Height(indexRow  )-y,
                        Width(indexCol+1)-x, Height(indexRow+1)-y};
   InvalidateRect(&rcInvalidate, FALSE);
}

void CTable::SetCurrentCell(SHORT indexCol, SHORT indexRow, bool isSelect) {
   if ((indexCol < m_iStaticCol) || (indexCol >= m_ColW.size()) ||
       (indexRow < m_iStaticRow) || (indexRow >= m_RowH.size())) return;
   COORD newCell = {indexCol, indexRow};
   if ((m_CurrentCell == newCell) && (m_bSelect == isSelect)) return;

   RECT rcInvalidateOld = {Width(m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                           Width(m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   m_bSelect = isSelect;
   m_CurrentCell = newCell;
   RECT rcInvalidateNew = {Width (m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                           Width (m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   RECT rcInvalidate = {min(rcInvalidateOld.left  ,rcInvalidateNew.left  )-GetScrollPos(SB_HORZ),
                        min(rcInvalidateOld.top   ,rcInvalidateNew.top   )-GetScrollPos(SB_VERT),
                        max(rcInvalidateOld.right ,rcInvalidateNew.right )-GetScrollPos(SB_HORZ),
                        max(rcInvalidateOld.bottom,rcInvalidateNew.bottom)-GetScrollPos(SB_VERT)};
   InvalidateRect(&rcInvalidate, FALSE);
   SetVisibleCell(indexCol, indexRow);
}

LPCTSTR CTable::GetCurrentText() const {
   if (m_CurrentCell == INVALID_CELL()) {
      return NULL;
   }
   return m_Data[m_CurrentCell.X][m_CurrentCell.Y]->m_strText;
}

void CTable::UnselectCell() {
   if (m_CurrentCell == INVALID_CELL()) return;
   RECT rcInvalidate = {Width(m_CurrentCell.X  ), Height(m_CurrentCell.Y  ),
                        Width(m_CurrentCell.X+1), Height(m_CurrentCell.Y+1)};
   m_CurrentCell = INVALID_CELL();
   InvalidateRect(&rcInvalidate, FALSE);
}

COORD CTable::FindCell(int xWin, int yWin) const {
   UINT x = xWin+GetScrollPos(SB_HORZ);
   UINT y = yWin+GetScrollPos(SB_VERT);
   COORD resultCell = INVALID_CELL();
   UINT result = 0;
   SHORT i = 0;
   for (; i<m_ColW.size(); i++) {
      if ((x >= result) && (x < result+m_ColW[i])) {
         resultCell.X = i;
         break;
      }
      result += m_ColW[i];
   }
   result = 0;
   for (i=0; i<m_RowH.size(); i++) {
      if ((y >= result) && (y < result+m_RowH[i])) {
         resultCell.Y = i;
         break;
      }
      result += m_RowH[i];
   }
   return ((resultCell.X == INVALID_CELL().X) ||
           (resultCell.Y == INVALID_CELL().Y)) ?
           INVALID_CELL() :
           resultCell;
}

void CTable::RecountScrollInfo() {
   RECT rcClient; GetClientRect(&rcClient);
   UINT x = Width (m_ColW.size());
   UINT y = Height(m_RowH.size());
   UINT h = GetScrollPos(SB_HORZ);
   UINT v = GetScrollPos(SB_VERT);
   SCROLLINFO scInfoHorz = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0,  x-1,
      min(x,     rcClient.right),
      min(x, min(rcClient.right, h)),
   };
   SetScrollInfo(SB_HORZ, &scInfoHorz, TRUE);
   static bool oldShowSBHorz = (rcClient.right < x);
          bool newShowSBHorz = (rcClient.right < x);
   if (oldShowSBHorz != newShowSBHorz) {
      oldShowSBHorz = newShowSBHorz;
      ShowScrollBar(SB_HORZ, newShowSBHorz); //      ,   ...
      RecountScrollInfo();                   // ...     
   }
   //EnableScrollBar(SB_HORZ, (rcClient.right < x) ? ESB_ENABLE_BOTH : ESB_DISABLE_BOTH);
   SCROLLINFO scInfoVert = {
      sizeof(SCROLLINFO),
      SIF_RANGE | SIF_PAGE | SIF_POS,
      0,  y-1,
      min(y,     rcClient.bottom),
      min(y, min(rcClient.bottom, v)),
   };
   SetScrollInfo(SB_VERT, &scInfoVert, TRUE);
   static bool oldShowSBVert = (rcClient.bottom < y);
          bool newShowSBVert = (rcClient.bottom < y);
   if (oldShowSBVert != newShowSBVert) {
      oldShowSBVert = newShowSBVert;
      ShowScrollBar(SB_VERT, newShowSBVert); //      ,   ...
      RecountScrollInfo();                   // ...     
   }
   //EnableScrollBar(SB_VERT, (rcClient.bottom < y) ? ESB_ENABLE_BOTH : ESB_DISABLE_BOTH);
}

bool CTable::VisibleCell(SHORT indexCol, SHORT indexRow) const { // a  ? (  ?)
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return false; //  
   RECT rcClient; GetClientRect(&rcClient);
   rcClient.left += Width (m_iStaticCol);
   rcClient.top  += Height(m_iStaticRow);
   if ((rcClient.left   <= Width (indexCol  )-GetScrollPos(SB_HORZ)) &&
       (rcClient.right  >= Width (indexCol+1)-GetScrollPos(SB_HORZ)) &&
       (rcClient.top    <= Height(indexRow  )-GetScrollPos(SB_VERT)) &&
       (rcClient.bottom >= Height(indexRow+1)-GetScrollPos(SB_VERT)))
      return true;
   else
      return false;
}

void CTable::SetVisibleCell(SHORT indexCol, SHORT indexRow) { //    ()  
   if ((indexCol < 0) || (indexCol >= m_ColW.size()) ||
       (indexRow < 0) || (indexRow >= m_RowH.size())) return;
   if (VisibleCell(indexCol, indexRow)) {
      return; //      
   }
   RECT rcClient; GetClientRect(&rcClient);
   if(GetScrollPos(SB_HORZ)<(Width (indexCol+1)-rcClient.right))
      SetScrollPos(SB_HORZ,  Width (indexCol+1)-rcClient.right,  TRUE);
   if(GetScrollPos(SB_VERT)<(Height(indexRow+1)-rcClient.bottom))
      SetScrollPos(SB_VERT,  Height(indexRow+1)-rcClient.bottom, TRUE);
   if(GetScrollPos(SB_HORZ)>(Width (indexCol  )-Width (m_iStaticCol)))
      SetScrollPos(SB_HORZ,  Width (indexCol  )-Width (m_iStaticCol), TRUE);
   if(GetScrollPos(SB_VERT)>(Height(indexRow  )-Height(m_iStaticRow)))
      SetScrollPos(SB_VERT,  Height(indexRow  )-Height(m_iStaticRow), TRUE);

   InvalidateRect(NULL, FALSE);
}

////////////////////////////////////////////////////////////////////////////////
//                          
////////////////////////////////////////////////////////////////////////////////
void CTable::DrawCell(CDC &hCDC, const COORD &coordCell, const RECT &rcCell) {
   hCDC.MoveTo(rcCell.left , rcCell.bottom-1);
   hCDC.LineTo(rcCell.left , rcCell.top);
   hCDC.LineTo(rcCell.right, rcCell.top);
   RECT rectText = {rcCell.left+2+4, rcCell.top+2, rcCell.right-1, rcCell.bottom-1};
   if (m_Data[coordCell.X][coordCell.Y]->m_pImage) {
      int m = min(rectText.right-rectText.left, rectText.bottom-rectText.top);
      RECT rectImage = {rectText.left, rectText.top, rectText.left+m, rectText.top+m};
      m_Data[coordCell.X][coordCell.Y]->m_pImage->Draw(hCDC, rectImage);
      rectText.left += m+4;
   }
   if (!m_Data[coordCell.X][coordCell.Y]->m_strText.IsEmpty()) {
      COLORREF colorTextOld;
      if ((coordCell == m_CurrentCell) && m_bSelect) {
         colorTextOld = hCDC.SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
      }
      hCDC.DrawText(m_Data[coordCell.X][coordCell.Y]->m_strText, -1, &rectText, m_Data[coordCell.X][coordCell.Y]->m_uFormatText);
      if ((coordCell == m_CurrentCell) && m_bSelect) {
         hCDC.SetTextColor(colorTextOld);
      }
   }
}

void CTable::DrawCellField(CDC &hCDC, const COORD &coordCell, const RECT &rcCell) {
   if (coordCell == m_CurrentCell) {
      if (m_bSelect) {
         HBRUSH hBrushNew = ::GetSysColorBrush(COLOR_HIGHLIGHT);
         HBRUSH hBrushOld = hCDC.SelectBrush(hBrushNew);
         hCDC.Rectangle(rcCell.left, rcCell.top, rcCell.right+1, rcCell.bottom+1);
         hCDC.SelectBrush(hBrushOld);
      } else {
         HPEN hPenNew = ::CreatePen(PS_DOT, 1, GetLineColor());
         HPEN hPenOld = hCDC.SelectPen(hPenNew);
         hCDC.MoveTo(rcCell.left +1, rcCell.top   +1);
         hCDC.LineTo(rcCell.left +1, rcCell.bottom-1);
         hCDC.LineTo(rcCell.right-1, rcCell.bottom-1);
         hCDC.LineTo(rcCell.right-1, rcCell.top   +1);
         hCDC.LineTo(rcCell.left +1, rcCell.top   +1);
         hCDC.SelectPen(hPenOld);
         ::DeleteObject(hPenNew);
      }
   }
   DrawCell(hCDC, coordCell, rcCell);
}

void CTable::DrawCellCaption(CDC &hCDC, const COORD &coordCell, const RECT &rcCell) {
   hCDC.Rectangle(rcCell.left, rcCell.top, rcCell.right+1, rcCell.bottom+1);
   {
      HPEN hPenOld = hCDC.SelectPen((HPEN)::GetStockObject(WHITE_PEN));
      hCDC.MoveTo(rcCell.left +1, rcCell.bottom-1);
      hCDC.LineTo(rcCell.left +1, rcCell.top   +1);
      hCDC.LineTo(rcCell.right-1, rcCell.top   +1);
      hCDC.SelectPen((HPEN)::GetStockObject(BLACK_PEN));
      hCDC.LineTo(rcCell.right-1, rcCell.bottom-1);
      hCDC.LineTo(rcCell.left +1, rcCell.bottom-1);
      hCDC.SelectPen(hPenOld);
   }
   DrawCell(hCDC, coordCell, rcCell);
}

void CTable::OnPaint(HDC /*hDC*/) {
   CPaintDC hDC(m_hWnd);

   CDC hCDC; hCDC.CreateCompatibleDC(hDC);
   RECT rcClient; GetClientRect(&rcClient);
   HBITMAP hBmpNew = ::CreateCompatibleBitmap(hDC, rcClient.right, rcClient.bottom);
   HBITMAP hBmpOld = hCDC.SelectBitmap(hBmpNew);
   UINT x = GetScrollPos(SB_HORZ);
   UINT y = GetScrollPos(SB_VERT);
   { // 
      HBRUSH hBrushNew = ::GetSysColorBrush(COLOR_WINDOW);
      HBRUSH hBrushOld = hCDC.SelectBrush(hBrushNew);
      hCDC.PatBlt(0,0, rcClient.right, rcClient.bottom, PATCOPY);
      hCDC.SelectBrush(hBrushOld);
   }
   { //    ...
      HPEN hPenNew = ::CreatePen(PS_SOLID, 1, GetLineColor());
      HPEN hPenOld = hCDC.SelectPen(hPenNew);
      int iBkModeOld = hCDC.SetBkMode(TRANSPARENT); //     
      LOGFONT Font = {8,0,0,0,
                      FW_NORMAL,//FW_BOLD,//
                      0,0,0,
                      DEFAULT_CHARSET,
                      OUT_DEFAULT_PRECIS,
                      CLIP_DEFAULT_PRECIS,
                      DEFAULT_QUALITY,
                      DEFAULT_PITCH | FF_DONTCARE,
                      _T("MS Sans Serif")};// _T("Times New Roman")
      HFONT hFont = ::CreateFontIndirect(&Font);
      HFONT hFontOld = hCDC.SelectFont(hFont);
      { // ...  a 
         for (SHORT i=m_iStaticCol; i<m_ColW.size(); i++)
            for (SHORT j=m_iStaticRow; j<m_RowH.size(); j++) {
               COORD coordCell = {i,j};
               RECT rcCell = {Width(i)-x,Height(j)-y,Width(i+1)-x,Height(j+1)-y};
               DrawCellField(hCDC, coordCell, rcCell);
         }
      }
      { // ...  a 
         HBRUSH hBrushNew = ::CreateSolidBrush(GetHeaderColor());
         HBRUSH hBrushOld = hCDC.SelectBrush(hBrushNew);
         { //  
            for (SHORT i=m_iStaticCol; i<m_ColW.size(); i++)
               for (SHORT j=0; j<min(m_iStaticRow, m_RowH.size()); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i)-x,Height(j),Width(i+1)-x,Height(j+1)};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         { //  
            for (SHORT i=0; i<min(m_iStaticCol, m_ColW.size()); i++)
               for (SHORT j=m_iStaticRow; j<m_RowH.size(); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i),Height(j)-y,Width(i+1),Height(j+1)-y};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         { //   
            for (SHORT i=0; i<min(m_iStaticCol, m_ColW.size()); i++)
               for (SHORT j=0; j<min(m_iStaticRow, m_RowH.size()); j++) {
                  COORD coordCell = {i,j};
                  RECT rcCell = {Width(i),Height(j),Width(i+1),Height(j+1)};
                  DrawCellCaption(hCDC, coordCell, rcCell);
               }
         }
         hCDC.SelectBrush(hBrushOld);
         ::DeleteObject(hBrushNew);
      }
      hCDC.MoveTo(Width(m_ColW.size())-x, 0                    -y);
      hCDC.LineTo(Width(m_ColW.size())-x, Height(m_RowH.size())-y);
      hCDC.LineTo(-1                  -x, Height(m_RowH.size())-y);
      hCDC.SelectPen(hPenOld);
      ::DeleteObject(hPenNew);
      hCDC.SetBkMode(iBkModeOld);
      hCDC.SelectFont(hFontOld);
      ::DeleteObject(hFont);
   }
   hDC.BitBlt(hDC.m_ps.rcPaint.left,  hDC.m_ps.rcPaint.top,
              hDC.m_ps.rcPaint.right -hDC.m_ps.rcPaint.left,
              hDC.m_ps.rcPaint.bottom-hDC.m_ps.rcPaint.top,
              hCDC, hDC.m_ps.rcPaint.left,  hDC.m_ps.rcPaint.top, SRCCOPY);
   hCDC.SelectBitmap(hBmpOld);
   ::DeleteObject(hBmpNew);
 //::DeleteDC(hCDC);
 //::EndPaint(hWnd, &hDC.m_ps);
   DrawHScroll();
   DrawVScroll();
}

void CTable::OnMouseMove(UINT keyFlags, const CPoint & pointCursor) {
   if (!(keyFlags & MK_LBUTTON)) return;
   COORD cell = FindCell(pointCursor.x, pointCursor.y);
   if (cell == INVALID_CELL()) return;
   SetCurrentCell(cell.X, cell.X, false);
}

void CTable::OnLButtonDown(UINT keyFlags, const CPoint &pointCursor) {
   SetFocus();
   SetCapture();
   COORD cell = FindCell(pointCursor.x, pointCursor.y);
   if (cell == INVALID_CELL()) return;
   bool fDoubleClick = (GetCurrentMessage()->message == WM_LBUTTONDBLCLK);
   SetCurrentCell(cell.X, cell.Y, fDoubleClick);
   if (fDoubleClick)
      FORWARD_WM_COMMAND(GetParent(), GetDlgCtrlID(), m_hWnd, CTable::NC_SELECT_CELL, ::PostMessage);
}

void CTable::OnLButtonUp(UINT keyFlags, const CPoint &pointCursor) {
   ::ReleaseCapture();
}

void CTable::OnRButtonDown(UINT keyFlags, const CPoint &pointCursor) {
   SetFocus();
}

void CTable::OnSize(UINT fwSizeType, const CSize &sizeNewClientArea) {
   RecountScrollInfo();
}

void CTable::OnHScroll(int nScrollCode, short nPos, HWND hWndScrollBar) {
   SCROLLINFO scInfo = {sizeof(SCROLLINFO), SIF_RANGE | SIF_PAGE | SIF_POS};
   GetScrollInfo(SB_HORZ, &scInfo);
   switch (nScrollCode) {
   case SB_TOP          : scInfo.nPos = scInfo.nMin; break;
   case SB_BOTTOM       : scInfo.nPos = scInfo.nMax; break;
   case SB_LINELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage/10); break;
   case SB_LINERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage/10); break;
   case SB_PAGELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage); break;
   case SB_PAGERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage); break;
   case SB_THUMBPOSITION:
   case SB_THUMBTRACK   : scInfo.nPos = nPos; break;
   case SB_ENDSCROLL    : break;
   }
   scInfo.fMask = SIF_POS;
   SetScrollInfo(SB_HORZ, &scInfo, TRUE);
   RECT rcClient; GetClientRect(&rcClient);
   RECT rcInvalidate = {Width(m_iStaticCol), 0, rcClient.right, rcClient.bottom};
   InvalidateRect(&rcInvalidate, FALSE);
}

void CTable::OnVScroll(int nScrollCode, short nPos, HWND hWndScrollBar) {
   SCROLLINFO scInfo = {sizeof(SCROLLINFO), SIF_RANGE | SIF_PAGE | SIF_POS};
   GetScrollInfo(SB_VERT, &scInfo);
   switch (nScrollCode) {
   case SB_TOP          : scInfo.nPos = scInfo.nMin; break;
   case SB_BOTTOM       : scInfo.nPos = scInfo.nMax; break;
   case SB_LINELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage/10); break;
   case SB_LINERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage/10); break;
   case SB_PAGELEFT     : scInfo.nPos = max(scInfo.nMin, scInfo.nPos-scInfo.nPage); break;
   case SB_PAGERIGHT    : scInfo.nPos = min(scInfo.nMax, scInfo.nPos+scInfo.nPage); break;
   case SB_THUMBPOSITION:
   case SB_THUMBTRACK   : scInfo.nPos = nPos; break;
   case SB_ENDSCROLL    : break;
   }
   scInfo.fMask = SIF_POS;
   SetScrollInfo(SB_VERT, &scInfo, TRUE);
   RECT rcClient; GetClientRect(&rcClient);
   RECT rcInvalidate = {0, Height(m_iStaticRow), rcClient.right, rcClient.bottom};
   InvalidateRect(&rcInvalidate, FALSE);
}

BOOL CTable::OnEraseBkgnd(HDC hDC) {
   return TRUE;
}

void CTable::OnKey(TCHAR nVirtKey, UINT cRepeat, UINT flags) {
   //g_Log.Put(CLogger::LL_DEBUG, "vk = 0x%08X", nVirtKey);
   //BOOL fDown = (GetCurrentMessage()->message == WM_KEYDOWN);
   //if (fDown) return;
   switch (nVirtKey) {
   case VK_LEFT  : SetCurrentCell(m_CurrentCell.X-1, m_CurrentCell.Y  , false); break;
   case VK_UP    : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y-1, false); break;
   case VK_RIGHT : SetCurrentCell(m_CurrentCell.X+1, m_CurrentCell.Y  , false); break;
   case VK_DOWN  : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y+1, false); break;
   case VK_HOME  : SetCurrentCell(m_CurrentCell.X  , m_iStaticRow     , false); break;
   case VK_END   : SetCurrentCell(m_CurrentCell.X  , m_RowH.size()  -1, false); break;
   case VK_PRIOR : SetCurrentCell(m_CurrentCell.X  , max(m_iStaticRow   , m_CurrentCell.Y-7), false); break;
   case VK_NEXT  : SetCurrentCell(m_CurrentCell.X  , min(m_RowH.size()-1, m_CurrentCell.Y+7), false); break;
   case VK_SPACE : SetCurrentCell(m_CurrentCell.X  , m_CurrentCell.Y, !m_bSelect); break;
   case VK_DELETE: FORWARD_WM_COMMAND(GetParent(), GetDlgCtrlID(), m_hWnd, CTable::NC_DELETE_CELL, ::PostMessage); break;
   }
}

LRESULT CTable::OnGetDlgCode(LPMSG lpMsg) {
   //g_Log.Put(CLogger::LL_DEBUG, "msg WM_GETDLGCODE: keyState = 0x%08X", keyState);
   if (GetKeyState(VK_SPACE) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_SPACE, count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_UP   ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_UP   , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_DOWN ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_DOWN , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_LEFT ) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_LEFT , count = 0, 0, SendMessage);
   }
   if (GetKeyState(VK_RIGHT) & 0x80000000) {
      static int count = 0;
      if (++count == 10) FORWARD_WM_KEYUP(m_hWnd, VK_RIGHT, count = 0, 0, SendMessage);
   }
   SetMsgHandled(FALSE);
   return 0;
}

#endif // _WTL_VER
