////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Table.h"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__CTABLE__
#define __FILE__CTABLE__

#include <vector>
#include "../Image.h"

#ifndef _WTL_VER

#ifdef _INC_WINDOWSX
   #undef SubclassWindow // protect template members from windowsx.h macros
#endif //_INC_WINDOWSX

class CTable {
public:
   enum  { // notification code
      NC_SELECT_CELL = 0,
      NC_DELETE_CELL = 1
   };
private:
   struct CTableData {
      CString       m_strText;
      UINT          m_uFormatText;
      CImage       *m_pImage;
      CTableData(): m_pImage(NULL), m_uFormatText(DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX) {};
     ~CTableData() { if (m_pImage) delete m_pImage; m_pImage = NULL; }
   };
   static const COORD& INVALID_CELL() { static COORD cellInvalid = {-1,-1}; return cellInvalid; };

   UINT  m_iDefW; // default width  cell
   UINT  m_iDefH; // default height cell
   SHORT m_iStaticCol; // -  
   SHORT m_iStaticRow; // -  
   COORD m_CurrentCell;  //  ( )   (   ) COORD.Y - 
   bool  m_bSelect; //    
   std::vector<UINT> m_ColW; //   
   std::vector<UINT> m_RowH; //   
   std::vector<std::vector<CTableData*> > m_Data; // [][]

public:
   static LPCTSTR GetWindowClass() {
      const static TCHAR szWndClass[] = _T("CTable_KSerg");
      return szWndClass;
   }
   HWND m_hWnd;
   CTable(HWND hWnd = NULL);
  ~CTable();
   BOOL SubclassWindow(HWND);
   HWND Create(HWND hWndParent, RECT &rcPos, LPCTSTR szWindowName = NULL,
			DWORD dwStyle = 0, DWORD dwExStyle = 0, UINT nID = 0);

private:
   static LRESULT CALLBACK WndProcThis(HWND hWnd, UINT, WPARAM, LPARAM);
   LRESULT WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      switch(uMsg){
      HANDLE_MSG(m_hWnd, WM_CREATE       , OnCreate);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND   , OnEraseBkgnd);
      HANDLE_MSG(m_hWnd, WM_PAINT        , OnPaint);
      HANDLE_MSG(m_hWnd, WM_MOUSEMOVE    , OnMouseMove);
      HANDLE_MSG(m_hWnd, WM_LBUTTONDOWN  , OnLButtonDown);
      HANDLE_MSG(m_hWnd, WM_LBUTTONDBLCLK, OnLButtonDown);
      HANDLE_MSG(m_hWnd, WM_RBUTTONDOWN  , OnRButtonDown);
      HANDLE_MSG(m_hWnd, WM_LBUTTONUP    , OnLButtonUp);
      HANDLE_MSG(m_hWnd, WM_SIZE         , OnSize);
      HANDLE_MSG(m_hWnd, WM_HSCROLL      , OnHScroll);
      HANDLE_MSG(m_hWnd, WM_VSCROLL      , OnVScroll);
      HANDLE_MSG(m_hWnd, WM_KEYUP        , OnKey);
      case WM_GETDLGCODE: OnGetDlgCode(m_hWnd, (LPMSG)lParam); break;
      case WM_DESTROY: m_hWnd = NULL;
      }
      return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
   }
   BOOL OnCreate     (HWND, LPCREATESTRUCT);
   void OnPaint      (HWND);
   void OnLButtonDown(HWND, BOOL, int, int, UINT);
   void OnRButtonDown(HWND, BOOL, int, int, UINT);
   void OnLButtonUp  (HWND, int, int, UINT);
   void OnMouseMove  (HWND, int, int, UINT);
   void OnSize       (HWND, UINT, int, int);
   void OnHScroll    (HWND, HWND, UINT, int);
   void OnVScroll    (HWND, HWND, UINT, int);
   BOOL OnEraseBkgnd (HWND, HDC);
   void OnKey        (HWND, UINT, BOOL, int, UINT);
   UINT OnGetDlgCode (HWND, LPMSG);

   COORD FindCell(int xWin, int yWin) const; //      
   virtual BOOL DrawHScroll() const { return FALSE; }
   virtual BOOL DrawVScroll() const { return FALSE; }
   void DrawCell       (HDC hCDC, const COORD &coordCell, const RECT &rcCell);
   void DrawCellField  (HDC hCDC, const COORD &coordCell, const RECT &rcCell);
   void DrawCellCaption(HDC hCDC, const COORD &coordCell, const RECT &rcCell);
   void RecountScrollInfo();
   void SetVisibleCell(SHORT indexCol, SHORT indexRow); //    ()  
   bool    VisibleCell(SHORT indexCol, SHORT indexRow) const; // a  ? (  ?)
protected:
   virtual COLORREF GetLineColor  () const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetHeaderColor() const { return ::GetSysColor(COLOR_BTNFACE); }

public:
   UINT Width (SHORT col) const; //   col 
   UINT Height(SHORT row) const; //   row 
   SHORT GetColNumber() const {return m_ColW.size();}; //   
   SHORT GetRowNumber() const {return m_RowH.size();}; //   
   void SetColNumber(SHORT nCol); //    
   void SetRowNumber(SHORT nRow); //    
   bool SetColWidth (SHORT indexCol, UINT width);  //    
   UINT GetColWidth (SHORT indexCol);              //        
   bool SetRowHeight(SHORT indexRow, UINT height); //    
   UINT GetRowHeight(SHORT indexRow);              //        
   void  SetText       (SHORT indexCol, SHORT indexRow, LPCTSTR);
   void  SetFormatText (SHORT indexCol, SHORT indexRow, UINT uFormatText);
   void  SetImage      (SHORT indexCol, SHORT indexRow, const CImageSerialize*);
   void  SetCurrentCell(SHORT indexCol, SHORT indexRow, bool isSelect);
   COORD GetCurrentCell() const {return m_CurrentCell;};
   LPCTSTR GetCurrentText() const;
   UINT  GetDefaultWidth () const {return m_iDefW;};
   UINT  GetDefaultHeight() const {return m_iDefH;};
   void SetDefaultWidth (UINT newDefW) {m_iDefW = newDefW;};
   void SetDefaultHeight(UINT newDefH) {m_iDefH = newDefH;};
   void UnselectCell();
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CTable : public CWindowImpl<CTable, CWindow,
                                  CWinTraits<WS_GROUP | WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL | WS_HSCROLL, WS_EX_CLIENTEDGE>//CControlWinTraits
                                 >
{
public:
   enum  { // notification code
      NC_SELECT_CELL = 0,
      NC_DELETE_CELL = 1
   };
private:
   struct CTableData {
      CString       m_strText;
      UINT          m_uFormatText;
      CImage       *m_pImage;
      CTableData(): m_pImage(NULL), m_uFormatText(DT_CENTER | DT_VCENTER | DT_SINGLELINE) {};
     ~CTableData() { if (m_pImage) delete m_pImage; m_pImage = NULL; }
   };
   static const COORD& INVALID_CELL() { static COORD cellInvalid = {-1,-1}; return cellInvalid; };

   UINT  m_iDefW; // default width  cell
   UINT  m_iDefH; // default height cell
   SHORT m_iStaticCol; // -  
   SHORT m_iStaticRow; // -  
   COORD m_CurrentCell;  //  ( )   (   ) COORD.Y - 
   bool  m_bSelect; //    
   std::vector<UINT> m_ColW; //   
   std::vector<UINT> m_RowH; //   
   std::vector<std::vector<CTableData*> > m_Data; // [][]

public:
   DECLARE_WND_CLASS_EX(_T("CTable_KSerg"), CS_DBLCLKS, ::GetSysColorBrush(COLOR_BTNFACE)-1)
   CTable(HWND hWnd = NULL);
  ~CTable();

private:
   BEGIN_MSG_MAP_EX(CGradientCursor)
      MSG_WM_CREATE       (OnCreate)
      MSG_WM_PAINT        (OnPaint)
      MSG_WM_MOUSEMOVE    (OnMouseMove)
      MSG_WM_LBUTTONDOWN  (OnLButtonDown)
      MSG_WM_LBUTTONDBLCLK(OnLButtonDown)
      MSG_WM_LBUTTONUP    (OnLButtonUp)
      MSG_WM_RBUTTONDOWN  (OnRButtonDown)
    //MSG_WM_RBUTTONDBLCLK(OnRButtonDown)
      MSG_WM_SIZE         (OnSize)
      MSG_WM_HSCROLL      (OnHScroll)
      MSG_WM_VSCROLL      (OnVScroll)
      MSG_WM_ERASEBKGND   (OnEraseBkgnd)
      MSG_WM_KEYUP        (OnKey)
      MSG_WM_GETDLGCODE   (OnGetDlgCode)
   END_MSG_MAP()

   int  OnCreate(LPCREATESTRUCT);
   void OnPaint(HDC);
   void OnLButtonDown(UINT, const CPoint&);
   void OnRButtonDown(UINT, const CPoint&);
   void OnLButtonUp  (UINT, const CPoint&);
   void OnMouseMove  (UINT, const CPoint&);
   void OnSize       (UINT, const CSize&);
   void OnHScroll(int, short, HWND);
   void OnVScroll(int, short, HWND);
   BOOL OnEraseBkgnd(HDC);
   void OnKey(TCHAR, UINT, UINT);
   LRESULT OnGetDlgCode(LPMSG lpMsg);

   COORD FindCell(int xWin, int yWin) const; //      
   virtual BOOL DrawHScroll() const { return FALSE; }
   virtual BOOL DrawVScroll() const { return FALSE; }
   void DrawCell       (CDC &hCDC, const COORD &coordCell, const RECT &rcCell);
   void DrawCellField  (CDC &hCDC, const COORD &coordCell, const RECT &rcCell);
   void DrawCellCaption(CDC &hCDC, const COORD &coordCell, const RECT &rcCell);
   void RecountScrollInfo();
   void SetVisibleCell(SHORT indexCol, SHORT indexRow); //    ()  
   bool    VisibleCell(SHORT indexCol, SHORT indexRow) const; // a  ? (  ?)
protected:
   virtual COLORREF GetLineColor  () const { return ::GetSysColor(COLOR_BTNFACE); }
   virtual COLORREF GetHeaderColor() const { return ::GetSysColor(COLOR_BTNFACE); }

public:
   UINT Width (SHORT col) const; //   col 
   UINT Height(SHORT row) const; //   row 
   SHORT GetColNumber() const {return m_ColW.size();}; //   
   SHORT GetRowNumber() const {return m_RowH.size();}; //   
   void SetColNumber(SHORT nCol); //    
   void SetRowNumber(SHORT nRow); //    
   bool SetColWidth (SHORT indexCol, UINT width);  //    
   UINT GetColWidth (SHORT indexCol);              //        
   bool SetRowHeight(SHORT indexRow, UINT height); //    
   UINT GetRowHeight(SHORT indexRow);              //        
   void  SetText       (SHORT indexCol, SHORT indexRow, LPCTSTR);
   void  SetFormatText (SHORT indexCol, SHORT indexRow, UINT uFormatText);
   void  SetImage      (SHORT indexCol, SHORT indexRow, const CImageSerialize*);
   void  SetCurrentCell(SHORT indexCol, SHORT indexRow, bool isSelect);
   COORD GetCurrentCell() const {return m_CurrentCell;};
   LPCTSTR GetCurrentText() const;
   UINT  GetDefaultWidth () const {return m_iDefW;};
   UINT  GetDefaultHeight() const {return m_iDefH;};
   void SetDefaultWidth (UINT newDefW) {m_iDefW = newDefW;};
   void SetDefaultHeight(UINT newDefH) {m_iDefH = newDefH;};
   void UnselectCell();
};

#endif // _WTL_VER

#endif // __FILE__CTABLE__
