////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "InputText.h"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__DIALOG_INPUT_TEXT__
#define __FILE__DIALOG_INPUT_TEXT__

#if !defined(__AFX_H__) && !defined(_WTL_USE_CSTRING)
   #include "../Lib/CStringKS.h"
#endif
#include "../Control/ButtonFM.h"
#include "../Control/EditFM.h"
#include "../Resource.h"

#ifndef _WTL_VER

class CDialogInputText 
{
public:
   enum EOperation {
      eSkinName,     //       
      ePlayerSelect,
      ePlayerNew,
      ePlayerNewPassword,
      ePlayerRename,
      //ePlayerRemove
   };
   HWND            m_hWnd;
   CString         m_strName;
   CString         m_strPassword;
private:
   CButtonExFM     m_BtnOk;
   CEditGradientFM m_EditName, m_EditPassword, m_EditConfirm;
   EOperation      m_eOperation;
   bool            m_bModal;
private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("InputText"); return szResourceName; }

public:
   CDialogInputText() : m_hWnd(NULL) {}
   CDialogInputText(EOperation eOperation, LPCTSTR szName = NULL);
   // modal dialogs
   HWND Create(HWND hWndParent);
	// modeless dialogs
   int DoModal(HWND);
private:
   static INT_PTR CALLBACK DialogProcThis(HWND, UINT, WPARAM, LPARAM);
   INT_PTR DialogProc(HWND m_hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
      switch (msg){
      HANDLE_MSG(m_hWnd, WM_INITDIALOG, OnInitDialog);
      HANDLE_MSG(m_hWnd, WM_COMMAND   , OnCommand);
      HANDLE_MSG(m_hWnd, WM_CLOSE     , OnClose);
      HANDLE_MSG(m_hWnd, WM_DESTROY   , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND, OnEraseBkgnd);
      }
      return FALSE;
   }

   BOOL OnInitDialog(HWND, HWND, LPARAM);
   void OnCommand   (HWND, int, HWND, UINT);
   void OnClose     (HWND, int idControl = 0);
   void OnDestroy   (HWND);
   BOOL OnEraseBkgnd(HWND, HDC);
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CDialogInputText : public CDialogImpl<CDialogInputText>, public CWinDataExchange<CDialogInputText>
{
public:
   enum EOperation {
      eSkinName,     //       
      ePlayerSelect,
      ePlayerNew,
      ePlayerNewPassword,
      ePlayerRename,
      //ePlayerRemove
   };

   CString         m_strName;
   CString         m_strPassword, m_strPasswordConfirm;
public:
//private:
   CButtonExFM     m_BtnOk;
   CEditGradientFM m_EditName, m_EditPassword, m_EditConfirm;
   EOperation      m_eOperation;
#ifndef _DEBUG
	bool            m_bModal;
#endif //_DEBUG
public:
//private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("InputText"); return szResourceName; }//enum { IDD = 0 };

public:
   CDialogInputText(EOperation eOperation, LPCTSTR szName = NULL);
   // modal dialogs
   int DoModal(HWND hWndParent = ::GetActiveWindow(), LPARAM dwInitParam = NULL);
	// modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL);
public:
//private:
   BEGIN_MSG_MAP_EX(CDialogInputText)
      COMMAND_HANDLER_EX(IDC_INPUTTEXT_EDIT_TEXT, EN_CHANGE , OnChangeLogin)
      COMMAND_HANDLER_EX(IDOK                   , BN_CLICKED, OnClickOk)
      COMMAND_HANDLER_EX(IDCANCEL               , BN_CLICKED, OnClickCancel)
      MSG_WM_INITDIALOG(OnInitDialog)
      MSG_WM_CLOSE     (OnClose)
      MSG_WM_DESTROY   (OnDestroy)
      MSG_WM_ERASEBKGND(OnEraseBkgnd)
   END_MSG_MAP()
	BEGIN_DDX_MAP(CDialogInputText)
		DDX_CONTROL(IDOK                           , m_BtnOk)
      DDX_CONTROL(IDC_INPUTTEXT_EDIT_TEXT        , m_EditName);
      DDX_CONTROL(IDC_INPUTTEXT_EDIT_PASSWORD    , m_EditPassword);
      DDX_CONTROL(IDC_INPUTTEXT_EDIT_CONFIRMATION, m_EditConfirm);
      DDX_TEXT   (IDC_INPUTTEXT_EDIT_TEXT        , m_strName);
      DDX_TEXT   (IDC_INPUTTEXT_EDIT_PASSWORD    , m_strPassword);
      DDX_TEXT   (IDC_INPUTTEXT_EDIT_CONFIRMATION, m_strPasswordConfirm);
	END_DDX_MAP()

   BOOL OnInitDialog(HWND, LPARAM);
   void OnChangeLogin(UINT, int, HWND);
   void OnClickOk    (UINT, int, HWND);
   void OnClickCancel(UINT, int, HWND);
   void OnClose(int idControl = 0);
   void OnDestroy();
   BOOL OnEraseBkgnd(HDC);
};
#endif // _WTL_VER

#endif // __FILE__DIALOG_INPUT_TEXT__
