////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "SelectMosaic.h"
//
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__SELECTMOSAIC__
#define __FILE__SELECTMOSAIC__

#include "../Mosaic.h"
#include "../Control/ButtonFM.h"
#include "../Control/EditFM.h"
#include "../Resource.h"

#ifndef _WTL_VER

class CDialogSelectMosaic {
private:
   static EMosaic NumberMosaicToNameMosaic(int);
   static int     NameMosaicToNumberMosaic(EMosaic);
public:
   static int GetGroup(EMosaic);

public:
   HWND            m_hWnd;
private:
   CButtonExFM     m_BtnOk;
   CEditGradientFM m_Edit;
   int             m_iFirstKey;
   int             m_iNumberMosaic;
   EMosaic         m_eMosaic;
private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("SelectMosaic"); return szResourceName; }
   static INT_PTR CALLBACK DialogProcThis(HWND, UINT, WPARAM, LPARAM);
   INT_PTR DialogProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      switch (uMsg) {
      HANDLE_MSG(m_hWnd, WM_INITDIALOG, OnInitDialog);
      HANDLE_MSG(m_hWnd, WM_COMMAND   , OnCommand);
      HANDLE_MSG(m_hWnd, WM_CLOSE     , OnClose);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND, OnEraseBkgnd);
      }
      return FALSE;
   }
public:
   CDialogSelectMosaic() : m_hWnd(NULL) {}
   HWND Create(HWND hWndParent);
   void SetFirstKey(int);

private:
   BOOL OnInitDialog(HWND, HWND, LPARAM);
   void OnCommand   (HWND, int, HWND, UINT);
   void OnClose     (HWND);
   void OnDestroy   (HWND);
   BOOL OnEraseBkgnd(HWND, HDC);
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CDialogSelectMosaic : public CDialogImpl<CDialogSelectMosaic>, public CWinDataExchange<CDialogSelectMosaic>
{
private:
   class CComboBoxImpl: public CWindowImpl<CComboBoxImpl, CComboBox  > { DECLARE_EMPTY_MSG_MAP() };
private:
   static EMosaic NumberMosaicToNameMosaic(int);
   static int     NameMosaicToNumberMosaic(EMosaic);
public:
   static int GetGroup(EMosaic);

private:
   CComboBoxImpl   m_ComboBox;
   CButtonExFM     m_BtnOk;
   CEditGradientFM m_Edit;
   int             m_iFirstKey;
   int             m_iNumberMosaic;
   EMosaic         m_eMosaic;
public:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("SelectMosaic"); return szResourceName; }//enum { IDD = 0 };

	// modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL);
   void SetFirstKey(int);

private:
   BEGIN_MSG_MAP_EX(CDialogSelectMosaic)
      COMMAND_HANDLER_EX(IDC_SELECTMOSAIC_COMBOBOX, CBN_SELCHANGE, OnChangeCombo)
      COMMAND_HANDLER_EX(IDC_SELECTMOSAIC_EDIT    , EN_CHANGE    , OnChangeEdit)
      COMMAND_HANDLER_EX(IDOK                     , BN_CLICKED   , OnClickOkCancel)
      COMMAND_HANDLER_EX(IDCANCEL                 , BN_CLICKED   , OnClickOkCancel)
      MSG_WM_INITDIALOG(OnInitDialog)
      MSG_WM_CLOSE     (OnClose)
      MSG_WM_DESTROY   (OnDestroy)
      MSG_WM_ERASEBKGND(OnEraseBkgnd)
   END_MSG_MAP()
	BEGIN_DDX_MAP(CDialogSelectMosaic)
		DDX_CONTROL(IDOK                     , m_BtnOk)
		DDX_CONTROL(IDC_SELECTMOSAIC_EDIT    , m_Edit)
		DDX_CONTROL(IDC_SELECTMOSAIC_COMBOBOX, m_ComboBox)
	END_DDX_MAP()

   BOOL OnInitDialog(HWND, LPARAM);
   void OnChangeCombo  (UINT, int, HWND);
   void OnChangeEdit   (UINT, int, HWND);
   void OnClickOkCancel(UINT, int, HWND);
   void OnClose();
   void OnDestroy();
   BOOL OnEraseBkgnd(HDC hDC);
};

#endif // _WTL_VER

#endif // __FILE__SELECTMOSAIC__
