////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "TemplateSelectImage.cpp" -     
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TemplateSelectImage.h"
#include <Dlgs.h>
#include <CDErr.h>
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "../Lang.h"
#include "./DialogMessageHook.h"

////////////////////////////////////////////////////////////////////////////////
//                             global variables
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

  WNDPROC_BUTTON(IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE   , GetFM2WinMain().GetSkin())
//WNDPROC_STATIC(IDC_TEMPLATESELECTIMAGE_INFO_TEXT      , GetFM2WinMain().GetSkin())
  WNDPROC_BUTTON(IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE, GetFM2WinMain().GetSkin())
  WNDPROC_STATIC(IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE  , GetFM2WinMain().GetSkin())
//WNDPROC_STATIC(stc32     , GetFM2WinMain().GetSkin()) // invisible; static   

  WNDPROC_ONLY_ERASEBKGND(DefaultSelectFile, GetFM2WinMain().GetSkin())

  WNDPROC_STATIC(stc4      , GetFM2WinMain().GetSkin()) // static 'Folder'
  WNDPROC_STATIC(cmb2      , GetFM2WinMain().GetSkin()) // combobox 'Folder'
//WNDPROC_STATIC(stc1      , GetFM2WinMain().GetSkin()) // invisible; static ???
  WNDPROC_STATIC(stc1      , GetFM2WinMain().GetSkin()) // toolbar 'Select folder, new folder, etc..'
  WNDPROC_STATIC(ctl1      , GetFM2WinMain().GetSkin()) // toolbar 'MyDoc, Desktop, MyComp, etc..'
//WNDPROC_STATIC(lst1      , GetFM2WinMain().GetSkin()) // invisible; ???
  WNDPROC_STATIC(lst2      , GetFM2WinMain().GetSkin()) //   , 
  WNDPROC_STATIC(stc3      , GetFM2WinMain().GetSkin()) // static 'File name'
  WNDPROC_STATIC(cmb13     , GetFM2WinMain().GetSkin()) // combobox 'File name'
  WNDPROC_STATIC(stc2      , GetFM2WinMain().GetSkin()) // static 'File type'
  WNDPROC_STATIC(cmb1      , GetFM2WinMain().GetSkin()) // combobox 'File type'
//WNDPROC_STATIC(chx1      , GetFM2WinMain().GetSkin()) // checkbox ' '
//WNDPROC_STATIC(IDOK      , GetFM2WinMain().GetSkin()) // see m_BtnOk
//WNDPROC_STATIC(IDCANSEL  , GetFM2WinMain().GetSkin()) // see m_BtnCancel
//WNDPROC_STATIC(pshHelp   , GetFM2WinMain().GetSkin()) // invisible;  button Help
  WNDPROC_STATIC(0xFFFFFFFF, GetFM2WinMain().GetSkin()) // scroollbar,     (ID == -1)

  ////////////////////////////////////////////////////////////////////////////////
//                          help functions
////////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK FuncEnumChildControl(HWND hWnd, LPARAM lParam) {
   // class [Static], idControl=0x0440
   // class [ToolbarWindow32], idControl=0x0440
   if (::GetDlgCtrlID(hWnd) == stc1)
      if (::GetClassName(hWnd).CompareNoCase(_T("ToolbarWindow32")) == 0)
         if (lParam) {
            *(HWND*)lParam = hWnd;
            return FALSE;
         }
   return TRUE;
}

HWND FindToolbarFolder(HWND hDlg) {
   //         ID - -    .
   //  .. GetDlgItem  ,       ...
   HWND hToolbar = NULL;
   ::EnumChildWindows(hDlg, FuncEnumChildControl, (LPARAM)&hToolbar);
   return hToolbar;
}

#ifdef _DEBUG
CString CDNotify(UINT code) {
   CString strRes;
   switch (code) {
   case CDN_INITDONE      : strRes = _T("CDN_INITDONE"      ); break;
   case CDN_SELCHANGE     : strRes = _T("CDN_SELCHANGE"     ); break;
   case CDN_FOLDERCHANGE  : strRes = _T("CDN_FOLDERCHANGE"  ); break;
   case CDN_SHAREVIOLATION: strRes = _T("CDN_SHAREVIOLATION"); break;
   case CDN_HELP          : strRes = _T("CDN_HELP"          ); break;
   case CDN_FILEOK        : strRes = _T("CDN_FILEOK"        ); break;
   case CDN_TYPECHANGE    : strRes = _T("CDN_TYPECHANGE"    ); break;
   case CDN_INCLUDEITEM   : strRes = _T("CDN_INCLUDEITEM"   ); break;
   default: strRes.Format(_T("0x%08X"), code);
   }
   return strRes;
}
#endif _DEBUG

////////////////////////////////////////////////////////////////////////////////
//                          implementation CDialogTemplateSelectImage
////////////////////////////////////////////////////////////////////////////////

#ifndef _WTL_VER

CDialogTemplateSelectImage::CDialogTemplateSelectImage(HWND hWndParent, LPCTSTR szFile, LPCTSTR szInitialDir, LPCTSTR szTitle, LPCTSTR szDefExt) :
   m_hWnd         (NULL),
   m_strFile      (szFile),
   m_strInitialDir(szInitialDir),
   m_strTitle     (szTitle),
   m_strDefExt    (szDefExt),
   m_iClickSubItem(-1)
{
   memset(&m_ofn, 0, sizeof(m_ofn)); // initialize structure to 0/NULL

   m_ofn.lStructSize = sizeof(m_ofn);
   m_ofn.hwndOwner = hWndParent;
   m_ofn.hInstance = ::GetModuleHandle(NULL);
   m_ofn.lpstrFilter = NULL; // see DoModal...
   m_ofn.lpstrFile = NULL; // see DoModal...
   m_ofn.nMaxFile = 0; // see DoModal...
   m_ofn.lpstrInitialDir = m_strInitialDir.IsEmpty() ? NULL : (LPCTSTR)m_strInitialDir;
   m_ofn.lpstrTitle      = m_strTitle     .IsEmpty() ? NULL : (LPCTSTR)m_strTitle;
   m_ofn.lpstrDefExt     = m_strDefExt    .IsEmpty() ? NULL : (LPCTSTR)m_strDefExt;
   m_ofn.Flags           = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_PATHMUSTEXIST | OFN_ENABLESIZING | OFN_ENABLEHOOK | OFN_ENABLETEMPLATE;
   m_ofn.lCustData       = (LPARAM)this;
   m_ofn.lpfnHook        = (LPOFNHOOKPROC)CDialogTemplateSelectImage::DialogProcThis;
   m_ofn.lpTemplateName  = CDialogTemplateSelectImage::IDD();

   m_Img.Load(szFile ? szFile : szInitialDir);
}

INT_PTR CDialogTemplateSelectImage::DoModal(HWND hWndParent) {
   if(m_ofn.hwndOwner == NULL)
      m_ofn.hwndOwner = hWndParent;

   TCHAR szFilter2[] = _T("\0*.bmp; *.ico; *.emf; *.wmf; *.ani; *.gif; *.jpg; *.jpeg; *.pcx; *.tif; *.exe; *.dll")
                       _T("\0BMP - Windows Bitmap (*.bmp)\0*.bmp")
                       _T("\0ICO - Icon (*.ico)\0*.ico")
                       _T("\0EMF - Enhanced Metafile (*.emf)\0*.emf")
                       _T("\0WMF - Windows Metafile (*.wmf)\0*.wmf")
                       _T("\0ANI - Windows Animated Cursor (*.ani)\0*.ani")
                       _T("\0GIF - CompuServe GIF (*.gif)\0*.gif")
                       _T("\0JPG - JPEG / JFIF (*.jpg; *.jpeg)\0*.jpg; *.jpeg")
                       _T("\0PCX - Zsoft Publisher's Paintbrush (*.pcx)\0*.pcx")
                       _T("\0TIF - TIFF Revision 6 (*.tif)\0*.tif")
                       _T("\0EXE - Executable resource file (*.exe)\0*.exe")
                       _T("\0DLL - Dynamic-link library resource file (*.dll)\0*.dll")
                       _T("\0\0");
   int iLen = CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_FILTER_ALL_IMAGES).GetLength();
   TCHAR *szFilter = new TCHAR [iLen + (sizeof(szFilter2)>>(sizeof(TCHAR)-1))];
   lstrcpy(szFilter, CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_FILTER_ALL_IMAGES));
   memcpy(szFilter+iLen, szFilter2, sizeof(szFilter2));
   m_ofn.lpstrFilter = szFilter;//_T("All files (*.*)\0*.*\0\0");//

   m_ofn.lpstrFile = NULL;
   m_ofn.nMaxFile = max(MAX_PATH, m_strFile.GetLength()+1);

   BOOL bRet = FALSE;
   bool bAnew = false;
   do {
      bAnew = false;
      if (m_ofn.lpstrFile == NULL) {
         m_ofn.lpstrFile = new TCHAR[m_ofn.nMaxFile]; m_ofn.lpstrFile[0] = 0;
         if (!m_strFile.IsEmpty()) lstrcpyn(m_ofn.lpstrFile, m_strFile, m_ofn.nMaxFile);
      }
      bRet = ::GetOpenFileName(&m_ofn);
      if (bRet) {
         m_strFile = m_ofn.lpstrFile;
      } else {
         DWORD dwError = ::CommDlgExtendedError();
         m_strFile.Empty();
         m_strResTypeAndName.Empty();
         switch (dwError) {
         case FNERR_BUFFERTOOSMALL:
            m_ofn.nMaxFile = *(short*)m_ofn.lpstrFile;// first 2 bytes of the buffer pointed to by the lpstrFile member are set to an integer value specifying the size required to receive the full name.
            delete [] m_ofn.lpstrFile; m_ofn.lpstrFile = NULL;
            bAnew = true;
            break;
         case FNERR_INVALIDFILENAME:
            bAnew = true;
            break;
         }
      }
   } while(bAnew);

   m_hWnd = NULL;
   delete [] szFilter; szFilter = NULL;
   delete [] m_ofn.lpstrFile; m_ofn.lpstrFile = NULL;

   return bRet ? IDOK : IDCANCEL;
}

BOOL CDialogTemplateSelectImage::OnInitDialog(HWND, HWND, LPARAM) {
   m_BtnOk    .SubclassWindow(::GetDlgItem(::GetParent(m_hWnd), IDOK));
   m_BtnCancel.SubclassWindow(::GetDlgItem(::GetParent(m_hWnd), IDCANCEL));
   m_InfoText .SubclassWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_INFO_TEXT));

   ::SetWindowText(m_BtnOk    .m_hWnd                                           , CLang::GetString(IDS__OK                                      ));
   ::SetWindowText(m_BtnCancel.m_hWnd                                           , CLang::GetString(IDS__CANCEL                                  ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE)   , CLang::GetString(IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_IMAGE   ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE), CLang::GetString(IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_RESOURCE));

   m_Img.SetPlacePlacement(CImg::ePlaceUniformly);
   m_Img.SetTransparent(false);

   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE   );
 //SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_INFO_TEXT      );
   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE);
   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE  );
 //SETNEWWNDPROC(::GetParent(m_hWnd), stc32);

   SETNEWWNDPROC_HWND(::GetParent(m_hWnd), DefaultSelectFile);
   SETNEWWNDPROC(::GetParent(m_hWnd), stc4 );
   SETNEWWNDPROC(::GetParent(m_hWnd), cmb2 );
 //SETNEWWNDPROC(::GetParent(m_hWnd), stc1 );
 //SETNEWWNDPROC_HWND(FindToolbarFolder(::GetParent(m_hWnd)), stc1 );
   SETNEWWNDPROC(::GetParent(m_hWnd), ctl1 );
 //SETNEWWNDPROC(::GetParent(m_hWnd), lst1 );
   SETNEWWNDPROC(::GetParent(m_hWnd), lst2 );
   SETNEWWNDPROC(::GetParent(m_hWnd), stc3 );
   SETNEWWNDPROC(::GetDlgItem(::GetParent(m_hWnd), cmb13), cmb13);
   SETNEWWNDPROC(::GetParent(m_hWnd), stc2 );
   SETNEWWNDPROC(::GetParent(m_hWnd), cmb1 );
 //SETNEWWNDPROC(::GetParent(m_hWnd), chx1 );
 //SETNEWWNDPROC(::GetParent(m_hWnd), IDOK);
 //SETNEWWNDPROC(::GetParent(m_hWnd), IDCANCEL);
 //SETNEWWNDPROC(::GetParent(m_hWnd), pshHelp);
   SETNEWWNDPROC(::GetParent(m_hWnd), 0xFFFFFFFF);

   return TRUE;
}

//void CDialogTemplateSelectImage::OnClose(HWND) {
//}

void CDialogTemplateSelectImage::OnDestroy(HWND) {
   ClearTable();
   m_BtnOk    .UnsubclassWindow(TRUE);
   m_BtnCancel.UnsubclassWindow(TRUE);
   m_hWnd = NULL;
}

void CDialogTemplateSelectImage::OnSize(HWND, UINT state, int cx, int cy) {
   // @TODO: ,        ,    - 

   SIZE sizeCl = ::GetClientSize(::GetParent(m_hWnd));

   RECTEX rcBorderRes = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE)));
   rcBorderRes.right = sizeCl.cx-13;
   ::MoveWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE), rcBorderRes, TRUE);

   RECTEX rcBorderImg = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE)));
   rcBorderImg.bottom = rcBorderRes.top-10;
   ::MoveWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE), rcBorderImg, TRUE);

   RECTEX rcInfTxt = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_InfoText.m_hWnd));
   rcInfTxt.alignRight(rcBorderImg.right-3);
   ::MoveWindow(m_InfoText.m_hWnd, rcInfTxt, TRUE);

   RECTEX rcResLst = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)));
   rcResLst.right = rcBorderImg.right-5;
   ::MoveWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), rcResLst, TRUE);
}

void CDialogTemplateSelectImage::OnPaint(HWND) {
   if (//::FileExist(szFileName) &&
       m_Img.Valid())
   {
      SIZE sizeImg = m_Img.GetSizeImage();
      ::SetWindowText(m_InfoText.m_hWnd, ::Format(_T("%ix%i"), sizeImg.cx, sizeImg.cy));

      RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE)));
      RECTEX rcInfTxt = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_InfoText.m_hWnd));
      rc.left += 3; rc.right -= 3; rc.top += 12; rc.bottom = rcInfTxt.top-3;
      PAINTSTRUCT ps;
      
      HDC hDC = ::BeginPaint(m_hWnd, &ps);
      
      /**/
      m_Img.Draw(hDC, rc);
      /** /
      HDC hCDC = ::CreateCompatibleDC(hDC);
      HBITMAP hBmpNew = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
      HGDIOBJ hBmpOld = ::SelectObject(hCDC, hBmpNew);
      
      m_Img.Draw(hCDC, RECTEX(rc.size()));
      ::BitBlt(hDC, rc, hCDC, SRCCOPY);
      
      ::SelectObject(hCDC, hBmpOld);
      ::DeleteObject(hBmpNew);
      ::DeleteDC(hCDC);
      /**/
      
      ::EndPaint(m_hWnd, &ps);
   }
}

void CDialogTemplateSelectImage::OnMeasureItemThis(HWND, LPMEASUREITEMSTRUCT lpMeasureItem) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnMeasureItem: "));
   if (lpMeasureItem) {
      MEASUREITEMSTRUCT &mi = *lpMeasureItem;
      if (mi.CtlType != ODT_LISTVIEW) {
         LOG_PUT(g_pLog, Put, LL_ERROR) _T("  CDialogTemplateSelectImage::OnMeasureItem:   ! CtlType != ODT_LISTVIEW "));
      } else {
         mi.itemHeight = 47;
      }
   }
}

void CDialogTemplateSelectImage::OnDrawItem(HWND, const DRAWITEMSTRUCT *lpDrawItem) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnDrawItem: "));
   if (lpDrawItem) {
      const DRAWITEMSTRUCT &di = *lpDrawItem;
      if (di.CtlType != ODT_LISTVIEW) {
         LOG_PUT(g_pLog, Put, LL_ERROR) _T("  CDialogTemplateSelectImage::OnDrawItem:   ! CtlType != ODT_LISTVIEW "));
      } else {
         //CString strText;
         //GetItemText(di.itemID, 0, strText);

	      int cols = Header_GetItemCount(ListView_GetHeader(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)));
	      for(int n = 0; n < cols; n++)
	      {
		      RECTEX rcCell;
		      GetCellRect(n, di.rcItem, rcCell);

            HDC hCDC = ::CreateCompatibleDC(di.hDC);
            HBITMAP hBmpNew = ::CreateCompatibleBitmap(di.hDC, rcCell.width(), rcCell.height());
            HGDIOBJ hBmpOld = ::SelectObject(hCDC, hBmpNew);

            HBRUSH hBrushNew = NULL;
            bool bNeedDelBrush = false;
		      if ((m_iClickSubItem==n) && (di.itemState & ODS_SELECTED)) {
               hBrushNew = ::GetSysColorBrush(COLOR_HIGHLIGHT);
            } else {
               hBrushNew = GetFM2WinMain().GetSkin().m_bToAll ? (bNeedDelBrush = true, ::CreateSolidBrush(GetFM2WinMain().GetSkin().GetBkColor())) : ::GetSysColorBrush(COLOR_BTNFACE);
            }
            HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrushNew);
            ::Rectangle(hCDC, 0, 0, rcCell.width(), rcCell.height());
            //::PatBlt(hCDC, 0,0, rcCell.width(), rcCell.height(), PATCOPY);
            ::SelectObject(hCDC, hBrushOld);
            if (bNeedDelBrush ) {
               ::DeleteObject(hBrushNew); hBrushNew = NULL;
            }

            //::DrawText(hCDC, m_ImgRes[n]->operator CImageSerialize().m_strPath, -1, &RECTEX(rcCell).move(-rcCell.left, -rcCell.top), DT_CENTER | DT_END_ELLIPSIS  | DT_VCENTER | DT_SINGLELINE);
            m_ImgRes[n]->Draw(hCDC, RECTEX(rcCell).move(-rcCell.pointLT().x, -rcCell.pointLT().y));

            ::BitBlt(di.hDC, rcCell, hCDC, SRCCOPY);
            ::SelectObject(hCDC, hBmpOld);
            ::DeleteDC(hCDC);
            ::DeleteObject(hBmpNew);
	      }
      }
   }
}

LRESULT CDialogTemplateSelectImage::OnNotifyCustomDraw(NMLVCUSTOMDRAW &nmcd) {
   //return CDRF_SKIPDEFAULT;
   return 0;
}

BOOL CDialogTemplateSelectImage::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hdc, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

LRESULT CDialogTemplateSelectImage::OnNotifySelectChange(OFNOTIFY &ofn) {
   TCHAR szFileName[MAX_PATH] = {0};
   int iRes = CommDlg_OpenSave_GetFilePath(ofn.hdr.hwndFrom, szFileName, MAX_PATH);//CommDlg_OpenSave_GetFilePath(::GetParent(m_hWnd), szFileName, MAX_PATH);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnNotifySelectChange: iRes=%i; szFileName [%s] "), iRes, szFileName);

   if (m_strFile == szFileName) return 0;

   ClearTable();

   m_strFile = szFileName;
   m_strResTypeAndName.Empty();
   if (::PathExist(szFileName)) {
      SetErrorText(NO_ERROR);
   } else {
      if (!m_Img.Load(szFileName)) {
         SetErrorText();
         m_Img.Reset();

         HMODULE hMod = ::LoadLibraryEx(szFileName, NULL, LOAD_LIBRARY_AS_DATAFILE);
         if (!hMod) {
            //SetErrorText();
         } else {
#if defined _DEBUG && defined __FILE__ENUM_OBJECTS__
            PutTraceResource(hMod);
#endif
            ::EnumResourceTypes(hMod, CDialogTemplateSelectImage::EnumResourceFunc, (LONG_PTR)this);
            if (m_ImgRes.empty()) {
               SetErrorText(ERROR_RESOURCE_TYPE_NOT_FOUND);
            } else {
               SetErrorText(NO_ERROR);
               FillTable();
             //FORWARD_WM_LBUTTONDOWN(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), false, 0,0, 0, ::PostMessage);
             //FORWARD_WM_LBUTTONUP(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), 0,0, 0, ::PostMessage);
            }
         }
      }
   }

   RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE)));
   ::InvalidateRect(m_hWnd, &rc, TRUE);

   return 0;
}

LRESULT CDialogTemplateSelectImage::OnNotifyClickCell(NMITEMACTIVATE &ia, bool bDbl) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnNotifyClickCell: iSubItem=%i "), ia.iSubItem);
   if (m_iClickSubItem != ia.iSubItem) {
      m_iClickSubItem = ia.iSubItem;
      m_strResTypeAndName = m_ImgRes[m_iClickSubItem]->operator CImageSerialize().m_strPath.Mid(m_strFile.GetLength());
      m_Img = *m_ImgRes[m_iClickSubItem];
      if (bDbl) {
         if(m_ImgRes[m_iClickSubItem]->GetPlacePlacement()!=CImg::ePlaceUniformly)
            m_ImgRes[m_iClickSubItem]->SetPlacePlacement(   CImg::ePlaceUniformly);
         else
            m_ImgRes[m_iClickSubItem]->SetPlacePlacement(   CImg::ePlaceStretch);
      }
      ::InvalidateRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), NULL, FALSE);
      RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE)));
      ::InvalidateRect(m_hWnd, &rc, TRUE);
   }
   return 0;
}

void CDialogTemplateSelectImage::ClearTable() {
   int i=0;
   for (; i<m_ImgRes.size(); i++) {
      delete m_ImgRes[i];
   }
   m_ImgRes.clear();
   int cols = Header_GetItemCount(ListView_GetHeader(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)));
   for (i=cols-1; i>=0; i--) {
      ListView_DeleteColumn(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), i);
   }
   ListView_DeleteAllItems(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE));

   ::EnableWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE), FALSE);
   ::EnableWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)  , FALSE);
}

void CDialogTemplateSelectImage::FillTable() {
   //ClearTable();

   int n = 0;
   for (; n<1; n++) {
      AddItem(n, 0, _T(""));
   }

   for (n=0; n<m_ImgRes.size(); n++) {
		//AddColumn(_T("col"), n);
      InsertColumn(n, _T("col"), LVCFMT_LEFT, 47, -1);
   }
}

int CDialogTemplateSelectImage::InsertColumn(int nCol, LPCTSTR lpszColumnHeading, int nFormat, int nWidth, int nSubItem) {
	LV_COLUMN column;
	column.mask = LVCF_TEXT|LVCF_FMT;
	column.pszText = (LPTSTR)lpszColumnHeading;
	column.fmt = nFormat;
	if (nWidth != -1)
	{
		column.mask |= LVCF_WIDTH;
		column.cx = nWidth;
	}
	if (nSubItem != -1)
	{
		column.mask |= LVCF_SUBITEM;
		column.iSubItem = nSubItem;
	}
	return ListView_InsertColumn(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), nCol, &column);
}

/*
int CDialogTemplateSelectImage::AddColumn(LPCTSTR strItem, int nItem, int nSubItem, int nMask, int nFmt) {
	LV_COLUMN lvc;
	lvc.mask = nMask;
	lvc.fmt = nFmt;
	lvc.pszText = (LPTSTR)strItem;
	lvc.cx = ListView_GetStringWidth(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), lvc.pszText) + 15;
	if(nMask & LVCF_SUBITEM)
		lvc.iSubItem = (nSubItem != -1) ? nSubItem : nItem;
	return ListView_InsertColumn(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), nItem, &lvc);
}
/**/

int CDialogTemplateSelectImage::AddItem(int nItem, int nSubItem, LPCTSTR strItem, int nImageIndex) {
	LVITEM lvItem;
	lvItem.mask = LVIF_TEXT;
	lvItem.iItem = nItem;
	lvItem.iSubItem = nSubItem;
	lvItem.pszText = (LPTSTR)strItem;
	if(nImageIndex != -1)
	{
		lvItem.mask |= LVIF_IMAGE;
		lvItem.iImage = nImageIndex;
	}
	if(nSubItem == 0)
		return ListView_InsertItem(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), &lvItem);
	return ListView_SetItem(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), &lvItem) ? nItem : -1;
}

/** /
int CDialogTemplateSelectImage::GetItemText(int nItem, int nSubItem, CString& strText) const {
	LVITEM lvi;
	memset(&lvi, 0, sizeof(LVITEM));
	lvi.iSubItem = nSubItem;

	strText.Empty();
	int nRes = 0;
	for(int nLen = 256; ; nLen *= 2)
	{
		lvi.cchTextMax = nLen;
		lvi.pszText = strText.GetBufferSetLength(nLen);
		if(lvi.pszText == NULL)
		{
			nRes = 0;
			break;
		}
		nRes  = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), LVM_GETITEMTEXT, (WPARAM)nItem, (LPARAM)&lvi);
		if(nRes < nLen - 1)
			break;
	}
	strText.ReleaseBuffer();
	return nRes;
}
/**/

void CDialogTemplateSelectImage::GetCellRect(int iCol, const RECTEX& rcItem, RECTEX& rcCell) const {
	RECTEX rcHeader;
	Header_GetItemRect(ListView_GetHeader(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)), iCol, &rcHeader);

	// If we don't do this, when we scroll to the right, we will be 
	// drawing as if we weren't and your cells won't line up with the
	// columns.
   int x_offset = -::GetScrollPos(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE), SB_HORZ);

	rcCell.left   = x_offset + rcHeader.left;
	rcCell.right  = x_offset + rcHeader.right;
	rcCell.top    = rcItem.top;
	rcCell.bottom = rcItem.bottom;
}

void CDialogTemplateSelectImage::AddToImgList(LPCTSTR szResTypeAndName) {
   CImage *pImg = new CImage(m_strFile+szResTypeAndName);
   pImg->SetPlacePlacement(CImg::ePlaceUniformly);
   pImg->SetTransparent(false);
   if (pImg->Valid()) {
      m_ImgRes.push_back(pImg);
      ::EnableWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE), TRUE);
      ::EnableWindow(::GetDlgItem(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE)  , TRUE);
   } else {
      delete pImg; pImg = NULL;
   }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

CDialogTemplateSelectImage::CDialogTemplateSelectImage(HWND hWndParent, LPCTSTR szFile, LPCTSTR szInitialDir, LPCTSTR szTitle, LPCTSTR szDefExt) :
   CFileDialogImpl<CDialogTemplateSelectImage>(
      TRUE,
      NULL, //szDefExt,
      NULL, //szFile,
      OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_PATHMUSTEXIST | OFN_ENABLESIZING | OFN_ENABLEHOOK | OFN_ENABLETEMPLATE,
      NULL,
      hWndParent
   ),
   m_strFile      (szFile),
   m_strInitialDir(szInitialDir),
   m_strTitle     (szTitle),
   m_strDefExt    (szDefExt),
   m_iClickSubItem(-1)
{
   m_ofn.lpstrFilter = NULL; // see DoModal...
   m_ofn.lpstrFile = NULL; // see DoModal...
   m_ofn.nMaxFile = 0; // see DoModal...
   m_ofn.lpstrInitialDir = m_strInitialDir.IsEmpty() ? NULL : (LPCTSTR)m_strInitialDir;
   m_ofn.lpstrTitle      = m_strTitle     .IsEmpty() ? NULL : (LPCTSTR)m_strTitle;
   m_ofn.lpstrDefExt     = m_strDefExt    .IsEmpty() ? NULL : (LPCTSTR)m_strDefExt;

   m_ofn.lpTemplateName  = CDialogTemplateSelectImage::IDD();

   m_Img.Load(szFile ? szFile : szInitialDir);
}

INT_PTR CDialogTemplateSelectImage::DoModal(HWND hWndParent) {
   TCHAR szFilter2[] = _T("\0*.bmp; *.ico; *.emf; *.wmf; *.ani; *.gif; *.jpg; *.jpeg; *.pcx; *.tif; *.exe; *.dll")
                       _T("\0BMP - Windows Bitmap (*.bmp)\0*.bmp")
                       _T("\0ICO - Icon (*.ico)\0*.ico")
                       _T("\0EMF - Enhanced Metafile (*.emf)\0*.emf")
                       _T("\0WMF - Windows Metafile (*.wmf)\0*.wmf")
                       _T("\0ANI - Windows Animated Cursor (*.ani)\0*.ani")
                       _T("\0GIF - CompuServe GIF (*.gif)\0*.gif")
                       _T("\0JPG - JPEG / JFIF (*.jpg; *.jpeg)\0*.jpg; *.jpeg")
                       _T("\0PCX - Zsoft Publisher's Paintbrush (*.pcx)\0*.pcx")
                       _T("\0TIF - TIFF Revision 6 (*.tif)\0*.tif")
                       _T("\0EXE - Executable resource file (*.exe)\0*.exe")
                       _T("\0DLL - Dynamic-link library resource file (*.dll)\0*.dll")
                       _T("\0\0");
   int iLen = CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_FILTER_ALL_IMAGES).GetLength();
   TCHAR *szFilter = new TCHAR [iLen + (sizeof(szFilter2)>>(sizeof(TCHAR)-1))];
   lstrcpy(szFilter, CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_FILTER_ALL_IMAGES));
   memcpy(szFilter+iLen, szFilter2, sizeof(szFilter2));
   m_ofn.lpstrFilter = szFilter;//_T("All files (*.*)\0*.*\0\0");//

   m_ofn.lpstrFile = NULL;
   m_ofn.nMaxFile = max(MAX_PATH, m_strFile.GetLength()+1);

   BOOL bRet = FALSE;
   bool bAnew = false;
   do {
      bAnew = false;
      if (m_ofn.lpstrFile == NULL) {
         m_ofn.lpstrFile = new TCHAR[m_ofn.nMaxFile]; m_ofn.lpstrFile[0] = 0;
         if (!m_strFile.IsEmpty()) lstrcpyn(m_ofn.lpstrFile, m_strFile, m_ofn.nMaxFile);
      }
      bRet = CFileDialogImpl<CDialogTemplateSelectImage>::DoModal(hWndParent) == IDOK;//GetOpenFileName(&m_ofn);
      if (bRet) {
         m_strFile = m_ofn.lpstrFile;
      } else {
         DWORD dwError = ::CommDlgExtendedError();
         m_strFile.Empty();
         m_strResTypeAndName.Empty();
         switch (dwError) {
         case FNERR_BUFFERTOOSMALL:
            m_ofn.nMaxFile = *(short*)m_ofn.lpstrFile;// first 2 bytes of the buffer pointed to by the lpstrFile member are set to an integer value specifying the size required to receive the full name.
            delete [] m_ofn.lpstrFile; m_ofn.lpstrFile = NULL;
            bAnew = true;
            break;
         case FNERR_INVALIDFILENAME:
            bAnew = true;
            break;
         }
      }
   } while(bAnew);

   m_hWnd = NULL;
   delete [] szFilter; szFilter = NULL;
   delete [] m_ofn.lpstrFile; m_ofn.lpstrFile = NULL;

   return bRet ? IDOK : IDCANCEL;
}

BOOL CDialogTemplateSelectImage::OnInitDialog(HWND, LPARAM) {
   DoDataExchange();
   m_BtnOk    .SubclassWindow(::GetDlgItem(GetParent(), IDOK));
   m_BtnCancel.SubclassWindow(::GetDlgItem(GetParent(), IDCANCEL));

   m_BtnOk    .SetWindowText(CLang::GetString(IDS__OK                                      ));
   m_BtnCancel.SetWindowText(CLang::GetString(IDS__CANCEL                                  ));
   m_BorderImg.SetWindowText(CLang::GetString(IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_IMAGE   ));
   m_BorderRes.SetWindowText(CLang::GetString(IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_RESOURCE));

   m_Img.SetPlacePlacement(CImg::ePlaceUniformly);
   m_Img.SetTransparent(false);

   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_IMAGE   );
 //SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_INFO_TEXT      );
   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_BORDER_RESOURCE);
   SETNEWWNDPROC(m_hWnd, IDC_TEMPLATESELECTIMAGE_LIST_RESOURCE  );
 //SETNEWWNDPROC(::GetParent(m_hWnd), stc32);

   SETNEWWNDPROC_HWND(GetParent(), DefaultSelectFile);

   SETNEWWNDPROC(GetParent(), stc4 );
   SETNEWWNDPROC(GetParent(), cmb2 );
 //SETNEWWNDPROC(GetParent(), stc1 );
 //SETNEWWNDPROC_HWND(FindToolbarFolder(GetParent()), stc1 );
   SETNEWWNDPROC(GetParent(), ctl1 );
 //SETNEWWNDPROC(GetParent(), lst1 );
   SETNEWWNDPROC(GetParent(), lst2 );
   SETNEWWNDPROC(GetParent(), stc3 );
   SETNEWWNDPROC(::GetDlgItem(GetParent(), cmb13), cmb13);
   SETNEWWNDPROC(GetParent(), stc2 );
   SETNEWWNDPROC(GetParent(), cmb1 );
 //SETNEWWNDPROC(GetParent(), chx1 );
 //SETNEWWNDPROC(GetParent(), IDOK);
 //SETNEWWNDPROC(GetParent(), IDCANCEL);
 //SETNEWWNDPROC(GetParent(), pshHelp);
   SETNEWWNDPROC(GetParent(), 0xFFFFFFFF);

   return TRUE;
}

//void CDialogTemplateSelectImage::OnClose() {
//}

void CDialogTemplateSelectImage::OnDestroy() {
   ClearTable();
   m_BtnOk    .UnsubclassWindow(TRUE);
   m_BtnCancel.UnsubclassWindow(TRUE);
}

void CDialogTemplateSelectImage::OnSize(UINT state, CSize &size) {
   // @TODO: ,        ,    - 

   SIZE sizeCl = ::GetClientSize(GetParent());

   RECTEX rcBorderRes = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_BorderRes));
   rcBorderRes.right = sizeCl.cx-13;
   m_BorderRes.MoveWindow(&rcBorderRes, TRUE);

   RECTEX rcBorderImg = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_BorderImg));
   rcBorderImg.bottom = rcBorderRes.top-10;
   m_BorderImg.MoveWindow(&rcBorderImg, TRUE);

   RECTEX rcInfTxt = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_InfoText));
   rcInfTxt.alignRight(rcBorderImg.right-3);
   m_InfoText.MoveWindow(&rcInfTxt, TRUE);

   RECTEX rcResLst = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_ListRes));
   rcResLst.right = rcBorderImg.right-5;
   m_ListRes.MoveWindow(&rcResLst, TRUE);

   SetMsgHandled(FALSE);
}

void CDialogTemplateSelectImage::OnPaint(HDC hDC) {
   if (//::FileExist(szFileName) &&
       m_Img.Valid())
   {
      SIZE sizeImg = m_Img.GetSizeImage();
      m_InfoText.SetWindowText(::Format(_T("%ix%i"), sizeImg.cx, sizeImg.cy));

      RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_BorderImg));
      RECTEX rcInfTxt = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_InfoText));
      rc.left += 3; rc.right -= 3; rc.top += 12; rc.bottom = rcInfTxt.top-3;
      CPaintDC hDC(m_hWnd);
      m_Img.Draw(hDC, rc);
   }
   SetMsgHandled(FALSE);
}

void CDialogTemplateSelectImage::OnMeasureItem(UINT idControl, LPMEASUREITEMSTRUCT lpMeasureItem) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnMeasureItem: "));
   if (lpMeasureItem) {
      MEASUREITEMSTRUCT &mi = *lpMeasureItem;
      if (mi.CtlType != ODT_LISTVIEW) {
         LOG_PUT(g_pLog, Put, LL_ERROR) _T("  CDialogTemplateSelectImage::OnMeasureItem:   ! CtlType != ODT_LISTVIEW "));
      } else {
         mi.itemHeight = 47;
      }
   }
}

void CDialogTemplateSelectImage::OnDrawItem(UINT idControl, LPDRAWITEMSTRUCT lpDrawItem) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnDrawItem: "));
   if (lpDrawItem) {
      const DRAWITEMSTRUCT &di = *lpDrawItem;
      if (di.CtlType != ODT_LISTVIEW) {
         LOG_PUT(g_pLog, Put, LL_ERROR) _T("  CDialogTemplateSelectImage::OnDrawItem:   ! CtlType != ODT_LISTVIEW "));
      } else {
         //CString strText;
         //GetItemText(di.itemID, 0, strText);

	      int cols = CHeaderCtrl(m_ListRes.GetHeader()).GetItemCount();
	      for(int n = 0; n < cols; n++)
	      {
		      RECTEX rcCell;
		      GetCellRect(n, di.rcItem, rcCell);

            CDC hCDC; hCDC.CreateCompatibleDC(di.hDC);
            HBITMAP hBmpNew = ::CreateCompatibleBitmap(di.hDC, rcCell.width(), rcCell.height());
            HBITMAP hBmpOld = hCDC.SelectBitmap(hBmpNew);

            HBRUSH hBrushNew = NULL;
            bool bNeedDelBrush = false;
		      if ((m_iClickSubItem==n) && (di.itemState & ODS_SELECTED)) {
               hBrushNew = ::GetSysColorBrush(COLOR_HIGHLIGHT);
            } else {
               hBrushNew = GetFM2WinMain().GetSkin().m_bToAll ? (bNeedDelBrush = true, ::CreateSolidBrush(GetFM2WinMain().GetSkin().GetBkColor())) : ::GetSysColorBrush(COLOR_BTNFACE);
            }
            HBRUSH hBrushOld = hCDC.SelectBrush(hBrushNew);
            hCDC.Rectangle(0, 0, rcCell.width(), rcCell.height());
            //hCDC.PatBlt(0,0, rcCell.width(), rcCell.height(), PATCOPY);
            hCDC.SelectBrush(hBrushOld);
            if (bNeedDelBrush) {
               ::DeleteObject(hBrushNew); hBrushNew = NULL;
            }

            //hCDC.DrawText(m_ImgRes[n]->operator CImageSerialize().m_strPath, -1, &RECTEX(rcCell).move(-rcCell.left, -rcCell.top), DT_CENTER | DT_END_ELLIPSIS  | DT_VCENTER | DT_SINGLELINE);
            m_ImgRes[n]->Draw(hCDC, RECTEX(rcCell).move(-rcCell.pointLT().x, -rcCell.pointLT().y));

            ::BitBlt(di.hDC, rcCell, hCDC, SRCCOPY);
            hCDC.SelectBitmap(hBmpOld);
            ::DeleteObject(hBmpNew);
	      }
      }
   }
}

LRESULT CDialogTemplateSelectImage::OnNotifyCustomDraw(LPNMHDR pnmh) {
   //NMLVCUSTOMDRAW &nmcd = *(LPNMLVCUSTOMDRAW)pnmh;
   //return CDRF_SKIPDEFAULT;
   return 0;
}

BOOL CDialogTemplateSelectImage::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hdc, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

LRESULT CDialogTemplateSelectImage::OnNotifySelectChange(LPNMHDR pnmh) {
   OFNOTIFY &ofn = *(LPOFNOTIFY)pnmh;
   TCHAR szFileName[MAX_PATH] = {0};
   int iRes = GetFilePath(szFileName, MAX_PATH);//GetFilePath(GetParent(), szFileName, MAX_PATH);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnNotifySelectChange: iRes=%i; szFileName [%s] "), iRes, szFileName);

   if (m_strFile == szFileName) return 0;

   ClearTable();

   m_strFile = szFileName;
   m_strResTypeAndName.Empty();
   if (::PathExist(szFileName)) {
      SetErrorText(NO_ERROR);
   } else {
      if (!m_Img.Load(szFileName)) {
         SetErrorText();
         m_Img.Reset();

         HMODULE hMod = ::LoadLibraryEx(szFileName, NULL, LOAD_LIBRARY_AS_DATAFILE);
         if (!hMod) {
            //SetErrorText();
         } else {
#if defined _DEBUG && defined __FILE__ENUM_OBJECTS__
            PutTraceResource(hMod);
#endif
            ::EnumResourceTypes(hMod, CDialogTemplateSelectImage::EnumResourceFunc, (LONG_PTR)this);
            if (m_ImgRes.empty()) {
               SetErrorText(ERROR_RESOURCE_TYPE_NOT_FOUND);
            } else {
               SetErrorText(NO_ERROR);
               FillTable();
            }
         }
      }
   }

   RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_BorderImg));
   InvalidateRect(&rc, TRUE);

   return 0;
}

LRESULT CDialogTemplateSelectImage::OnNotifyClickCell(LPNMHDR pnmh) {
   NMITEMACTIVATE &ia = *(LPNMITEMACTIVATE)pnmh;
   bool bDbl = (pnmh->code == NM_DBLCLK);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogTemplateSelectImage::OnNotifyClickCell: iSubItem=%i "), ia.iSubItem);
   if ((ia.iSubItem >= 0) &&
       (ia.iSubItem != m_iClickSubItem))
   {
      m_iClickSubItem = ia.iSubItem;
      m_strResTypeAndName = m_ImgRes[m_iClickSubItem]->operator CImageSerialize().m_strPath.Mid(m_strFile.GetLength());
      m_Img = *m_ImgRes[m_iClickSubItem];
      if (bDbl) {
         if(m_ImgRes[m_iClickSubItem]->GetPlacePlacement()!=CImg::ePlaceUniformly)
            m_ImgRes[m_iClickSubItem]->SetPlacePlacement(   CImg::ePlaceUniformly);
         else
            m_ImgRes[m_iClickSubItem]->SetPlacePlacement(   CImg::ePlaceStretch);
      }
      m_ListRes.InvalidateRect(NULL, FALSE);
      RECTEX rc = ::ScreenToClient(m_hWnd, ::GetWindowRect(m_BorderImg));
      InvalidateRect(&rc, TRUE);
   }
   return 0;
}

void CDialogTemplateSelectImage::ClearTable() {
   int i=0;
   for (; i<m_ImgRes.size(); i++) {
      delete m_ImgRes[i];
   }
   m_ImgRes.clear();
   int cols = CHeaderCtrl(m_ListRes.GetHeader()).GetItemCount();
   for (i=cols-1; i>=0; i--) {
      m_ListRes.DeleteColumn(i);
   }
   m_ListRes.DeleteAllItems();

   m_BorderRes.EnableWindow(FALSE);
   m_ListRes.EnableWindow(FALSE);
}

void CDialogTemplateSelectImage::FillTable() {
   //ClearTable();

   int n = 0;
   for (; n < 1; n++) {
      AddItem(n, 0, _T(""));
   }

   for (n=0; n<m_ImgRes.size(); n++) {
		//AddColumn(_T("col"), n);
      InsertColumn(n, _T("col"), LVCFMT_LEFT, 47, -1);
   }
}

int CDialogTemplateSelectImage::InsertColumn(int nCol, LPCTSTR lpszColumnHeading, int nFormat, int nWidth, int nSubItem) {
	LV_COLUMN column;
	column.mask = LVCF_TEXT|LVCF_FMT;
	column.pszText = (LPTSTR)lpszColumnHeading;
	column.fmt = nFormat;
	if (nWidth != -1)
	{
		column.mask |= LVCF_WIDTH;
		column.cx = nWidth;
	}
	if (nSubItem != -1)
	{
		column.mask |= LVCF_SUBITEM;
		column.iSubItem = nSubItem;
	}
	return m_ListRes.InsertColumn(nCol, &column);
}

/*
int CDialogTemplateSelectImage::AddColumn(LPCTSTR strItem, int nItem, int nSubItem, int nMask, int nFmt) {
	LV_COLUMN lvc;
	lvc.mask = nMask;
	lvc.fmt = nFmt;
	lvc.pszText = (LPTSTR)strItem;
	lvc.cx = m_ListRes.GetStringWidth(lvc.pszText) + 15;
	if(nMask & LVCF_SUBITEM)
		lvc.iSubItem = (nSubItem != -1) ? nSubItem : nItem;
	return m_ListRes.InsertColumn(nItem, &lvc);
}
/**/

int CDialogTemplateSelectImage::AddItem(int nItem, int nSubItem, LPCTSTR strItem, int nImageIndex) {
	LVITEM lvItem;
	lvItem.mask = LVIF_TEXT;
	lvItem.iItem = nItem;
	lvItem.iSubItem = nSubItem;
	lvItem.pszText = (LPTSTR)strItem;
	if(nImageIndex != -1)
	{
		lvItem.mask |= LVIF_IMAGE;
		lvItem.iImage = nImageIndex;
	}
	if(nSubItem == 0)
		return m_ListRes.InsertItem(&lvItem);
	return m_ListRes.SetItem(&lvItem) ? nItem : -1;
}

void CDialogTemplateSelectImage::GetCellRect(int iCol, const RECTEX& rcItem, RECTEX& rcCell) const {
	RECTEX rcHeader;
   CHeaderCtrl(m_ListRes.GetHeader()).GetItemRect(iCol, &rcHeader);

	// If we don't do this, when we scroll to the right, we will be 
	// drawing as if we weren't and your cells won't line up with the
	// columns.
   int x_offset = -m_ListRes.GetScrollPos(SB_HORZ);

	rcCell.left   = x_offset + rcHeader.left;
	rcCell.right  = x_offset + rcHeader.right;
	rcCell.top    = rcItem.top;
	rcCell.bottom = rcItem.bottom;
}

void CDialogTemplateSelectImage::AddToImgList(LPCTSTR szResTypeAndName) {
   CImage *pImg = new CImage(m_strFile+szResTypeAndName);
   pImg->SetPlacePlacement(CImg::ePlaceUniformly);
   pImg->SetTransparent(false);
   if (pImg->Valid()) {
      m_ImgRes.push_back(pImg);
      m_BorderRes.EnableWindow(TRUE);
      m_ListRes.EnableWindow(TRUE);
   } else {
      delete pImg; pImg = NULL;
   }
}

#endif // _WTL_VER

/* static */ BOOL CALLBACK CDialogTemplateSelectImage::EnumResLangFunc(HMODULE hMod, LPCTSTR lpszType, LPCTSTR lpszName, WORD wIDLanguage,  LONG lParam) {
   CString strType;
   if (IS_INTRESOURCE(lpszType)) {
      switch ((ULONG_PTR)lpszType) {
      case RT_ANICURSOR   :
      case RT_ANIICON     :
      case RT_BITMAP      :
      case RT_CURSOR      :
      case RT_GROUP_CURSOR:
      case RT_GROUP_ICON  :
      case RT_ICON        :
      case RT_RCDATA      : strType.Format(_T("%i"), lpszType);
      }
   } else {
      strType = lpszType;
   }
   if (!strType.IsEmpty()) {
      CString strName;
      if (IS_INTRESOURCE(lpszName)) {
         strName.Format(_T("%i"), lpszName);
      } else {
         strName = lpszName;
      }
      ((CDialogTemplateSelectImage*)lParam)->AddToImgList(_T('?')+strType+_T('#')+strName);
   }
   return TRUE;
}

/* static */ BOOL CALLBACK CDialogTemplateSelectImage::EnumResNameFunc(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG lParam) {
   return ::EnumResourceLanguages(hMod, lpszType, lpszName, CDialogTemplateSelectImage::EnumResLangFunc, lParam);
}
 
/* static */ BOOL CALLBACK CDialogTemplateSelectImage::EnumResourceFunc(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
   return ::EnumResourceNames(hMod, lpszType, CDialogTemplateSelectImage::EnumResNameFunc, lParam);
}
