////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "FMToolbar.cpp"
//    "m_AssistantInfo Options"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "ToolbarFM.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "../DynLoadLib.h"

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

////////////////////////////////////////////////////////////////////////////////
//                          implementation
////////////////////////////////////////////////////////////////////////////////

#ifndef _WTL_VER

HWND CToolbarFM::Create(HWND hWndParent) {
   return ::CreateDialogParam(::GetModuleHandle(NULL), CToolbarFM::IDD(), hWndParent, (DLGPROC)CToolbarFM::DialogProcThis, (LPARAM)this);
}

/*static*/ INT_PTR CALLBACK CToolbarFM::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CToolbarFM::DialogProcThis: %s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CToolbarFM *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CToolbarFM*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CToolbarFM*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CToolbarFM::OnInitDialog(HWND, HWND hWndFocus, LPARAM lParam) {
   m_BtnNew   .SubclassWindow(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_BUTTON_NEW));
   m_BtnPause .SubclassWindow(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_BUTTON_PAUSE));
   m_EditMines.SubclassWindow(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_EDIT_MINES));
   m_EditTime .SubclassWindow(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_EDIT_TIME));
   return TRUE;
}

void CToolbarFM::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_FMTOOBAR_BUTTON_NEW:
      if (codeNotify == BN_CLICKED) {
         FORWARD_WM_COMMAND(::GetParent(m_hWnd), IDA_NEW_GAME, hWndCtl, 0, ::SendMessage);
      }
      if (codeNotify == BN_SETFOCUS) {
         ::SetFocus(::GetParent(m_hWnd));
      }
      break;
   case IDC_FMTOOBAR_BUTTON_PAUSE:
      if (codeNotify == BN_CLICKED) {
         FORWARD_WM_COMMAND(::GetParent(m_hWnd), IDA_PAUSE, hWndCtl, 0, ::SendMessage);
      }
      if (codeNotify == BN_SETFOCUS) {
         ::SetFocus(::GetParent(m_hWnd));
      }
      break;
   case IDC_FMTOOBAR_EDIT_MINES:
   case IDC_FMTOOBAR_EDIT_TIME:
      if (codeNotify == EN_SETFOCUS) {
         ::EnableWindow(hWndCtl, FALSE);
         ::EnableWindow(hWndCtl, TRUE );
         ::SetFocus(::GetParent(m_hWnd));
      }
      break;
   }
}

void CToolbarFM::OnSize(HWND, UINT state, int cx, int cy) {
   BOOL bXPStyle = CWinXPTheme::IsThemeActive() && CWinXPTheme::IsThemeActive()();
   const int WIDTH_BUTTON_NEW_PAUSE = 30+!!bXPStyle;
   const int HEIGHT_BUTTON_NEW_PAUSE = 30+!!bXPStyle;
   SIZE EDIT_COUNT_TIMER = {40, 21}; //    edit 

   ::MoveWindow(m_BtnPause.m_hWnd,
      cx/2,
      cy/2 - HEIGHT_BUTTON_NEW_PAUSE/2,
      WIDTH_BUTTON_NEW_PAUSE,
      HEIGHT_BUTTON_NEW_PAUSE,
      TRUE);
   ::MoveWindow(m_BtnNew.m_hWnd,
      cx/2 - WIDTH_BUTTON_NEW_PAUSE,
      cy/2 - HEIGHT_BUTTON_NEW_PAUSE/2,
      WIDTH_BUTTON_NEW_PAUSE,
      HEIGHT_BUTTON_NEW_PAUSE,
      TRUE);
   SIZE sizeMosaic = GetFM2WinMain().GetSizeMosaicWindow();
   int w = sizeMosaic.cx/2-WIDTH_BUTTON_NEW_PAUSE;
   int x = w;
   if (x > EDIT_COUNT_TIMER.cx) x = EDIT_COUNT_TIMER.cx;
   int ofs =  w/2 - x/2;
   if (ofs > 10) ofs = 10;
   if (ofs < 10) x -= 3;
   ::MoveWindow(m_EditMines.m_hWnd,
      ofs,
      cy/2 - EDIT_COUNT_TIMER.cy/2,
      x,
      EDIT_COUNT_TIMER.cy,
      TRUE); //
   ::MoveWindow(m_EditTime.m_hWnd,
      cx - x - ofs,
      cy/2 - EDIT_COUNT_TIMER.cy/2,
      x,
      EDIT_COUNT_TIMER.cy,
      TRUE); //
}

void CToolbarFM::OnDestroy(HWND) {
   m_hWnd = NULL;
}

BOOL CToolbarFM::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

void CToolbarFM::SetFont(const LOGFONT &font) {
   RECT Rect = ::GetClientRect(m_EditMines.m_hWnd);
   LOGFONT logFont = font;
   logFont.lfHeight = Rect.bottom;

   ::DeleteObject(FORWARD_WM_GETFONT(m_EditMines.m_hWnd, ::SendMessage)); //    DeleteObject  . ,      
   ::DeleteObject(FORWARD_WM_GETFONT(m_EditTime .m_hWnd, ::SendMessage)); //  --//--
   FORWARD_WM_SETFONT(m_EditMines.m_hWnd, ::CreateFontIndirect(&logFont), TRUE, ::SendMessage);
   FORWARD_WM_SETFONT(m_EditTime .m_hWnd, ::CreateFontIndirect(&logFont), TRUE, ::SendMessage);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// modeless dialogs
HWND CToolbarFM::Create(HWND hWndParent, LPARAM dwInitParam)
{
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, this);
#ifdef _DEBUG
	m_bModal = false;
#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CToolbarFM::IDD(), hWndParent, (DLGPROC)CToolbarFM::StartDialogProc, dwInitParam);
	ATLASSERT(hWnd == m_hWnd);
	return m_hWnd;
}

BOOL CToolbarFM::OnInitDialog(HWND hWndFocus, LPARAM lParam) {

   DoDataExchange();


   return TRUE;
}

void CToolbarFM::OnSize(UINT state, CSize &size) {
   BOOL bXPStyle = CWinXPTheme::IsThemeActive() && CWinXPTheme::IsThemeActive()();
   const int WIDTH_BUTTON_NEW_PAUSE = 30+!!bXPStyle;
   const int HEIGHT_BUTTON_NEW_PAUSE = 30+!!bXPStyle;
   SIZE EDIT_COUNT_TIMER = {40, 21}; //    edit 

   m_BtnPause.MoveWindow(
      size.cx/2,
      size.cy/2 - HEIGHT_BUTTON_NEW_PAUSE/2,
      WIDTH_BUTTON_NEW_PAUSE,
      HEIGHT_BUTTON_NEW_PAUSE,
      TRUE);
   m_BtnNew.MoveWindow(
      size.cx/2 - WIDTH_BUTTON_NEW_PAUSE,
      size.cy/2 - HEIGHT_BUTTON_NEW_PAUSE/2,
      WIDTH_BUTTON_NEW_PAUSE,
      HEIGHT_BUTTON_NEW_PAUSE,
      TRUE);
   SIZE sizeMosaic = GetFM2WinMain().GetSizeMosaicWindow();
   int w = sizeMosaic.cx/2-WIDTH_BUTTON_NEW_PAUSE;
   int x = w;
   if (x > EDIT_COUNT_TIMER.cx) x = EDIT_COUNT_TIMER.cx;
   int ofs =  w/2 - x/2;
   if (ofs > 10) ofs = 10;
   if (ofs < 10) x -= 3;
   m_EditMines.MoveWindow(
      ofs,
      size.cy/2 - EDIT_COUNT_TIMER.cy/2,
      x,
      EDIT_COUNT_TIMER.cy,
      TRUE); //
   m_EditTime.MoveWindow(
      size.cx - x - ofs,
      size.cy/2 - EDIT_COUNT_TIMER.cy/2,
      x,
      EDIT_COUNT_TIMER.cy,
      TRUE); //

   //SetMsgHandled(FALSE);
}

BOOL CToolbarFM::OnEraseBkgnd(HDC hdc) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE;
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hdc, GetFM2WinMain().GetSkin().GetBkColor());
}

void CToolbarFM::SetFont(const LOGFONT &font) {
   RECT Rect = ::GetClientRect(m_EditMines.m_hWnd);
   LOGFONT logFont = font;
   logFont.lfHeight = Rect.bottom;

   ::DeleteObject(m_EditMines.GetFont()); //    DeleteObject  . ,      
   ::DeleteObject(m_EditTime .GetFont()); //  --//--
   m_EditMines.SetFont(::CreateFontIndirect(&logFont));
   m_EditTime .SetFont(::CreateFontIndirect(&logFont));
}

#endif // _WTL_VER
