////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Image.cpp"
//
//   CImage -     
//
// Copyright (C) 2003-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Image.h"

////////////////////////////////////////////////////////////////////////////////
//                            types & constants
////////////////////////////////////////////////////////////////////////////////

#define HIMETRIC_INCH   2540

#ifndef INVALID_SIZE
#define INVALID_SIZE SIZEEX(-1,-1)
#endif

////////////////////////////////////////////////////////////////////////////////
//                            other function
////////////////////////////////////////////////////////////////////////////////

SIZE PictureSize(IPicture *pPicture, bool bPixel) {
   SIZE size = {0,0};
   HRESULT hRes = S_OK;
   if (!pPicture) {
      hRes = E_INVALIDARG;
   } else {
      OLE_YSIZE_HIMETRIC hmHeight = 0;
      hRes = pPicture->get_Height(&hmHeight);
      if (SUCCEEDED(hRes)) {
         OLE_XSIZE_HIMETRIC hmWidth = 0;
         hRes = pPicture->get_Width(&hmWidth);
         if (SUCCEEDED(hRes)) {
            if (bPixel) {
               HDC hDC = ::GetDC(::GetDesktopWindow());
               size.cx = ::MulDiv(hmWidth , ::GetDeviceCaps(hDC, LOGPIXELSX), HIMETRIC_INCH);
               size.cy = ::MulDiv(hmHeight, ::GetDeviceCaps(hDC, LOGPIXELSY), HIMETRIC_INCH);
               ::ReleaseDC(::GetDesktopWindow(), hDC);
            } else {
               size.cx = hmWidth;
               size.cy = hmHeight;
            }
         }
      }
   }
   ::SetLastError(hRes);
   return size;
}

HRESULT LoadPictFromFile(OUT IPicture **ppPicture, IN HGLOBAL hGlobal, IN DWORD dwSize) {
   HRESULT hRes = S_OK;
   if (!ppPicture || !hGlobal || !dwSize) {
      hRes = E_INVALIDARG;
   } else {
      *ppPicture = NULL;
   }

   LPSTREAM pStm = NULL;
   if (SUCCEEDED(hRes)) {
      // create IStream* from global memory
      hRes = ::CreateStreamOnHGlobal(hGlobal, TRUE, &pStm);
      if (SUCCEEDED(hRes) && !pStm) {
         hRes = E_FAIL;
      }
   }

   if (SUCCEEDED(hRes)) {
      // Create IPicture from image file
      hRes = ::OleLoadPicture(pStm, dwSize, FALSE, IID_IPicture, (LPVOID*)ppPicture);
      if (SUCCEEDED(hRes) && (*ppPicture == NULL)) {
         hRes = E_FAIL;
      }
   }

   // free data
   if (pStm) {
      pStm->Release(); pStm = NULL;
   }

   ::SetLastError(hRes);

   return hRes;
}

HRESULT LoadPictFromFile(OUT IPicture **ppPicture, IN LPCTSTR szFile) {
   HRESULT hRes = S_OK;
   if (!ppPicture || !szFile || !szFile[0]) {
      hRes = E_INVALIDARG;
   } else {
      *ppPicture = NULL;
   }

   HANDLE hFile = INVALID_HANDLE_VALUE;
   DWORD dwFileSize = 0;
   if (SUCCEEDED(hRes)) {
      hFile = ::CreateFile(szFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
      if (hFile == INVALID_HANDLE_VALUE) {
         hRes = ::GetLastError();
      } else {
         dwFileSize = ::GetFileSize(hFile, NULL);
         if (dwFileSize == 0xFFFFFFFF) {
            hRes = ::GetLastError();
         }
      }
   }

   HGLOBAL hGlobal = NULL;
   LPVOID pData = NULL;
   if (SUCCEEDED(hRes)) {
      // alloc memory based on file size
      hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, dwFileSize);
      if (!hGlobal) {
         hRes = ::GetLastError();
      } else {
         pData = ::GlobalLock(hGlobal);
         if (!pData) {
            hRes = ::GetLastError();
         }
      }
   }

   DWORD dwNBOR = 0;
   if (SUCCEEDED(hRes)) {
      // read file and store in global memory
      if (!::ReadFile(hFile, pData, dwFileSize, &dwNBOR, NULL) || (dwFileSize!=dwNBOR)) {
         hRes = ::GetLastError();
      }
   }
   
   if (SUCCEEDED(hRes)) {
      hRes = LoadPictFromFile(ppPicture, hGlobal, dwFileSize);
   }

   // free data
   if (hGlobal) {
      ::GlobalUnlock(hGlobal);
      ::GlobalFree(hGlobal);
   }
   if (hFile != INVALID_HANDLE_VALUE) {
      ::CloseHandle(hFile);
   }

   ::SetLastError(hRes);

   return hRes;
}

//    HDC   
#define IsMemoryHDC(hDC) !::WindowFromDC(hDC)
/** /
BOOL IsMemoryHDC_(HDC hDC) {
   HBITMAP hBmpNil = ::CreateCompatibleBitmap(hDC, 0, 0); // create 1x1-pixel monochrome bitmap
   HGDIOBJ hBmpOld = ::SelectObject(hDC, hBmpNil); // Bitmaps can be selected for memory DCs only, and for only one DC at a time.
   BOOL bRes = ((hBmpOld != NULL) && (hBmpOld != HGDI_ERROR));
   if (bRes) {
      ::SelectObject(hDC, hBmpOld);
   }
   ::DeleteObject(hBmpNil);
   return bRes;
}/**/

////////////////////////////////////////////////////////////////////////////////
//                         implementation CImageEasy
////////////////////////////////////////////////////////////////////////////////

CImageEasy::CImageEasy(LPCTSTR szFileName) : m_pPicture (NULL) {
   if (szFileName) {
      Load(szFileName);
   }
}

BOOL CImageEasy::Load(LPCTSTR szFileName) {
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;
   if (!szFileName || !szFileName[0]) {
      dwErrCode = ERROR_INVALID_PARAMETER;
      bRes = FALSE;
   } else {
      HANDLE hImageNew = NULL;
      IPicture *pPicture = NULL;
      bRes = SUCCEEDED(::LoadPictFromFile(&pPicture, szFileName));
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         Release();
         m_pPicture = pPicture;
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL CImageEasy::Release() {
   BOOL bRes = !!m_pPicture;
   if (m_pPicture) {
      m_pPicture->Release();
      m_pPicture = NULL;
   }
   return bRes;
}

SIZE CImageEasy::GetSizeImage() const {
   return PictureSize(m_pPicture, true);
}

BOOL CImageEasy::Draw(HDC hDC, const RECT &rect) const {
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;
   if (m_pPicture) {
      SIZE sizeH = PictureSize(m_pPicture, false);
      HRESULT hRes = m_pPicture->Render(hDC, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, 0, sizeH.cy, sizeH.cx, -sizeH.cy, &rect);
      if (FAILED(hRes)) {
         dwErrCode = hRes;
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

////////////////////////////////////////////////////////////////////////////////
//                         implementation CImage
////////////////////////////////////////////////////////////////////////////////
#define CImage_INIT \
   m_ImageType(eImageUnk), \
   m_hImage    (NULL), \
   m_Size      (INVALID_SIZE), \
   mm_hMask    (NULL), \
   mm_hCDCMask (NULL), \
   mm_hCDCImage(NULL), \
   mm_hMaskOld (NULL), \
   mm_hImageOld(NULL)

CImage::CImage(LPCTSTR szFileName) : CImage_INIT {
   if (szFileName) {
      Load(szFileName);
   }
}

CImage::CImage(const CString& strFileName) : CImage_INIT {
   if (!strFileName.IsEmpty()) {
      Load(strFileName);
   }
}

CImage::CImage(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType) : CImage_INIT {
   if (szResourceType) {
      Load(szModuleName, szResourceName, szResourceType);
   }
}

CImage::CImage(const CImageSerialize &img) : CImage_INIT {
   if (!img.m_strPath.IsEmpty()) {
      Load(img);
   }
}

BOOL CImage::Load(const CImageSerialize &img) {
   BOOL bRes = FALSE;
   if (!img.m_strPath.IsEmpty()) {
      bRes = CImage::Load((LPCTSTR)img.m_strPath);
      if (bRes) {
         SetTransparent(img.m_bTransparent);
         SetTransparentColor(img.m_clrTransparent);
         SetPlace(img.m_Place);
      }
   }
   return bRes;
}

BOOL CImage::Load(LPCTSTR szFileName) {
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;
   if (!szFileName || !szFileName[0]) {
      dwErrCode = ERROR_INVALID_PARAMETER;
      bRes = FALSE;
   } else {
      CResourceHelper Help(szFileName);
      if (Help.Valid()) {
         bRes = Load(Help.GetModuleName(), Help.GetResourceName(), Help.GetResourceType());
         dwErrCode = ::GetLastError();
      } else {
         HANDLE hImageNew = NULL;
         EImageType imageTypeNew = eImageUnk;
         { // bitmap
            hImageNew = ::LoadImage(NULL, szFileName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR);
            bRes = !!hImageNew;
            if (bRes) {
               imageTypeNew = eImageBMP;
            } else {
               dwErrCode = ::GetLastError();
            }
         }
         if (!bRes) { // icon
            hImageNew = ::LoadImage(NULL, szFileName, IMAGE_ICON, 0, 0, LR_LOADFROMFILE);
            bRes = !!hImageNew;
            if (bRes) {
               imageTypeNew = eImageICO;
            } else {
               dwErrCode = ::GetLastError();
            }
         }
         /** /
         if (!bRes) { // Windows metafile
            hImageNew = GetMetaFile(szFileName);
            HANDLE hFile = ::CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
            if (hFile == INVALID_HANDLE_VALUE) {
               dwErrCode = ::GetLastError();
            } else {
               DWORD dwFS = ::GetFileSize(hFile, NULL);
               if ((dwFS==0) || (dwFS==INVALID_FILE_SIZE)) {
                  dwErrCode = ::GetLastError();
               } else {
                  BYTE *pnbBuf = new BYTE[dwFS];
                  if (pnbBuf == NULL) {
                     dwErrCode = ERROR_OUTOFMEMORY;
                  } else {
                     DWORD dwNOBR = 0;
                     if (!::ReadFile(hFile, pnbBuf, dwFS, &dwNOBR, NULL) || (dwFS != dwNOBR)) {
                        dwErrCode = ::GetLastError();
                     } else {
                        const o = 7;
                        hImageNew = ::SetMetaFileBitsEx(dwFS-o, pnbBuf+o);
                        bRes = !!hImageNew;
                        if (bRes) {
                           imageTypeNew= eImageEMF;
                        } else {
                           dwErrCode = ::GetLastError();
                        }
                     }
                     delete [] pnbBuf; pnbBuf = NULL;
                  }
               }
               ::CloseHandle(hFile);
            }
         }
         /**/
         if (!bRes) { // enhanced-format metafile
            hImageNew = ::GetEnhMetaFile(szFileName);
            bRes = !!hImageNew;
            if (bRes) {
               imageTypeNew = eImageEMF;
            } else {
               dwErrCode = ::GetLastError();
            }
         }

         if (!bRes) {
            bRes = CImageEasy::Load(szFileName);
            if (bRes) {
               imageTypeNew = eImageOTH;
            }  else {
               dwErrCode = ::GetLastError();
            }
         }
         if (bRes) {
            DeleteHandle();
            m_strPath = szFileName;
            m_hImage = hImageNew;
            m_ImageType = imageTypeNew;
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL CImage::Load(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType) {
   HINSTANCE hInstance = NULL;
   BOOL bLoadLib = FALSE;
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;

   CResourceHelper Help(szModuleName, szResourceName, szResourceType);
   if (!Help.Valid()) {
      dwErrCode = ERROR_INVALID_PARAMETER;
      bRes = FALSE;
   }
   if (bRes) {
      if (szModuleName && szModuleName[0]) {
         hInstance = ::GetModuleHandle(szModuleName);
         if (!hInstance) {
            hInstance = ::LoadLibraryEx(szModuleName, NULL, LOAD_LIBRARY_AS_DATAFILE);
            bRes = bLoadLib = !!hInstance;
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
         }
      } else {
         hInstance = ::GetModuleHandle(NULL);
      }
   }
   if (bRes) {
      HANDLE hImageNew = NULL;
      EImageType imageTypeNew = eImageUnk;

      /** /
      if (!IS_INTRESOURCE(szResourceType) && (!szResourceType || !szResourceType[0])) {
         hImageNew = ::LoadBitmap(hInstance, szResourceName);
         dwErrCode = ::GetLastError();
         bRes = (hImageNew != NULL);
         if (bRes) {
            imageTypeNew = eImageBMP;
         }
      } else/**/
      {
         switch ((ULONG_PTR)szResourceType) {
         case RT_BITMAP:
            hImageNew = ::LoadImage(hInstance, szResourceName, IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR);
            bRes = !!hImageNew;
            if (bRes) {
               imageTypeNew = eImageBMP;
            } else {
               dwErrCode = ::GetLastError();
            }
            break;
         case RT_ICON:
         case RT_GROUP_ICON:
            hImageNew = ::LoadImage(hInstance, szResourceName, IMAGE_ICON, 0, 0, 0);
            bRes = !!hImageNew;
            if (bRes) {
               imageTypeNew = eImageICO;
            } else {
               dwErrCode = ::GetLastError();
            }
            break;
         default:
            {
               bRes = FALSE;
               HRSRC hRsrc = ::FindResource(hInstance, szResourceName, szResourceType);
               if (!hRsrc) {
                  dwErrCode = ::GetLastError();
               } else {
                  HGLOBAL hGlobal = ::LoadResource(hInstance, hRsrc);
                  if (!hGlobal) {
                     dwErrCode = ::GetLastError();
                  } else {
                     DWORD dwSize = ::SizeofResource(hInstance, hRsrc);
                     if (dwSize == 0) {
                        dwErrCode = ::GetLastError();
                     } else {
                        try {
                           hImageNew = ::SetEnhMetaFileBits(dwSize, (LPBYTE)::LockResource(hGlobal));
                        } catch(...) {}
                        bRes = !!hImageNew;
                        if (bRes) {
                           imageTypeNew = eImageEMF;
                        } else {
                           //PutTraceResource(hInstance);
                           CString strTempFile = Help.SaveToTempRandomFile();
                           if (strTempFile.IsEmpty()) {
                              dwErrCode = ::GetLastError();
                           } else {
                              bRes = this->Load(strTempFile);
                              if (bRes) {
                                 imageTypeNew = m_ImageType; // save
                                 hImageNew    = m_hImage;    // save

                                 m_ImageType = eImageUnk;
                                 m_hImage    = NULL;
                                 m_strPath.Empty();
                              }  else {
                                 dwErrCode = ::GetLastError();
                              }
                              ::DeleteFile(strTempFile); // del temp file
                           }
                        }
                     }
                  }
               }
            }
         }
      }
      if (bRes) {
         DeleteHandle();
         m_strPath = Help.MakePath();
         m_hImage = hImageNew;
         m_ImageType = imageTypeNew;
      }
   }
   if (bLoadLib) {
      ::FreeLibrary(hInstance); hInstance = NULL;
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

/*
//   ...
BOOL CImage::Load(HBITMAP hBmp, BOOL bTransparent) {
   BOOL bRes = FALSE;
   if (hBmp) {
      BITMAP bmp = {0,0,0,0,0,0,NULL};
      bRes = (sizeof(BITMAP) == ::GetObject(hBmp, sizeof(BITMAP), &bmp));
      if (bRes) {
         m_hImage = hBmp;
         m_ImageType = eImageBMP;
         m_bTransparent = bTransparent;
      }
   }
   return bRes;
}*/

BOOL CImage::DeleteMutable() const {
   BOOL bRes = TRUE;
   if (mm_hCDCImage) {
      ::SelectObject(mm_hCDCImage, mm_hImageOld); mm_hImageOld = NULL;
      bRes &= ::DeleteDC(mm_hCDCImage); mm_hCDCImage = NULL;
   }
   if (mm_hCDCMask) {
      ::SelectObject(mm_hCDCMask, mm_hMaskOld); mm_hMaskOld = NULL; //    
      bRes &= ::DeleteDC(mm_hCDCMask); mm_hCDCMask = NULL;
   }
   if (mm_hMask) {
      bRes &= ::DeleteObject(mm_hMask); mm_hMask = NULL;
   }
   return bRes;
}

BOOL CImage::DeleteHandle() {
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;
   if (Valid()) {
      switch (m_ImageType) {
      case eImageBMP: bRes = ::DeleteObject     (GetHandleBMP()); break;
      case eImageICO: bRes = ::DestroyIcon      (GetHandleICO()); break;
      case eImageWMF: bRes = ::DeleteMetaFile   (GetHandleWMF()); break;
      case eImageEMF: bRes = ::DeleteEnhMetaFile(GetHandleEMF()); break;
      case eImageUnk: bRes = Release(); break;
      }
      if (!bRes)
         dwErrCode = ::GetLastError();
      m_ImageType = eImageUnk;
      m_hImage = NULL;
      m_Size = INVALID_SIZE;
   }
   bRes &= DeleteMutable();
   ::SetLastError(dwErrCode);
   return bRes;
}

SIZE CImage::GetSizeImage() const {
   if (m_Size != INVALID_SIZE) return m_Size;
   SIZE size = INVALID_SIZE;
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = FALSE;
   switch (m_ImageType) {
   case eImageBMP:
      BITMAP bmp;
      bRes = ::GetObject(m_hImage, sizeof(BITMAP), &bmp);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         size.cx = bmp.bmWidth;
         size.cy = bmp.bmHeight;
      }
      break;
   case eImageICO:
      ICONINFO iconInfo;
      bRes = ::GetIconInfo((HICON)m_hImage, &iconInfo);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         size.cx = iconInfo.xHotspot<<1;
         size.cy = iconInfo.yHotspot<<1;
      }
      break;
   case eImageWMF:
      break;
   case eImageEMF:
      ENHMETAHEADER emfHeader;
      bRes = ::GetEnhMetaFileHeader((HENHMETAFILE)m_hImage, sizeof(ENHMETAHEADER), &emfHeader);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         size.cx = emfHeader.rclBounds.right  - emfHeader.rclBounds.left;//emfHeader.szlDevice.cx;
         size.cy = emfHeader.rclBounds.bottom - emfHeader.rclBounds.top; //emfHeader.szlDevice.cy;
      }
      break;
   case eImageOTH:
      size = CImageEasy::GetSizeImage();
      dwErrCode = ::GetLastError();
      break;
   }
   if (bRes) {
      m_Size = size;
   }
   ::SetLastError(dwErrCode);
   return size;
}

#define m_bIsCachedBMP (mm_hCDCMask && mm_hMask && mm_hCDCImage)

BOOL CImage::DrawType(HDC hDC, const RECT &rc) const {
   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;

   switch (m_ImageType) {
   default: 
      dwErrCode = ERROR_FUNCTION_FAILED; // ERROR_CALL_NOT_IMPLEMENTED
      bRes = FALSE;
      break;
   case eImageBMP:
      {
         if (GetTransparent()) {
            /**/
            if (!mm_hMask) {
               mm_hMask = ::CreateMask((HBITMAP)m_hImage, m_clrTransparent);
               mm_hCDCMask = ::CreateCompatibleDC(hDC);
               if (mm_hCDCMask) {
                  mm_hMaskOld = (HBITMAP)::SelectObject(mm_hCDCMask, mm_hMask); //    . 
               }
               mm_hCDCImage = ::CreateCompatibleDC(hDC);
               if (mm_hCDCImage) {
                  mm_hImageOld = (HBITMAP)::SelectObject(mm_hCDCImage, m_hImage);
               }
               //  ,    , ..  . - BitBltMasked/StretchBltMasked,    .   -  .
               //  ,           . .. ,    ...

               if (!m_bIsCachedBMP) {
                  DeleteMutable();
               }
            }
            /**/

            switch (GetPlacePlacement()) {
            case ePlaceAsIs:
            case ePlaceTile:
               if (m_bIsCachedBMP) {
                  bRes = ::BitBltMasked(
                     hDC, rc,
                     mm_hCDCImage, POINTEX(0,0),
                     mm_hCDCMask , POINTEX(0,0));
               } else {
                  bRes = ::BitBltMasked(
                     hDC, rc,
                     GetHandleBMP(), POINTEX(0,0),
                     m_clrTransparent);
               }
               break;
            default:
               if (m_bIsCachedBMP) {
                  bRes = ::StretchBltMasked(
                     hDC, rc,
                     mm_hCDCImage, (POINTEX)GetSizeImage(),
                     mm_hCDCMask , ::SizeBitmap(mm_hMask));
               } else {
                  bRes = ::StretchBltMasked(
                     hDC, rc,
                     GetHandleBMP(), (POINTEX)GetSizeImage(),
                     m_clrTransparent);
               }
            }
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
         } else {
            HDC hCDC = ::CreateCompatibleDC(hDC);
            HGDIOBJ hGObj = ::SelectObject(hCDC, m_hImage);
            switch (GetPlacePlacement()) {
            case ePlaceAsIs:
            case ePlaceTile:
               bRes = ::BitBlt(hDC, rc, hCDC, SRCCOPY);
               break;
            default:
               bRes = ::SetStretchBltMode(hDC, COLORONCOLOR) &&
                      ::StretchBlt(hDC, rc, hCDC, GetSizeImage(), SRCCOPY);
            }
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
            ::SelectObject(hCDC, hGObj);
            ::DeleteDC(hCDC);
         }
      }
      break;
   case eImageICO:
      bRes = ::DrawIconEx(hDC, rc.left, rc.top,
         (HICON)m_hImage,
         rc.right -rc.left,
         rc.bottom-rc.top,
         0, NULL, DI_NORMAL);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      }
      break;
   //case eImageWMF:
   //   break;
   case eImageEMF:
      bRes = ::PlayEnhMetaFile(hDC, (HENHMETAFILE)m_hImage, &rc);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      }
      break;
   case eImageOTH:
      bRes = CImageEasy::Draw(hDC, rc);
      break;
   }

   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL CImage::Draw(HDC hDC, const RECT &rc) const {
   if (!Valid()) {
      ::SetLastError(ERROR_FUNCTION_NOT_CALLED);
      return FALSE;
   }

   BOOL bRes = TRUE;
   DWORD dwErrCode = NO_ERROR;

   RECTEX rcOut; //        

   //   
   switch (GetPlacePlacement()) {
   case ePlaceAsIs      : rcOut = GetSizeImage(); break;
   case ePlaceUniformly : rcOut = ::FindInnerRect((RECTEX)GetSizeImage(), rc); break;
   case ePlaceStretch   : rcOut = rc; break;
   case ePlaceTile      : rcOut = rc; break;
   }

   //  
   switch (GetPlacePlacement()) {
   case ePlaceAsIs      :
   case ePlaceUniformly :
      switch (GetPlaceAlignmentH()) {
      case ePlaceLeft  : rcOut.alignLeft (rc.left); break;
      case ePlaceCenter: rcOut.alignLeft (rc.left+((rc.right-rc.left)>>1)-(rcOut.width()>>1)); break;
      case ePlaceRight : rcOut.alignRight(rc.right); break;
      }
      switch (GetPlaceAlignmentV()) {
      case ePlaceTop   : rcOut.alignTop   (rc.top); break;
      case ePlaceMiddle: rcOut.alignTop   (rc.top+((rc.bottom-rc.top)>>1)-(rcOut.height()>>1)); break;
      case ePlaceBottom: rcOut.alignBottom(rc.bottom); break;
      }
      break;
   }

   BOOL bMem = IsMemoryHDC(hDC);
   BOOL bLimit = FALSE; // TRUE        ,  
                        //       
   switch (GetPlacePlacement()) {
   case ePlaceAsIs:
      bLimit = (rcOut.left < rc.left) || (rcOut.right  > rc.right ) ||
               (rcOut.top  < rc.top ) || (rcOut.bottom > rc.bottom);
      break;
   case ePlaceTile:
      bLimit = true;
   }

   RECTEX rcLimit;
   if (bLimit) {
      ::IntersectRect(&rcLimit, &rcOut, &rc);
   } else {
      if (!bMem) {
         rcLimit = rcOut;
      }
   }
   HDC hCDC = NULL;
   HBITMAP hBmp = NULL, hBmpOld = NULL;
   if (bLimit || !bMem) {
      hCDC = ::CreateCompatibleDC(hDC);
      hBmp = ::CreateCompatibleBitmap(hDC, rcLimit.right, rcLimit.bottom);
      hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
      ::BitBlt(hCDC, rcLimit, hDC, rcLimit.pointLT(), SRCCOPY);
   }

   // 
   switch (GetPlacePlacement()) {
   case ePlaceAsIs      :
   case ePlaceUniformly :
   case ePlaceStretch   :
      bRes =  DrawType(hCDC ? hCDC : hDC, rcOut);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      }
      break;
   case ePlaceTile:
      {
         SIZE size = GetSizeImage();
         rcOut = RECTEX(rc.left, rc.top, rc.left+size.cx, rc.top+size.cy);
         for (; bRes && rcOut.top < rc.bottom; rcOut.moveY(size.cy)) {
            RECTEX rcOut2 = rcOut;
            for (; bRes && rcOut2.left < rc.right; rcOut2.moveX(size.cx)) {
               bRes &= DrawType(hCDC ? hCDC : hDC, rcOut2);
            }
         }
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
      }
   }

   if (bLimit || !bMem) {
      ::BitBlt(hDC, rcLimit, hCDC, rcLimit.pointLT(), SRCCOPY);
      ::SelectObject(hCDC, hBmpOld); hBmpOld = NULL;
      ::DeleteObject(hBmp); hBmp = NULL;
      ::DeleteDC(hCDC); hCDC = NULL;
   }

   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL CImage::DrawRotated(HDC hDC, const RECTEX &rc, float angle, COLORREF clrBack) const {
   DWORD dwErrCode = NO_ERROR;
   HDC hCDC = ::CreateCompatibleDC(hDC); //    ,   -  
   HBITMAP hBmpNew = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
   HGDIOBJ hBmpOld1 = ::SelectObject(hCDC, hBmpNew);
   HBRUSH  hBrNew = ::CreateSolidBrush(clrBack);
   HGDIOBJ hBrOld = ::SelectObject(hCDC, hBrNew);

   if (clrBack == CLR_INVALID) {
      clrBack = ::GetPixel(hDC, rc.left,rc.top);
   }
   BOOL bRes = ::PatBlt(hCDC, 0, 0, rc.width(), rc.height(), PATCOPY); //    , ..       
   bRes &= this->Draw(hCDC, RECTEX(rc.size()));
   ::SelectObject(hCDC, hBmpOld1); //  hBmpNew

   HBITMAP hBmpRot = ::CreateRotatedBitmap(hBmpNew, angle, clrBack);
   bRes &= !!hBmpRot;
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      SIZEEX sizeBmp = ::SizeBitmap(hBmpRot);
   
      ::SelectObject(hCDC, hBmpNew);
      bRes &= ::PatBlt(hCDC, 0, 0, rc.width(), rc.height(), PATCOPY); //   

      HDC hRDC = ::CreateCompatibleDC(hDC); //    
      HGDIOBJ hBmpOld2 = ::SelectObject(hRDC, hBmpRot);
      bRes &= ::BitBlt(hCDC, RECTEX(sizeBmp).alignCenter(rc).move(-rc.left, -rc.top), hRDC, SRCCOPY);
      bRes &= ::BitBlt(hDC, rc, hCDC, SRCCOPY);

      ::SelectObject(hRDC, hBmpOld2);
      ::DeleteDC(hRDC);
   }

   ::SelectObject(hCDC, hBmpOld1);
   ::SelectObject(hCDC, hBrOld);
   ::DeleteDC(hCDC);
   ::DeleteObject(hBrNew);
   ::DeleteObject(hBmpNew);
   ::DeleteObject(hBmpRot);

   return bRes;
}

BOOL CImage::DrawRotatedTransparent(HDC hDC, const RECTEX &rc, float angle, COLORREF clrTransparent) const {
   DWORD dwErrCode = NO_ERROR;
   HDC hCDC = ::CreateCompatibleDC(hDC); //    ,   -  
   HBITMAP hBmpNew = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
   HGDIOBJ hBmpOld1 = ::SelectObject(hCDC, hBmpNew);
   HBRUSH  hBrNew = ::CreateSolidBrush(clrTransparent);
   HGDIOBJ hBrOld = ::SelectObject(hCDC, hBrNew);

   if (clrTransparent == CLR_INVALID) {
      if ((m_ImageType == CImg::eImageBMP) && m_bTransparent)
         clrTransparent = m_clrTransparent;
      if (clrTransparent == CLR_INVALID)
         clrTransparent = ::GetPixel(hDC, rc.left,rc.top);
   }
   BOOL bRes = ::PatBlt(hCDC, 0, 0, rc.width(), rc.height(), PATCOPY); //    , ..       
   bRes &= this->Draw(hCDC, RECTEX(rc.size()));
   ::SelectObject(hCDC, hBmpOld1); //  hBmpNew

   HBITMAP hBmpRot = ::CreateRotatedBitmap(hBmpNew, angle, clrTransparent);
   bRes &= !!hBmpRot;
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      SIZEEX sizeBmp = ::SizeBitmap(hBmpRot);
   
      ::SelectObject(hCDC, hBmpNew);
      bRes &= ::PatBlt(hCDC, 0, 0, rc.width(), rc.height(), PATCOPY); //   

      HDC hRDC = ::CreateCompatibleDC(hDC); //    
      HGDIOBJ hBmpOld2 = ::SelectObject(hRDC, hBmpRot);
      bRes &= ::BitBlt(hCDC, RECTEX(sizeBmp).alignCenter(rc).move(-rc.left, -rc.top), hRDC, SRCCOPY);
      //bRes &= ::BitBltMasked(hDC, rc, hCDC, POINTEX(), clrTransparent);
      ::SelectObject(hCDC, hBmpOld1);
      bRes &= ::BitBltMasked(hDC, rc, hBmpNew, POINTEX(), clrTransparent);

      ::SelectObject(hRDC, hBmpOld2);
      ::DeleteDC(hRDC);
   }

   ::SelectObject(hCDC, hBrOld);
   ::DeleteDC(hCDC);
   ::DeleteObject(hBrNew);
   ::DeleteObject(hBmpNew);
   ::DeleteObject(hBmpRot);

   ::SetLastError(dwErrCode);
   return bRes;
}

////////////////////////////////////////////////////////////////////////////////
//                         implementation CImage::CResourceHelper
////////////////////////////////////////////////////////////////////////////////

CImage::CResourceHelper::CResourceHelper(LPCTSTR szFileName) :
   m_iResourceType(0),
   m_iResourceName(0)
{
   LPTSTR szModuleName = _tcsdup(szFileName);
   if (szModuleName) {
      TCHAR *pchT = _tcschr(szModuleName, _T('?'));
      if (pchT) {
         TCHAR *pchN = _tcschr(pchT, _T('#'));
         if (pchN) {
            *pchT = *pchN = 0;
            m_strModuleName = szModuleName;
            m_iResourceType = _ttoi(pchT+1);
            m_iResourceName = _ttoi(pchN+1);
          //int iResT = _stscanf(pchT+1, _T("%i"), &m_iResourceType); // iResT must == 1 if converted
          //int iResN = _stscanf(pchN+1, _T("%i"), &m_iResourceName); // iResN must == 1 if converted
            if (m_iResourceType==0)
               m_strResourceType = pchT+1;
            if (m_iResourceName==0)
               m_strResourceName = pchN+1;
         }
      }
      ::free(szModuleName); szModuleName = NULL;
   }
}

CImage::CResourceHelper::CResourceHelper(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType) :
   m_iResourceType(0),
   m_iResourceName(0)
{
   m_strModuleName = szModuleName;
   if (IS_INTRESOURCE(szResourceType)) m_iResourceType = (int)szResourceType;
                                else m_strResourceType =      szResourceType;
   if (IS_INTRESOURCE(szResourceName)) m_iResourceName = (int)szResourceName;
                                else m_strResourceName =      szResourceName;
}

CString CImage::CResourceHelper::SaveToTempRandomFile(LPCTSTR szFilePrefix) const {
   CString strTempFile = ::GetTempPath() +
                         (szFilePrefix ? szFilePrefix : ::GUID2String(::CoCreateGuid()))+_T('_')+
                         GetResourceNameAsString();
   if (m_iResourceType) {
      switch ((ULONG_PTR)m_iResourceType) {
      case RT_BITMAP    : strTempFile += _T(".bmp"); break;
      case RT_ICON      :
      case RT_GROUP_ICON: strTempFile += _T(".ico"); break;
      default           : strTempFile += _T(".image"); break;
      }
   } else {
      strTempFile +=  _T('.')+m_strResourceType;
   }
   /** /
   LOG_PUT(g_pLog, Put, LL_DEBUG) _T("ResourceName-[%s] ResourceType-[%s] fileName-[%s]"),
      (LPCTSTR)GetResourceTypeAsString(),
      (LPCTSTR)GetResourceNameAsString(),
      (LPCTSTR)strTempFile);
   /**/
   HINSTANCE hInstance = ::LoadLibraryEx(m_strModuleName, NULL, LOAD_LIBRARY_AS_DATAFILE);
   BOOL bRes = ::SaveResourceAsFile(
      hInstance,
      strTempFile,
      GetResourceName(), GetResourceType(), TRUE);
   DWORD dwErrCode = NO_ERROR;
   if (!bRes) {
      dwErrCode = ::GetLastError();
      ::DeleteFile(strTempFile); // del temp file
      strTempFile.Empty();
   }
   ::FreeLibrary(hInstance);
   ::SetLastError(dwErrCode);
   return strTempFile;
}
