////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Lang.cpp"
//
//   
// Copyright (C) 2004-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
//
// 2007.03.01
//    Add identifier  -  IDS__DIALOG_CHANGESKIN__TIMER_PAUSE
//
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Lang.h"
#include "./Lib/CommonLib.h"
#include "./Resource.h"
#include "./Image.h"
#include "./ODItems/MenuItem.h"

////////////////////////////////////////////////////////////////////////////////
//                               implementetion
////////////////////////////////////////////////////////////////////////////////
BOOL CLang::Load(LPCTSTR szFileName) {
   CString strPath(GetModuleDir(::GetModuleHandle(NULL)) + _T("Language\\") + szFileName);
   HANDLE hFile = ::CreateFile(
      strPath,
      GENERIC_READ,
      FILE_SHARE_READ,
      NULL,
      OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL,
      NULL
   );
   BOOL bRes = (hFile != INVALID_HANDLE_VALUE);
   if (bRes) {
      DWORD dwNBR = 0;
      BYTE pbSignature[2] = {0,0};
      bRes = ::ReadFile(hFile, pbSignature, 2, &dwNBR, NULL) && (dwNBR==2);
      if (bRes) {
         bool bUnicodeFile = (pbSignature[0]==0xFF) && (pbSignature[1]==0xFE);
         if (!bUnicodeFile) {
            DWORD dwRes = ::SetFilePointer(hFile, 0,0, FILE_BEGIN);
            bRes = (dwRes != INVALID_SET_FILE_POINTER);
         }
         if (bRes) {
            bRes = Parse(hFile, bUnicodeFile);
         }
      }
      ::CloseHandle(hFile); hFile = NULL;
   }
   Finalisation();
   return bRes;
}

CString VerifyLine(const CString &str) {
   CString strRes(str);
   int len = str.GetLength();
   int ofs = 0;
   for (int i=0; ((i+ofs)<len); i++) {
      if (str[i+ofs] != _T('\\')) {
         strRes.SetAt(i, str[i+ofs]);
      } else {
         ofs++;
         TCHAR ch = str[i+ofs];
         switch (ch) {
         case _T('n'):   strRes.SetAt(i, _T('\n'));    break;
         case _T('r'):   strRes.SetAt(i, _T('\r'));    break;
         case _T('t'):   strRes.SetAt(i, _T('\t'));    break;
         case _T('0'):   strRes.SetAt(i, _T('\0'));    break;
         default:
            strRes.SetAt(i  , _T('\\'));
            strRes.SetAt(i+1, ch);
         }
      }
   }
   if (ofs) {
      strRes.SetAt(len-ofs, 0);
   }

   return strRes;
}

bool CLang::ReadLine(HANDLE hFile, bool bUnicode) {
   if (bUnicode) {
      static const WCHAR szEOL[3] = {13, 10, 0};
      DWORD dwNBR;
      const DWORD sizeBuff = 1023;
      WCHAR szBuf[sizeBuff+1];
      BOOL bRes = ::ReadFile(hFile, szBuf, sizeBuff<<(sizeof(WCHAR)-1), &dwNBR, NULL);
      if (!bRes || (dwNBR < 1)) return false;
      szBuf[sizeBuff] = 0;
      WCHAR *szFind = wcsstr(szBuf, szEOL);
      if (!szFind)
         return false;
      DWORD dwRes = ::SetFilePointer(hFile, (((szFind+2-szBuf)<<(sizeof(WCHAR)-1))-dwNBR), NULL, FILE_CURRENT);
      if (dwRes == INVALID_SET_FILE_POINTER)
         return false;

      *szFind = 0;
      {
         szFind = wcschr(szBuf, L'=');
         if (!szFind)
            return true;
         *szFind = 0;

         int i, iRes = 
               #if (_MSC_VER >= 1400)
                      swscanf_s
               #else
                      swscanf
               #endif
                                 (szBuf, L"%d", &i);
         if ((iRes == 0) || (iRes == EOF)) {
            return true;
         }
         CString str = szFind+1;

         m_StrArr()[i] = VerifyLine(str);
      }
   } else {
      static const CHAR szEOL[3] = {13, 10, 0};
      DWORD dwNBR;
      const DWORD sizeBuff = 1023;
      CHAR szBuf[sizeBuff+1];
      BOOL bRes = ::ReadFile(hFile, szBuf, sizeBuff<<(sizeof(CHAR)-1), &dwNBR, NULL);
      if (!bRes || (dwNBR < 1)) return false;
      szBuf[sizeBuff] = 0;
      CHAR *szFind = strstr(szBuf, szEOL);
      if (!szFind)
         return false;
      DWORD dwRes = ::SetFilePointer(hFile, (((szFind+2-szBuf)<<(sizeof(CHAR)-1))-dwNBR), NULL, FILE_CURRENT);
      if (dwRes == INVALID_SET_FILE_POINTER)
         return false;

      *szFind = 0;
      {
         szFind = strchr(szBuf, '=');
         if (!szFind)
            return true;
         *szFind = 0;

         int i, iRes = 
               #if (_MSC_VER >= 1400)
                      sscanf_s
               #else
                      sscanf
               #endif
                                 (szBuf, "%d", &i);
         if ((iRes == 0) || (iRes == EOF)) {
            return true;
         }
         CString str = szFind+1;

         m_StrArr()[i] = VerifyLine(str);
      }
   }
   return true;
}

BOOL CLang::Parse(HANDLE hFile, bool bUnicode) {
   BOOL bRes = FALSE;
   do {
      bRes = ReadLine(hFile, bUnicode);
   } while (bRes);
   return bRes;
}


bool CLang::RestoreDefaultLanguage() {
   m_StrArr()[IDS__LOGO] = _T("FastMines");

   m_StrArr()[IDS__OK         ] = _T("Ok");
   m_StrArr()[IDS__CANCEL     ] = _T("Cancel");
   m_StrArr()[IDS__ERROR      ] = _T("Error");
   m_StrArr()[IDS__INFORMATION] = _T("Information");
   m_StrArr()[IDS__SEC        ] = _T("sec.");
   m_StrArr()[IDS__SECOND     ] = _T("seconds");
   m_StrArr()[IDS__MILISECOND ] = _T("milliseconds");
   m_StrArr()[IDS__MAX        ] = _T("max");

   m_StrArr()[IDS__MINES  ] = _T("Mines");
   m_StrArr()[IDS__TIME   ] = _T("Time");
   m_StrArr()[IDS__VICTORY] = _T("Victory");
   m_StrArr()[IDS__DEFEAT ] = _T("Defeat");

   m_StrArr()[IDS__LOAD_FASTMINES_GAME      ] = _T("Load FastMines Game");
   m_StrArr()[IDS__SAVE_FASTMINES_GAME      ] = _T("Save FastMines Game");
   m_StrArr()[IDS__FASTMINES_EXTENSIONS_FILE] = _T("FastMines file");
   m_StrArr()[IDS__FMS_FILE__ERROR_CREATE   ] = _T("Can't create FMS file");
   m_StrArr()[IDS__FMS_FILE__ERROR_OPEN     ] = _T("Can't open FMS file");
   m_StrArr()[IDS__FMS_FILE__ERROR_READ     ] = _T("Can't read FMS file");
   m_StrArr()[IDS__FMS_FILE__ERROR_WRITE    ] = _T("Can't write FMS file");
   m_StrArr()[IDS__FMS_FILE__ERROR_VERSION  ] = _T("FMS file - version error");

   m_StrArr()[IDS__RESTORE_LAST_GAME] = _T("Restore last game?");
   m_StrArr()[IDS__QUESTION         ] = _T("Question");

   m_StrArr()[IDS__MOSAIC_NAME_00     ] = _T("Triangle 60-60-60");
   m_StrArr()[IDS__MOSAIC_NAME_01     ] = _T("Triangle 60-60-60 (offset)");
   m_StrArr()[IDS__MOSAIC_NAME_02     ] = _T("Triangle 45-90-45");
   m_StrArr()[IDS__MOSAIC_NAME_03     ] = _T("Triangle 30-30-120");
   m_StrArr()[IDS__MOSAIC_NAME_04     ] = _T("Square 1");
   m_StrArr()[IDS__MOSAIC_NAME_05     ] = _T("Square 2 (offset)");
   m_StrArr()[IDS__MOSAIC_NAME_06     ] = _T("Rectangle 1 (Parquet 'Herring-bone')");
   m_StrArr()[IDS__MOSAIC_NAME_07     ] = _T("Rectangle 2");
   m_StrArr()[IDS__MOSAIC_NAME_08     ] = _T("Trapezoid 1");
   m_StrArr()[IDS__MOSAIC_NAME_09     ] = _T("Trapezoid 2");
   m_StrArr()[IDS__MOSAIC_NAME_10     ] = _T("Trapezoid 3");
   m_StrArr()[IDS__MOSAIC_NAME_11     ] = _T("Rhombus");
   m_StrArr()[IDS__MOSAIC_NAME_12     ] = _T("Quadrilateral 120-90-60-90");
   m_StrArr()[IDS__MOSAIC_NAME_13     ] = _T("Pentagon (type 2 and 4)");
   m_StrArr()[IDS__MOSAIC_NAME_14     ] = _T("Pentagon (type 5)");
   m_StrArr()[IDS__MOSAIC_NAME_15     ] = _T("Pentagon (type 10)");
   m_StrArr()[IDS__MOSAIC_NAME_16     ] = _T("Hexagon");
   m_StrArr()[IDS__MOSAIC_NAME_17     ] = _T("Square-Triangle 1");
   m_StrArr()[IDS__MOSAIC_NAME_18     ] = _T("Square-Triangle 2");
   m_StrArr()[IDS__MOSAIC_NAME_19     ] = _T("Square-Triangle-Hexagon");
   m_StrArr()[IDS__MOSAIC_NAME_INVALID] = _T("!!! Invalid name !!!");

   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL                           ] = _T("Custom skill level");
   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL__X_WIDTH                  ] = _T("Size X (width) = ");
   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL__Y_HEIGHT                 ] = _T("Size Y (height) = ");
   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL__NUMBER_MINES             ] = _T("Number of Mines = ");
   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__CURRENT_SIZE] = _T("Full Screen\n( current cell size )");
   m_StrArr()[IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__MINIMAL_SIZE] = _T("Full Screen\n( minimal cell size )");

   m_StrArr()[IDS__DIALOG_ABOUT           ] = _T("About");
   m_StrArr()[IDS__DIALOG_ABOUT__VERSION  ] = _T("Version");
   m_StrArr()[IDS__DIALOG_ABOUT__COPYRIGHT] = _T("Copyright");
   m_StrArr()[IDS__DIALOG_ABOUT__COMMEMTS ] = _T("Comments - free software, open source (GPL)");
   m_StrArr()[IDS__DIALOG_ABOUT__RESPONSE ] = _T("Comments: e-mail");
   m_StrArr()[IDS__DIALOG_ABOUT__SITE     ] = _T("FastMines Web page:");

   m_StrArr()[IDS__DIALOG_ADMINISTRATION                        ] = _T("Player Administration");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__SELECT                ] = _T("Select Player");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__NEWPLAYER             ] = _T("New Player");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__NEWPASSWORD           ] = _T("New Password");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__AVATAR                ] = _T("Avatar");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__RENAME                ] = _T("Rename");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__REMOVE                ] = _T("Remove");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__AUTOSTART             ] = _T("Window autoloading");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__TABLE                 ] = _T("Players");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__ENTER_PASSWORD        ] = _T("Enter password");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__ENTER_NAME            ] = _T("Enter name:");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__ENTER_NEW_NAME        ] = _T("Enter new Name");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__NEW_NAME              ] = _T("New name:");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__CANT_USE_VIRT_NAME    ] = _T("Sorry, this name is used by a virtual!");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__INVALID_NAME          ] = _T("Invalid name");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__PASSWORD_NOT_CONFIRMED] = _T("Password not confirmed!");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__THIS_VIRTUAL_PLAYER   ] = _T("Sorry, this is a virtual player!");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__INCORRECT_PASSWORD    ] = _T("Incorrect password");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__PASSWORD              ] = _T("Password:");
   m_StrArr()[IDS__DIALOG_ADMINISTRATION__CONFIRMATION          ] = _T("Confirmation:");

   m_StrArr()[IDS__DIALOG_CHANGESKIN                                ] = _T("Change skin");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_TRANSPARENT            ] = _T("Transparent");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_IMAGE                  ] = _T("Image...");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_FONT                   ] = _T("Font...");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_APPLY_TO_ALL           ] = _T("Apply to all windows");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_COLOR                  ] = _T("Color...");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_LIGHT_COLOR            ] = _T("Light  color...");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__BUTTON_SHADOW_COLOR           ] = _T("Shadow color...");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__OPENFILENAME_FILTER_ALL_IMAGES] = _T("Images");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__OPENFILENAME_TITLE_OPEN_IMAGE ] = _T("Open image");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_IMAGE_MINE        ] = _T("'Mine' image");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_IMAGE_FLAG        ] = _T("'Flag' image");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_IMAGE_PAUSE       ] = _T("'Pause' image");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_BUTTON_PAUSE      ] = _T("'Pause' button");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_BUTTON_GAME       ] = _T("'Game' button");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_IMAGE_BACKGROUND  ] = _T("Background image");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_COLOR_BACKGROUND  ] = _T("Background color");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_BORDER_COLOR_WIDTH] = _T("Border color/width");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_CHARACTER_COLOR   ] = _T("Character Colors");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__CHECKBUTTON_FONT_TYPE         ] = _T("Font Type");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__RESET_TO_DEFAULT              ] = _T("Reset to default");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE_PLACEMENT               ] = _T("Placement");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE_ALIGN                   ] = _T("Align");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE__AS_IS                  ] = _T("As is");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE__UNIFORMLY              ] = _T("Uniformly");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE__STRETCH                ] = _T("Stretch");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__PLACE__TILE                   ] = _T("Tile");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__WIDTH_BORDER                  ] = _T("Width =");
   m_StrArr()[IDS__DIALOG_CHANGESKIN__TIMER_PAUSE                   ] = _T("Image change frequency (sec.)");

   m_StrArr()[IDS__DIALOG_INPUTTEXT_CAPTION       ] = _T("Save Skin");
   m_StrArr()[IDS__DIALOG_INPUTTEXT_STATIC_TEXT   ] = _T("File name");
   m_StrArr()[IDS__FILESKIN__OVERWRITE            ] = _T("File exists. Overwrite?");
   m_StrArr()[IDS__FILESKIN__FILE_COPYING         ] = _T("File copying");
   m_StrArr()[IDS__FILESKIN__ERROR_CREATE         ] = _T("Can't create file");
   m_StrArr()[IDS__FILESKIN__ERROR_OPEN           ] = _T("Can't open file");
   m_StrArr()[IDS__FILESKIN__ERROR_READ           ] = _T("Can't load file");
   m_StrArr()[IDS__FILESKIN__ERROR_WRITE          ] = _T("Can't write file");
   m_StrArr()[IDS__FILESKIN__ERROR_VERSION        ] = _T("SKN file - version error");
   m_StrArr()[IDS__FILESKIN__RESTORED_DEFAULT_SKIN] = _T("The default skin has been restored");

   m_StrArr()[IDS__DIALOG_ASSISTANT                    ] = _T("Assistant Options");
   m_StrArr()[IDS__DIALOG_ASSISTANT__TIMEOUT_USER_FIRST] = _T("Launch after\nif no response from user");
   m_StrArr()[IDS__DIALOG_ASSISTANT__TIMEOUT_NEXT      ] = _T("Assistant work frequency:");
   m_StrArr()[IDS__DIALOG_ASSISTANT__NEW_GAME_AUTOSTART] = _T("Launch new game automatically");
   m_StrArr()[IDS__DIALOG_ASSISTANT__STOPWORK          ] = _T("Stop if there are no definitely safe moves");
   m_StrArr()[IDS__DIALOG_ASSISTANT__IGNOREPAUSE       ] = _T("Ignore pause");

   m_StrArr()[IDS__DIALOG_SELECTMOSAIC        ] = _T("Select mosaic");
   m_StrArr()[IDS__DIALOG_SELECTMOSAIC__NUMBER] = _T("Mosaic number:");
   m_StrArr()[IDS__DIALOG_SELECTMOSAIC__NAME  ] = _T("Mosaic name:");

   m_StrArr()[IDS__CHAMPIONS                  ] = _T("Champions");
   m_StrArr()[IDS__CHAMPIONS__PLAYER_NAME     ] = _T("Player name");
   m_StrArr()[IDS__CHAMPIONS__GAME_TIME       ] = _T("Game time");
   m_StrArr()[IDS__CHAMPIONS__ERROR_CREATE    ] = _T("Can't create Champions file");
   m_StrArr()[IDS__CHAMPIONS__ERROR_OPEN      ] = _T("Can't open Champions file");
   m_StrArr()[IDS__CHAMPIONS__ERROR_READ      ] = _T("Can't load Champions file");
   m_StrArr()[IDS__CHAMPIONS__ERROR_WRITE     ] = _T("Can't write Champions file");
   m_StrArr()[IDS__CHAMPIONS__ERROR_VERSION   ] = _T("BST file  version error");

   m_StrArr()[IDS__PLAYERSandSTATISTIC                        ] = _T("Statistics");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__PLAYER_NAME           ] = _T("Player name");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__NUMBER__GAMES         ] = _T("Number of games");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__GAMES_WON             ] = _T("Games win");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__OPEN                  ] = _T("Open");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__AVERAGED_GAME_TIME    ] = _T("Averaged game time");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__AVERAGED_NUMBER_CLICKS] = _T("Averaged number of clicks");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__ERROR_CREATE          ] = _T("Can't create Statistics file");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__ERROR_OPEN            ] = _T("Can't open Statistics file");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__ERROR_READ            ] = _T("Can't load Statistics file");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__ERROR_WRITE           ] = _T("Can't write Statistics file");
   m_StrArr()[IDS__PLAYERSandSTATISTIC__ERROR_VERSION         ] = _T("STC file - version error");

   m_StrArr()[IDS__MENU_GAME              ] = _T("&Game");
   m_StrArr()[IDS__MENU_GAME__NEW_GAME    ] = _T("&New Game");
   m_StrArr()[IDS__MENU_GAME__BEGINNER    ] = _T("&Beginner");
   m_StrArr()[IDS__MENU_GAME__AMATEUR     ] = _T("&Amateur");
   m_StrArr()[IDS__MENU_GAME__PROFESSIONAL] = _T("&Professional");
   m_StrArr()[IDS__MENU_GAME__CRAZY       ] = _T("C&razy");
   m_StrArr()[IDS__MENU_GAME__CUSTOM      ] = _T("&Custom...");
   m_StrArr()[IDS__MENU_GAME__CREATE      ] = _T("Create Game");
   m_StrArr()[IDS__MENU_GAME__SAVE        ] = _T("Save Game");
   m_StrArr()[IDS__MENU_GAME__LOAD        ] = _T("Open Game");
   m_StrArr()[IDS__MENU_GAME__PLAYERS     ] = _T("&Players...");
   m_StrArr()[IDS__MENU_GAME__EXIT        ] = _T("&Exit");

   m_StrArr()[IDS__MENU_MOSAIC                ] = _T("&Mosaics");
   m_StrArr()[IDS__MENU_MOSAIC__TRIANGLES     ] = _T("&Triangle");
   m_StrArr()[IDS__MENU_MOSAIC__QUADRILATERALS] = _T("&Quadrilaterals");
   m_StrArr()[IDS__MENU_MOSAIC__PENTAGONS     ] = _T("&Pentagons");
   m_StrArr()[IDS__MENU_MOSAIC__HEXAGONS      ] = _T("&Hexagons");
   m_StrArr()[IDS__MENU_MOSAIC__OTHER         ] = _T("&Other");

   m_StrArr()[IDS__MENU_OPTIONS                    ] = _T("&Options");
   m_StrArr()[IDS__MENU_OPTIONS__ALWAYSMAXSIZE     ] = _T("Always maximal size");
   m_StrArr()[IDS__MENU_OPTIONS__AREA_MAX          ] = _T("Maximal mosaic size");
   m_StrArr()[IDS__MENU_OPTIONS__AREA_MIN          ] = _T("Minimal mosaic size");
   m_StrArr()[IDS__MENU_OPTIONS__AREA_INCREMENT    ] = _T("Increase mosaic size");
   m_StrArr()[IDS__MENU_OPTIONS__AREA_DECREMENT    ] = _T("Decrease mosaic size");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN              ] = _T("&Skin");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN__LOAD        ] = _T("&Load");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN__LOAD_DEFAULT] = _T("Default");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN__SAVE        ] = _T("&Save");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN__CHANGE      ] = _T("&Change");
   m_StrArr()[IDS__MENU_OPTIONS__SKIN__WEB_PAGE    ] = _T("Skin WEB page");
   m_StrArr()[IDS__MENU_OPTIONS__LANGUAGE          ] = _T("Language");
   m_StrArr()[IDS__MENU_OPTIONS__LANGUAGE_ENGLISH  ] = _T("English");
   m_StrArr()[IDS__MENU_OPTIONS__USE_UNKNOWN       ] = _T("&Use '?'");
   m_StrArr()[IDS__MENU_OPTIONS__SHOW_TOOLBAR      ] = _T("&Show toolbar");
   m_StrArr()[IDS__MENU_OPTIONS__SHOW_MENU         ] = _T("&Show menu");
   m_StrArr()[IDS__MENU_OPTIONS__SHOW_CAPTION      ] = _T("&Show caption");
   m_StrArr()[IDS__MENU_OPTIONS__TO_TRAY           ] = _T("Minimize to tray");

   m_StrArr()[IDS__MENU_HELP                    ] = _T("&Help");
   m_StrArr()[IDS__MENU_HELP__CHAMPIONS         ] = _T("&Champions");
   m_StrArr()[IDS__MENU_HELP__STATISTICS        ] = _T("&Statistics");
   m_StrArr()[IDS__MENU_HELP__ASSISTANT         ] = _T("&Assistant");
   m_StrArr()[IDS__MENU_HELP__ASSISTANT__ONOFF  ] = _T("Assistant on");
   m_StrArr()[IDS__MENU_HELP__ASSISTANT__OPTIONS] = _T("Options...");
   m_StrArr()[IDS__MENU_HELP__ASSISTANT__SUGGEST] = _T("Suggest left click");
   m_StrArr()[IDS__MENU_HELP__ABOUT             ] = _T("&About");

   m_StrArr()[IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_IMAGE   ] = _T("Image");
   m_StrArr()[IDS__TEMPLATEDLG_SELECTIMAGE__BORDER_RESOURCE] = _T("Resource");

   m_StrArr()[IDS__INI_FILE__ERROR_CREATE ] = _T("Can't create INI file");
   m_StrArr()[IDS__INI_FILE__ERROR_OPEN   ] = _T("Can't open INI file");
   m_StrArr()[IDS__INI_FILE__ERROR_READ   ] = _T("Can't read data from the INI file");
   m_StrArr()[IDS__INI_FILE__ERROR_WRITE  ] = _T("Can not write data to the INI file");
   m_StrArr()[IDS__INI_FILE__ERROR_VERSION] = _T("INI file - version error");

   m_StrArr()[IDS__WEB_PAGE__FASTMINES     ] = _T("http://kserg77.narod.ru/FastMines_eng.html");
   m_StrArr()[IDS__WEB_PAGE__FASTMINES_SKIN] = _T("http://kserg77.narod.ru/FastMinesSkin.html");

   m_StrArr()[IDS__CAPTION_BUTTON_MENU] = _T("Menu");

   Finalisation();
   //SaveLanguage(_T("english_default.lng"));

   return true;
}

void CLang::SaveLanguage(LPCTSTR szFileName) {
   CString strPath(GetModuleDir(::GetModuleHandle(NULL)) + szFileName);
   HANDLE hFile = ::CreateFile(
      strPath,
      GENERIC_WRITE,
      FILE_SHARE_READ,
      NULL,
      CREATE_ALWAYS,
      FILE_ATTRIBUTE_NORMAL,
      NULL
   );
   if (hFile != INVALID_HANDLE_VALUE) {
      int nOld = 0;
      for (CLang::MAP_IntStr::const_iterator i=CLang::m_StrArr().begin(); i!=CLang::m_StrArr().end(); i++) {
         DWORD dwNOBW;
         if (abs(i->first - nOld) > 1) {
            WriteFile(hFile, _T("\r\n"), 2<<(sizeof(TCHAR)-1), &dwNOBW, NULL);
         }
         CString strResult(i->second);
         strResult.Replace(_T("\n"), _T("\\n"));
         strResult.Replace(_T("\t"), _T("\\t"));
         strResult.Replace(_T("\r"), _T("\\r"));
         CString strLineToFile;
         strLineToFile.Format(_T("%i=%s\r\n"), i->first, (LPCTSTR)strResult);
         int iByteSize = strLineToFile.GetLength()<<(sizeof(TCHAR)-1);
         if (!::WriteFile(hFile, strLineToFile, iByteSize, &dwNOBW, NULL) ||
             (iByteSize != dwNOBW)
            )
         {
            break;
         }
         nOld = i->first;
      }
      ::CloseHandle(hFile);
   }
}

void CLang::Finalisation() {
   WORD wVerMSHigh = 0;
   ::GetFileVersion(::GetModuleFileName(), &wVerMSHigh);
   if (GetString(IDS__LOGO).Find(_T(' ')) == -1) {
      m_StrArr()[IDS__LOGO_VERS] = m_StrArr()[IDS__LOGO]+::Format(_T("%i"), int(wVerMSHigh));
   } else {
      m_StrArr()[IDS__LOGO_VERS] = m_StrArr()[IDS__LOGO]+_T(" ")+::Format(_T("%i"), int(wVerMSHigh));
   }
}

BOOL CLang::LoadMenuList(HMENU hMenu) {
   BOOL bRes = FALSE;
   CString strPath(::GetModuleDir(::GetModuleHandle(NULL)) + _T("Language\\*.lng"));
   WIN32_FIND_DATA findFileData;
   memset(&findFileData, 0, sizeof(WIN32_FIND_DATA));
   HANDLE h = ::FindFirstFile(strPath, &findFileData);
   if (h != INVALID_HANDLE_VALUE) {
      MENUITEMINFO mii = { sizeof(MENUITEMINFO),             // cbSize
                           MIIM_TYPE | MIIM_STATE | MIIM_ID, // fMask
                           MFT_STRING,                       // fType
                           MFS_ENABLED,                      // fState
                           0,                                // wID
                           NULL,                             // hSubMenu
                           NULL,                             // hbmpChecked
                           NULL,                             // hbmpUnchecked
                           0,                                // dwItemData
                           findFileData.cFileName,           // dwTypeData
                           MAX_PATH };                       // cch
      int i=0;
      do {
         i++;
         if (lstrcmpi(findFileData.cFileName, _T("English.lng")) != 0) {
            mii.wID = IDM_OPTIONS_LANG_LOAD0+i;
            findFileData.cFileName[lstrlen(findFileData.cFileName)-4] = _T('\0');
            InsertMenuItem(hMenu, i, TRUE, &mii);
         }
      } while (::FindNextFile(h, &findFileData));
      ::FindClose(h);

      if (i>1) {
         bRes = TRUE;
      }
   }
   return bRes;
}

void CLang::LoadImages(HMENU hMenu) {
   int iItems = ::GetMenuItemCount(hMenu);
   for (int i=0; i<iItems; i++) {
      UINT uID = ::GetMenuItemID(hMenu, i);
      BOOL fByPosition = ((uID==0) || (uID==-1));
      CString strIcoFile = ::GetMenuString(hMenu, uID, fByPosition);
      strIcoFile = ::GetModuleDir(::GetModuleHandle(NULL)) + _T("Language\\") + strIcoFile + _T(".ico");
      if  (::FileExist(strIcoFile)) {
         CMenuItem *pMI = CMenuItem::GetInstance(hMenu, fByPosition ? i : uID, fByPosition);
         if (pMI) {
            CImage img(strIcoFile);
            if (img.Valid()) {
               pMI->m_Img = img;
            }
         }
      }
   }
}