////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Base.cpp"
//
//    CCell
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Base.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CCell::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   SIZE result = {200, 100};
   return result;
}

int CCell::SizeInscribedSquare(int iArea, int iBorderWidth) {
   return 7; // , 7  ... :)
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

void CCell::Cell_SetDown(bool bDown) {
   m_bDown = bDown;
}

void CCell::Cell_SetStatus(EStatus Status, CClickReportContext *pClickRepContext) {
   if (pClickRepContext) {
      if (Status == _Open) {
         if (m_Cell.m_Open == _Nil)
            pClickRepContext->m_SetOpenNil.insert(this);
         else
            pClickRepContext->m_SetOpen.insert(this);
      }
   }
   m_Cell.m_Status = Status;
}

CCell::EStatus CCell::Cell_GetStatus() const {
   return m_Cell.m_Status;
}

void CCell::Cell_DefineValue() {
   if (m_Cell.m_Open == _Mine) return;
   //        
   int count = 0;
   for (int i=0; i<m_iNeighborNumber; i++) {
      if(!m_ppLinkNeighbor[i]) continue; //   ?
      if (m_ppLinkNeighbor[i]->Cell_GetOpen() == _Mine) count++;
   }
   m_Cell.m_Open = EOpen(count);
}

bool CCell::Cell_SetMine() {
   if (m_bLockMine || (m_Cell.m_Open == _Mine)) return false;
   m_Cell.m_Open = _Mine;
   return true;
}

CCell::EOpen CCell::Cell_GetOpen() const {
   return m_Cell.m_Open;
}

void CCell::Cell_SetClose(EClose Close, CClickReportContext *pClickRepContext) {
   if (pClickRepContext) {
      if ((         Close == _Flag) || //   
          (m_Cell.m_Close == _Flag))   //   
      {
         pClickRepContext->m_SetFlag.insert(this);
      }
   }
   m_Cell.m_Close = Close;
}

CCell::EClose CCell::Cell_GetClose() const {
   return m_Cell.m_Close;
}

CCell::~CCell() {
   delete [] m_ppLinkNeighbor;
   delete [] m_pNeighbor;
   delete [] m_pRegion;
}

void CCell::VerifyNeighbor(const SIZE &sizeField) {
   for (int i=0; i<m_iNeighborNumber; i++)
      if ((m_pNeighbor[i].X >= sizeField.cx) ||
          (m_pNeighbor[i].Y >= sizeField.cy) ||
          (m_pNeighbor[i].X < 0) ||
          (m_pNeighbor[i].Y < 0))
      {
         m_pNeighbor[i] = INCORRECT_COORD;
      }
}

CCell::CCell(
   const COORD &Coord,
   const SIZE &sizeField,
   int iArea,
   const CGraphicContext &gContext,
   int iNeighborNumber,
   int iVertexNumber,
   int iDirection
):
   m_Coord           (Coord),
   m_GContext        (gContext),
   m_iNeighborNumber (iNeighborNumber),
   m_iVertexNumber   (iVertexNumber),
   m_iDirection      (iDirection),
   m_pNeighbor       (new COORD[iNeighborNumber]),
   m_pRegion         (new POINT[iVertexNumber]),
   m_ppLinkNeighbor  (NULL)
{
   Reset();
   //     
}

void CCell::Lock() {
   m_bLockMine = true;
}

void CCell::LockNeighbor() {
   m_bLockMine = true;
   //     ,
   for (int i=0; i<m_iNeighborNumber; i++) {
      if (!m_ppLinkNeighbor[i]) continue; //   ?
      m_ppLinkNeighbor[i]->Lock();
   }
}

void CCell::SetNeighborLink(CCell *const *const ppLinkNeighbor, int numberNeighbor) {
   m_ppLinkNeighbor = new CCell*[numberNeighbor];
   for (int i=0; i<numberNeighbor; i++)
      m_ppLinkNeighbor[i] = ppLinkNeighbor[i];
}

const COORD &CCell::GetNeighborCoord(int index) const {
   static COORD errCoord = INCORRECT_COORD;
   return (index<m_iNeighborNumber) ? m_pNeighbor[index] : errCoord;
}

COORD CCell::GetCoord() const {
   return m_Coord;
}

int CCell::GetNeighborNumber() const { //   
   return m_iNeighborNumber;
}

POINT CCell::GetCenterPixel() const {
   POINT result = {m_Square.left + (m_Square.right -m_Square.left)/2,
                   m_Square.top  + (m_Square.bottom-m_Square.top )/2};
   return result;
}

bool CCell::PointInRegion(const POINT &point) const { //      
   return PointInPolygon(point, m_pRegion, m_iVertexNumber);
}

void CCell::SetPoint(int /*iArea*/) {
   //      !
   //    
   if (m_iVertexNumber) {
      m_rcOuter.left = m_rcOuter.right  = m_pRegion[0].x;
      m_rcOuter.top  = m_rcOuter.bottom = m_pRegion[0].y;
      for (int i=1; i<m_iVertexNumber; i++) {
         if (m_rcOuter.left   > m_pRegion[i].x) m_rcOuter.left   = m_pRegion[i].x;
         if (m_rcOuter.top    > m_pRegion[i].y) m_rcOuter.top    = m_pRegion[i].y;
         if (m_rcOuter.right  < m_pRegion[i].x) m_rcOuter.right  = m_pRegion[i].x;
         if (m_rcOuter.bottom < m_pRegion[i].y) m_rcOuter.bottom = m_pRegion[i].y;
      }
   }
}

void CCell::Reset() {
   m_Cell.m_Status = _Close;
   m_Cell.m_Open   = _Nil;
   m_Cell.m_Close  = _Clear;
   m_bLockMine = false;
   m_bDown = false;
}

void CCell::Paint() const {
   BitBlt(m_GContext.m_hDCTmp,
      m_Square.left,
      m_Square.top,
      m_Square.right -m_Square.left,
      m_Square.bottom-m_Square.top,
      m_GContext.m_hDCBck,
      m_Square.left,
      m_Square.top,
      SRCCOPY);

   // output Pictures
   if ((m_Cell.m_Status == _Close) && /**/(m_Cell.m_Close == _Flag) && m_GContext.m_ImgFlag.Valid())
      m_GContext.m_ImgFlag.Draw(m_GContext.m_hDCTmp, m_Square);
   else
   if ((m_Cell.m_Status == _Open ) && /**/(m_Cell.m_Open  == _Mine) && m_GContext.m_ImgMine.Valid())
      m_GContext.m_ImgMine.Draw(m_GContext.m_hDCTmp, m_Square);
   else
   // output text
   {
      LPCTSTR szCaption;
      if (m_Cell.m_Status == _Close) {
         SetTextColor(m_GContext.m_hDCTmp, m_GContext.m_ColorText.m_colorClose[m_Cell.m_Close]);
         szCaption = CCell::GetCaptionClose(m_Cell.m_Close);
      } else {
         SetTextColor(m_GContext.m_hDCTmp, m_GContext.m_ColorText.m_colorOpen[m_Cell.m_Open]);
         szCaption = CCell::GetCaptionOpen(m_Cell.m_Open);
      }
      RECT sq_tmp = {m_Square.left   + !!m_bDown,
                     m_Square.top    + !!m_bDown,
                     m_Square.right  + !!m_bDown,
                     m_Square.bottom + !!m_bDown};
      DrawText(m_GContext.m_hDCTmp, szCaption,
               -1, &sq_tmp, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
   }
}

void CCell::LButtonDown() {
   if (m_Cell.m_Close  == _Flag) return;
   if (m_Cell.m_Status == _Close) {
      m_bDown = true;
      Paint();
      return;
   }
   //     
   if ((m_Cell.m_Status == _Open) && (m_Cell.m_Open != _Nil))
      for (int i=0; i<m_iNeighborNumber; i++) {
         if (!m_ppLinkNeighbor[i]) continue; //   ?
         if ((m_ppLinkNeighbor[i]->Cell_GetStatus() == _Open) ||
             (m_ppLinkNeighbor[i]->Cell_GetClose()  == _Flag)) continue;
         m_ppLinkNeighbor[i]->Cell_SetDown(true);
         m_ppLinkNeighbor[i]->Paint();
      }
}

CLeftUpReturn CCell::LButtonUp(bool isMy, CClickReportContext *pClickRepContext) {
   CLeftUpReturn result = {0, 0, 0, false, false};

   if (m_Cell.m_Close == _Flag) return result;
   //    
   if ((m_Cell.m_Status == _Open) && (m_Cell.m_Open != _Nil))
      for (int i=0; i<m_iNeighborNumber; i++) {
         if (!m_ppLinkNeighbor[i]) continue; //   ?
         if ((m_ppLinkNeighbor[i]->Cell_GetStatus() == _Open) ||
             (m_ppLinkNeighbor[i]->Cell_GetClose()  == _Flag)) continue;
         m_ppLinkNeighbor[i]->Cell_SetDown(false);
         m_ppLinkNeighbor[i]->Paint();
      }
   //      
   if (m_Cell.m_Status == _Close) {
      if (!isMy){
         m_bDown = false;
         Paint();
         return result;
      } else {
         result.m_iCountUnknown += (m_Cell.m_Close == _Unknown) ? -1 : 0;
         result.m_iCountOpen++;
         Cell_SetStatus(_Open, pClickRepContext);
         m_bDown = true;
         Paint();
      }
   }
   // !      
   //  -       
   int countFlags = 0;
   int countClear = 0;
   if (m_Cell.m_Open != _Nil)
      for (int i=0; i<m_iNeighborNumber; i++) {
         if(!m_ppLinkNeighbor[i]) continue; //   ?
         if (m_ppLinkNeighbor[i]->Cell_GetStatus() == _Open) continue;
         if (m_ppLinkNeighbor[i]->Cell_GetClose()  == _Flag)
            countFlags++;
         else countClear++;
      }
   //   
   if ((m_Cell.m_Open != _Nil) && (countFlags+countClear == m_Cell.m_Open))
      for (int i=0; i<m_iNeighborNumber; i++) {
         if(!m_ppLinkNeighbor[i]) continue; //   ?
         if ((m_ppLinkNeighbor[i]->Cell_GetStatus() == _Open) ||
             (m_ppLinkNeighbor[i]->Cell_GetClose()  == _Flag)) continue;
         result.m_iCountUnknown += (m_ppLinkNeighbor[i]->Cell_GetClose() == _Unknown) ? -1 : 0;
         result.m_iCountFlag++;
         m_ppLinkNeighbor[i]->Cell_SetClose(_Flag, pClickRepContext);
         m_ppLinkNeighbor[i]->Paint();
      }
   if (!isMy) return result;
   //  
   if ((countFlags+result.m_iCountFlag) == m_Cell.m_Open)
      for (int i=0; i<m_iNeighborNumber; i++) {
         if (!m_ppLinkNeighbor[i]) continue; //   ?
         if ((m_ppLinkNeighbor[i]->Cell_GetStatus() == _Open) ||
             (m_ppLinkNeighbor[i]->Cell_GetClose()  == _Flag)) continue;
         result.m_iCountUnknown += (m_ppLinkNeighbor[i]->Cell_GetClose() == _Unknown) ? -1 : 0;
         result.m_iCountOpen++;
         m_ppLinkNeighbor[i]->Cell_SetDown(true);
         m_ppLinkNeighbor[i]->Cell_SetStatus(_Open, pClickRepContext);
         m_ppLinkNeighbor[i]->Paint();
         if (m_ppLinkNeighbor[i]->Cell_GetOpen() == _Nil) {
            const CLeftUpReturn result2 = m_ppLinkNeighbor[i]->LButtonUp(true, pClickRepContext);
            result.m_iCountFlag    += result2.m_iCountFlag;
            result.m_iCountOpen    += result2.m_iCountOpen;
            result.m_iCountUnknown += result2.m_iCountUnknown;
            if (result.m_bEndGame) {
               result.m_bEndGame = result2.m_bEndGame;
               result.m_bVictory = result2.m_bVictory;
            }
         }
         if (m_ppLinkNeighbor[i]->Cell_GetOpen() == _Mine) {
            result.m_bEndGame = true;
            result.m_bVictory = false;
            return result;
         }
      }
   if (m_Cell.m_Open == _Mine) {
      result.m_bEndGame = true;
      result.m_bVictory = false;
   }
   return result;
}

CRightDownReturn CCell::RButtonDown(EClose Close, CClickReportContext *pClickRepContext) {
   CRightDownReturn result = {0,0};

   if ((m_Cell.m_Status == _Open) || m_bDown) return result;
   switch (m_Cell.m_Close) {
   case _Clear:
      switch (Close) {
         case _Flag:    result.m_iCountFlag    = +1;  break;
         case _Unknown: result.m_iCountUnknown = +1;
      }
      if (m_Cell.m_Close != Close) Cell_SetClose(Close, pClickRepContext);
      break;
   case _Flag:
      switch (Close) {
         case _Unknown: result.m_iCountUnknown = +1;
         case _Clear:   result.m_iCountFlag    = -1;
      }
      if (m_Cell.m_Close != Close) Cell_SetClose(Close, pClickRepContext);
      break;
   case _Unknown:
      switch (Close) {
         case _Flag:    result.m_iCountFlag    = +1;
         case _Clear:   result.m_iCountUnknown = -1;
      }
      if (m_Cell.m_Close != Close) Cell_SetClose(Close, pClickRepContext);
   }
   Paint();
   return result;
}
