////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Quadrangle1.cpp"
//
//   CQuadrangle1 -  120-90-60-90
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Quadrangle1.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CQuadrangle1::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(iArea/SQRT3())*2;
   m_b() = m_a()/2;
   m_h() = m_b()*SQRT3();
 //m_n() = m_a()*0.75f;
   m_m() = m_h()/2;

   SIZE result = {toLONG(m_m() + m_m()*((sizeField.cx+2)/3)+
                                 m_h()*((sizeField.cx+1)/3)+
                                 m_m()*((sizeField.cx+0)/3)),
                  toLONG(m_b() + m_b()*((sizeField.cy+1)/2)+
                                 m_a()*((sizeField.cy+0)/2))};
   return result;
}

int CQuadrangle1::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = sqrt(iArea/SQRT3())*2;
   m_sq() = (m_a()*SQRT3()-2*iBorderWidth*(1+SQRT3()))/(SQRT3()+2); //  ,   
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CQuadrangle1::CQuadrangle1(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           9, 4,
           (Coord.Y&3)*3+(Coord.X%3) // 0..11
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case  0:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+2;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X+1;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+2;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  1:
      m_pNeighbor[0].X = m_Coord.X-2;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X  ;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;  
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  2:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-2;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-2;   m_pNeighbor[5].Y = m_Coord.Y+1;  
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  3:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+2;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X+2;   m_pNeighbor[6].Y = m_Coord.Y  ;
      m_pNeighbor[7].X = m_Coord.X-1;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X  ;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  4:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-2;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  5:
      m_pNeighbor[0].X = m_Coord.X-2;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X  ;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-2;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X+1;   m_pNeighbor[6].Y = m_Coord.Y  ;
      m_pNeighbor[7].X = m_Coord.X-1;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X  ;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  6:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X+1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-2;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-2;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  7:
      m_pNeighbor[0].X = m_Coord.X  ;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X+1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X-1;   m_pNeighbor[2].Y = m_Coord.Y  ;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+2;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X+1;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+2;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  8:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+2;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X-1;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case  9:
      m_pNeighbor[0].X = m_Coord.X-2;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X-1;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X  ;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+1;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-2;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X+1;   m_pNeighbor[6].Y = m_Coord.Y  ;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case 10:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X+2;   m_pNeighbor[3].Y = m_Coord.Y-1;
      m_pNeighbor[4].X = m_Coord.X-1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X+1;   m_pNeighbor[5].Y = m_Coord.Y  ;
      m_pNeighbor[6].X = m_Coord.X+2;   m_pNeighbor[6].Y = m_Coord.Y  ;
      m_pNeighbor[7].X = m_Coord.X  ;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+1;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   case 11:
      m_pNeighbor[0].X = m_Coord.X-1;   m_pNeighbor[0].Y = m_Coord.Y-1;
      m_pNeighbor[1].X = m_Coord.X  ;   m_pNeighbor[1].Y = m_Coord.Y-1;
      m_pNeighbor[2].X = m_Coord.X+1;   m_pNeighbor[2].Y = m_Coord.Y-1;
      m_pNeighbor[3].X = m_Coord.X-1;   m_pNeighbor[3].Y = m_Coord.Y  ;
      m_pNeighbor[4].X = m_Coord.X+1;   m_pNeighbor[4].Y = m_Coord.Y  ;
      m_pNeighbor[5].X = m_Coord.X-1;   m_pNeighbor[5].Y = m_Coord.Y+1;
      m_pNeighbor[6].X = m_Coord.X  ;   m_pNeighbor[6].Y = m_Coord.Y+1;
      m_pNeighbor[7].X = m_Coord.X+1;   m_pNeighbor[7].Y = m_Coord.Y+1;
      m_pNeighbor[8].X = m_Coord.X+2;   m_pNeighbor[8].Y = m_Coord.Y+1;
      break;
   }
   VerifyNeighbor(sizeField);
}

void CQuadrangle1::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(iArea/SQRT3())*2;
      m_b() = m_a()/2;
      m_h() = m_b()*SQRT3();
      m_n() = m_a()*0.75f;
      m_m() = m_h()/2;
      m_sq() = (m_a()*SQRT3()-2*m_GContext.m_Border.m_iWidth*(1+SQRT3()))/(SQRT3()+2); //  ,   
      m_Z()  = m_a()/(1+SQRT3());
      m_Zx() = m_Z()*SQRT3()/2;
      m_Zy() = m_Z()/2; 
   }

   //    
   float oX = (m_h()*2)*(m_Coord.X/3) + m_h()+m_m(); // offset X
   float oY = (m_a()*3)*(m_Coord.Y/4) + m_a()+m_n(); // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX - m_h()       ); m_pRegion[0].y = toLONG(oY - m_n()-m_n());
      m_pRegion[1].x = toLONG(oX - m_m()       ); m_pRegion[1].y = toLONG(oY - m_n()      );
      m_pRegion[2].x = toLONG(oX - m_h()-m_m() ); m_pRegion[2].y = toLONG(oY - m_n()      );
      m_pRegion[3].x = toLONG(oX - m_h()-m_m() ); m_pRegion[3].y = toLONG(oY - m_n()-m_b());
      break;
   case 1:
      m_pRegion[0].x = toLONG(oX               ); m_pRegion[0].y = toLONG(oY - m_n()-m_n());
      m_pRegion[1].x = toLONG(oX - m_m()       ); m_pRegion[1].y = toLONG(oY - m_n()-m_a());
      m_pRegion[2].x = toLONG(oX - m_h()       ); m_pRegion[2].y = toLONG(oY - m_n()-m_n());
      m_pRegion[3].x = toLONG(oX - m_m()       ); m_pRegion[3].y = toLONG(oY - m_n()      );
      break;
   case 2:
      m_pRegion[0].x = toLONG(oX + m_m()       ); m_pRegion[0].y = toLONG(oY - m_n()-m_b());
      m_pRegion[1].x = toLONG(oX               ); m_pRegion[1].y = toLONG(oY - m_n()-m_n());
      m_pRegion[2].x = toLONG(oX - m_m()       ); m_pRegion[2].y = toLONG(oY - m_n()      );
      m_pRegion[3].x = toLONG(oX + m_m()       ); m_pRegion[3].y = toLONG(oY - m_n()      );
      break;
   case 3:
      m_pRegion[0].x = toLONG(oX - m_m()       ); m_pRegion[0].y = toLONG(oY - m_n()      );
      m_pRegion[1].x = toLONG(oX - m_h()       ); m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX - m_h()-m_m() ); m_pRegion[2].y = toLONG(oY - m_n()+m_b());
      m_pRegion[3].x = toLONG(oX - m_h()-m_m() ); m_pRegion[3].y = toLONG(oY - m_n()      );
      break;
   case 4:
      m_pRegion[0].x = toLONG(oX - m_m()       ); m_pRegion[0].y = toLONG(oY - m_n()      );
      m_pRegion[1].x = toLONG(oX               ); m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX - m_m()       ); m_pRegion[2].y = toLONG(oY - m_n()+m_a());
      m_pRegion[3].x = toLONG(oX - m_h()       ); m_pRegion[3].y = toLONG(oY              );
      break;
   case 5:
      m_pRegion[0].x = toLONG(oX + m_m()       ); m_pRegion[0].y = toLONG(oY - m_n()      );
      m_pRegion[1].x = toLONG(oX - m_m()       ); m_pRegion[1].y = toLONG(oY - m_n()      );
      m_pRegion[2].x = toLONG(oX               ); m_pRegion[2].y = toLONG(oY              );
      m_pRegion[3].x = toLONG(oX + m_m()       ); m_pRegion[3].y = toLONG(oY - m_n()+m_b());
      break;
   case 6:
      m_pRegion[0].x = toLONG(oX - m_m()       ); m_pRegion[0].y = toLONG(oY + m_n()-m_b());
      m_pRegion[1].x = toLONG(oX - m_h()       ); m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX - m_h()-m_m() ); m_pRegion[2].y = toLONG(oY + m_n()      );
      m_pRegion[3].x = toLONG(oX - m_m()       ); m_pRegion[3].y = toLONG(oY + m_n()      );
      break;
   case 7:
      m_pRegion[0].x = toLONG(oX               ); m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX + m_m()       ); m_pRegion[1].y = toLONG(oY + m_n()      );
      m_pRegion[2].x = toLONG(oX - m_m()       ); m_pRegion[2].y = toLONG(oY + m_n()      );
      m_pRegion[3].x = toLONG(oX - m_m()       ); m_pRegion[3].y = toLONG(oY + m_n()-m_b());
      break;
   case 8:
      m_pRegion[0].x = toLONG(oX + m_h()       ); m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX + m_m()       ); m_pRegion[1].y = toLONG(oY + m_n()-m_a());
      m_pRegion[2].x = toLONG(oX               ); m_pRegion[2].y = toLONG(oY              );
      m_pRegion[3].x = toLONG(oX + m_m()       ); m_pRegion[3].y = toLONG(oY + m_n()      );
      break;
   case 9:
      m_pRegion[0].x = toLONG(oX - m_m()       ); m_pRegion[0].y = toLONG(oY + m_n()      );
      m_pRegion[1].x = toLONG(oX - m_h()-m_m() ); m_pRegion[1].y = toLONG(oY + m_n()      );
      m_pRegion[2].x = toLONG(oX - m_h()       ); m_pRegion[2].y = toLONG(oY + m_n()+m_n());
      m_pRegion[3].x = toLONG(oX - m_m()       ); m_pRegion[3].y = toLONG(oY + m_n()+m_b());
      break;
   case 10:
      m_pRegion[0].x = toLONG(oX + m_m()       ); m_pRegion[0].y = toLONG(oY + m_n()      );
      m_pRegion[1].x = toLONG(oX               ); m_pRegion[1].y = toLONG(oY + m_n()+m_n());
      m_pRegion[2].x = toLONG(oX - m_m()       ); m_pRegion[2].y = toLONG(oY + m_n()+m_b());
      m_pRegion[3].x = toLONG(oX - m_m()       ); m_pRegion[3].y = toLONG(oY + m_n()      );
      break;
   case 11:
      m_pRegion[0].x = toLONG(oX + m_m()       ); m_pRegion[0].y = toLONG(oY + m_n()      );
      m_pRegion[1].x = toLONG(oX + m_h()       ); m_pRegion[1].y = toLONG(oY + m_n()+m_n());
      m_pRegion[2].x = toLONG(oX + m_m()       ); m_pRegion[2].y = toLONG(oY + m_n()+m_a());
      m_pRegion[3].x = toLONG(oX               ); m_pRegion[3].y = toLONG(oY + m_n()+m_n());
      break;
   }

   //       -     /
   {
      POINTFLOAT center; //      
      switch (m_iDirection) {
      case 0:  center.x = oX - m_h()-m_m()+m_Zx(); center.y = oY - m_n()-m_b()+m_Zy(); break;
      case 1:  center.x = oX - m_m();              center.y = oY - m_n()-m_a()+m_Z (); break;
      case 2:  center.x = oX + m_m()      -m_Zx(); center.y = oY - m_n()-m_b()+m_Zy(); break;
      case 3:  center.x = oX - m_h()-m_m()+m_Zx(); center.y = oY - m_n()+m_b()-m_Zy(); break;
      case 4:  center.x = oX - m_m();              center.y = oY - m_n()+m_a()-m_Z (); break;
      case 5:  center.x = oX + m_m()      -m_Zx(); center.y = oY - m_n()+m_b()-m_Zy(); break;
      case 6:  center.x = oX - m_m()      -m_Zx(); center.y = oY + m_n()-m_b()+m_Zy(); break;
      case 7:  center.x = oX - m_m()      +m_Zx(); center.y = oY + m_n()-m_b()+m_Zy(); break;
      case 8:  center.x = oX + m_m();              center.y = oY + m_n()-m_a()+m_Z (); break;
      case 9:  center.x = oX - m_m()      -m_Zx(); center.y = oY + m_n()+m_b()-m_Zy(); break;
      case 10: center.x = oX - m_m()      +m_Zx(); center.y = oY + m_n()+m_b()-m_Zy(); break;
      case 11: center.x = oX + m_m();              center.y = oY + m_n()+m_a()-m_Z (); break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CQuadrangle1::Paint() const {
   /**/
   int minX = min(m_pRegion[1].x, min(m_pRegion[2].x, m_pRegion[3].x));
   int maxX = max(m_pRegion[0].x, m_pRegion[1].x);
   int minY = min(m_pRegion[0].y, min(m_pRegion[1].y, m_pRegion[3].y));
   int maxY = max(m_pRegion[1].y, max(m_pRegion[2].y, m_pRegion[3].y));
   BitBlt(m_GContext.m_hDCTmp, minX, minY, maxX - minX, maxY - minY,
          m_GContext.m_hDCBck, minX, minY, SRCCOPY);
   /**/
   CCell::Paint();

   switch (m_iDirection) {
   case 0: case 7:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   case 1: case 8:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   case 2: case 6:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   case 3: case 10:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   case 4: case 11:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   case 5: case 9:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      break;
   }

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 4, GetOuterRect().pointLT());
}
