////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "TrSq2.cpp"
//
//   CTrSq2 -   24   12  ( 1   2 )
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TrSq2.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CTrSq2::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(6*iArea/(2+SQRT3())); //     
   m_b() = m_a()/2;
   m_h() = m_b()*SQRT3();

   SIZE result = {toLONG(m_b()+m_h()*((sizeField.cx+2)/3)+
                               m_a()*((sizeField.cx+1)/3)+
                               m_b()*((sizeField.cx+0)/3)),
                  toLONG(m_b()+m_h()*((sizeField.cy+2)/3)+
                               m_a()*((sizeField.cy+1)/3)+
                               m_b()*((sizeField.cy+0)/3))};
   return result;
}

int CTrSq2::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = sqrt(6*iArea/(2+SQRT3())); //     
   m_sq() = (m_a()*SQRT3()-iBorderWidth*6)/(SQRT3()+2) - 1;
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CTrSq2::CTrSq2(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           12, 4,
           (Coord.Y%6)*6+(Coord.X%6) // 0..35
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case 0:
   case 21: m_pNeighbor[ 0].X = m_Coord.X+1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
            m_pNeighbor[10].X = m_Coord.X-1;   m_pNeighbor[10].Y = m_Coord.Y+2;
            m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+2;
            break;
   case 1:
   case 22: m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 2:
   case 23: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 3:
   case 18: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-2;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9].X = m_Coord.X  ;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
            m_pNeighbor[10].X = m_Coord.X+1;   m_pNeighbor[10].Y = m_Coord.Y+1;
            m_pNeighbor[11].X = m_Coord.X+1;   m_pNeighbor[11].Y = m_Coord.Y+2;
            break;
   case 4:
   case 19: m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
            m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
            m_pNeighbor[ 5].X = m_Coord.X  ;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
            m_pNeighbor[ 6].X = m_Coord.X+1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+2;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+2;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 5:
   case 20: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-2;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y+2;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 6:
   case 27: m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
            m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X+2;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
            m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+2;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 7:
   case 28: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
            m_pNeighbor[10].X = m_Coord.X+2;   m_pNeighbor[10].Y = m_Coord.Y+1;
            m_pNeighbor[11].X = m_Coord.X+3;   m_pNeighbor[11].Y = m_Coord.Y+1;
            break;
   case 8:
   case 29: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-3;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
            m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
            m_pNeighbor[ 6].X = m_Coord.X-2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y  ;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 9:
   case 24: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-2;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 10:
   case 25: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
            m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y+2;
            m_pNeighbor[ 9].X = m_Coord.X  ;   m_pNeighbor[ 9].Y = m_Coord.Y+2;
            m_pNeighbor[10].X = m_Coord.X+1;   m_pNeighbor[10].Y = m_Coord.Y+2;
            m_pNeighbor[11].X = m_Coord.X+1;   m_pNeighbor[11].Y = m_Coord.Y+3;
            break;
   case 11:
   case 26: m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
            m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
            m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X+1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 12:
   case 33: m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 13:
   case 34: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 14:
   case 35: m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-2;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-2;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9].X = m_Coord.X  ;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
            m_pNeighbor[10].X = m_Coord.X+1;   m_pNeighbor[10].Y = m_Coord.Y+1;
            m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+2;
            break;
   case 15:
   case 30: m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
            m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
            m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
            m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
            m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y+2;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 16:
   case 31: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
            m_pNeighbor[ 2].X = m_Coord.X-3;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
            m_pNeighbor[ 5].X = m_Coord.X-2;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
            m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
            m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9] =
            m_pNeighbor[10] =
            m_pNeighbor[11] = INCORRECT_COORD;
            break;
   case 17:
   case 32: m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
            m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
            m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-2;
            m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
            m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
            m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
            m_pNeighbor[ 6].X = m_Coord.X+1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
            m_pNeighbor[ 7].X = m_Coord.X+2;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
            m_pNeighbor[ 8].X = m_Coord.X-1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
            m_pNeighbor[ 9].X = m_Coord.X  ;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
            m_pNeighbor[10].X = m_Coord.X+1;   m_pNeighbor[10].Y = m_Coord.Y+1;
            m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+2;
            break;
   }
   for (int i=0; i<12; i++)
      if ((m_pNeighbor[i].X >= sizeField.cx) ||
          (m_pNeighbor[i].Y >= sizeField.cy) ||
          (m_pNeighbor[i].X < 0) ||
          (m_pNeighbor[i].Y < 0)) {
         m_pNeighbor[i] = INCORRECT_COORD;
      }
}

bool CTrSq2::PointInRegion(const POINT& point) const { //      
   switch (m_iDirection) {
   case 0:  case 21:
   case 3:  case 18:
   case 7:  case 28:
   case 10: case 25:
   case 14: case 35:
   case 17: case 32:
      return PointInPolygon(point, m_pRegion, 4);
   case 1:  case 22:
   case 2:  case 23:
   case 4:  case 19:
   case 5:  case 20:
   case 6:  case 27:
   case 8:  case 29:
   case 9:  case 24:
   case 11: case 26:
   case 12: case 33:
   case 13: case 34:
   case 15: case 30:
   case 16: case 31:
      return PointInPolygon(point, m_pRegion, 3);
   }
   return false;
}

void CTrSq2::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(6*iArea/(2+SQRT3())); //     
      m_b() = m_a()/2;
      m_h() = m_b()*SQRT3();
      m_sq() = (m_a()*SQRT3()-m_GContext.m_Border.m_iWidth*6)/(SQRT3()+2) - 1;
   }

   //    
   float oX; // offset X
   float oY; // offset Y
   switch (m_iDirection) {
   case  0: case  1: case  2: case  6: case  7: case  8: case 12: case 13: case 14: oX = (m_h()*2+m_a()*3)*(m_Coord.X/6) + m_h()+m_b();
                                                                                    oY = (m_h()*2+m_a()*3)*(m_Coord.Y/6) + m_h();            break;
   case  3: case  4: case  5: case  9: case 10: case 11: case 15: case 16: case 17: oX = (m_h()*2+m_a()*3)*(m_Coord.X/6) + m_h()*2+m_a()+m_b();
                                                                                    oY = (m_h()*2+m_a()*3)*(m_Coord.Y/6) + m_h()+m_b();        break;
   case 18: case 19: case 20: case 24: case 25: case 26: case 30: case 31: case 32: oX = (m_h()*2+m_a()*3)*(m_Coord.X/6) + m_h();
                                                                                    oY = (m_h()*2+m_a()*3)*(m_Coord.Y/6) + m_h()*2+m_a()*2;    break;
   case 21: case 22: case 23: case 27: case 28: case 29: case 33: case 34: case 35: oX = (m_h()*2+m_a()*3)*(m_Coord.X/6) + m_h()*2+m_a()*2;
                                                                                    oY = (m_h()*2+m_a()*3)*(m_Coord.Y/6) + m_h()*2+m_a()+m_b();  break;
   }
   switch (m_iDirection) {
   case 0: case 21:
      m_pRegion[0].x = toLONG(oX - m_b()      );  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX - m_b()-m_h());  m_pRegion[1].y = toLONG(oY + m_b()-m_h());
      m_pRegion[2].x = toLONG(oX - m_h()      );  m_pRegion[2].y = toLONG(oY + m_b()      );
      m_pRegion[3].x = toLONG(oX              );  m_pRegion[3].y = toLONG(oY              );
      break;
   case 1: case 22:
      m_pRegion[0].x = toLONG(oX + m_b()      );  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX - m_b()      );  m_pRegion[1].y = toLONG(oY - m_h()      );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 2: case 23:
      m_pRegion[0].x = toLONG(oX + m_a()+m_b());  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX + m_b()      );  m_pRegion[1].y = toLONG(oY - m_h()      );
      m_pRegion[2].x = toLONG(oX + m_a()      );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 3: case 18:
      m_pRegion[0].x = toLONG(oX - m_h()+m_b());  m_pRegion[0].y = toLONG(oY - m_b()-m_h());
      m_pRegion[1].x = toLONG(oX - m_h()      );  m_pRegion[1].y = toLONG(oY - m_b()      );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY              );
      m_pRegion[3].x = toLONG(oX + m_b()      );  m_pRegion[3].y = toLONG(oY - m_h()      );
      break;
   case 4: case 19:
      m_pRegion[0].x = toLONG(oX + m_b()      );  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX + m_a()      );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 5: case 20:
      m_pRegion[0].x = toLONG(oX + m_a()+m_b());  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX + m_b()      );  m_pRegion[1].y = toLONG(oY - m_h()      );
      m_pRegion[2].x = toLONG(oX + m_a()      );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 6: case 27:
      m_pRegion[0].x = toLONG(oX              );  m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX - m_h()      );  m_pRegion[1].y = toLONG(oY + m_b()      );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()      );
      break;
   case 7: case 28:
      m_pRegion[0].x = toLONG(oX + m_a()      );  m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()      );
      m_pRegion[3].x = toLONG(oX + m_a()      );  m_pRegion[3].y = toLONG(oY + m_a()      );
      break;
   case 8: case 29:
      m_pRegion[0].x = toLONG(oX + m_b()      );  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX + m_a()      );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 9: case 24:
      m_pRegion[0].x = toLONG(oX - m_h()      );  m_pRegion[0].y = toLONG(oY - m_b()      );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX - m_h()      );  m_pRegion[2].y = toLONG(oY + m_b()      );
      break;
   case 10: case 25:
      m_pRegion[0].x = toLONG(oX + m_a()      );  m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()      );
      m_pRegion[3].x = toLONG(oX + m_a()      );  m_pRegion[3].y = toLONG(oY + m_a()      );
      break;
   case 11: case 26:
      m_pRegion[0].x = toLONG(oX + m_a()+m_b());  m_pRegion[0].y = toLONG(oY - m_h()      );
      m_pRegion[1].x = toLONG(oX + m_a()+m_a());  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX + m_a()      );  m_pRegion[2].y = toLONG(oY              );
      break;
   case 12: case 33:
      m_pRegion[0].x = toLONG(oX - m_h()      );  m_pRegion[0].y = toLONG(oY + m_b()      );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY + m_a()      );
      m_pRegion[2].x = toLONG(oX - m_h()      );  m_pRegion[2].y = toLONG(oY + m_a()+m_b());
      break;
   case 13: case 34:
      m_pRegion[0].x = toLONG(oX              );  m_pRegion[0].y = toLONG(oY + m_a()      );
      m_pRegion[1].x = toLONG(oX - m_h()      );  m_pRegion[1].y = toLONG(oY + m_a()+m_b());
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()+m_a());
      break;
   case 14: case 35:
      m_pRegion[0].x = toLONG(oX + m_a()      );  m_pRegion[0].y = toLONG(oY + m_a()      );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY + m_a()      );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()+m_a());
      m_pRegion[3].x = toLONG(oX + m_a()      );  m_pRegion[3].y = toLONG(oY + m_a()+m_a());
      break;
   case 15: case 30:
      m_pRegion[0].x = toLONG(oX - m_h()      );  m_pRegion[0].y = toLONG(oY + m_b()      );
      m_pRegion[1].x = toLONG(oX              );  m_pRegion[1].y = toLONG(oY + m_a()      );
      m_pRegion[2].x = toLONG(oX - m_h()      );  m_pRegion[2].y = toLONG(oY + m_a()+m_b());
      break;
   case 16: case 31:
      m_pRegion[0].x = toLONG(oX              );  m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX - m_h()      );  m_pRegion[1].y = toLONG(oY + m_b()      );
      m_pRegion[2].x = toLONG(oX              );  m_pRegion[2].y = toLONG(oY + m_a()      );
      break;
   case 17: case 32:
      m_pRegion[0].x = toLONG(oX + m_a()+m_a());  m_pRegion[0].y = toLONG(oY              );
      m_pRegion[1].x = toLONG(oX + m_a()      );  m_pRegion[1].y = toLONG(oY              );
      m_pRegion[2].x = toLONG(oX + m_a()      );  m_pRegion[2].y = toLONG(oY + m_a()      );
      m_pRegion[3].x = toLONG(oX + m_a()+m_a());  m_pRegion[3].y = toLONG(oY + m_a()      );
      break;
   }

   //       -     /
   {
      POINTFLOAT center;
      float wsq2 = m_GContext.m_Border.m_iWidth+m_sq()/2;
      switch (m_iDirection) {
      case  0: case 21: center.x = oX - (m_b()+m_h())/2; center.y = oY + (m_b()-m_h())/2; break;
      case  1: case 22: center.x = oX;                   center.y = oY - m_h()+wsq2;      break;
      case  2: case 23: center.x = oX + m_a();           center.y = oY - m_h()+wsq2;      break;
      case  3: case 18: center.x = oX + (m_b()-m_h())/2; center.y = oY - (m_b()+m_h())/2; break;
      case  4: case 19: center.x = oX + m_b();           center.y = oY - wsq2;            break;
      case  5: case 20: center.x = oX + m_a();           center.y = oY - m_h()+wsq2;      break;
      case  6: case 27: center.x = oX - wsq2;            center.y = oY + m_b();           break;
      case  7: case 28: center.x = oX + m_b();           center.y = oY + m_b();           break;
      case  8: case 29: center.x = oX + m_b();           center.y = oY - wsq2;            break;
      case  9: case 24: center.x = oX - m_h()+wsq2;      center.y = oY;                   break;
      case 10: case 25: center.x = oX + m_b();           center.y = oY + m_b();           break;
      case 11: case 26: center.x = oX + m_a()+m_b();     center.y = oY - wsq2;            break;
      case 12: case 33: center.x = oX - m_h()+wsq2;      center.y = oY + m_a();           break;
      case 13: case 34: center.x = oX - wsq2;            center.y = oY + m_a()+m_b();     break;
      case 14: case 35: center.x = oX + m_b();           center.y = oY + m_a()+m_b();     break;
      case 15: case 30: center.x = oX - m_h()+wsq2;      center.y = oY + m_a();           break;
      case 16: case 31: center.x = oX - wsq2;            center.y = oY + m_b();           break; 
      case 17: case 32: center.x = oX + m_a()+m_b();     center.y = oY + m_b();           break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CTrSq2::Paint() const {
/**
   int minX = min(m_pRegion[1].x, min(m_pRegion[2].x, m_pRegion[3].x));
   int maxX = max(m_pRegion[0].x, m_pRegion[1].x);
   int minY = m_pRegion[0].y;
   int maxY = m_pRegion[2].y;
   BitBlt(m_GContext.m_hDCTmp, minX, minY, maxX - minX, maxY - minY,
          hDCBck, minX, minY, SRCCOPY);
/**/
   CCell::Paint();

   switch (m_iDirection) {
   case 4: case  8: case 11: case 19: case 26: case 29:
   case 9: case 12: case 15: case 24: case 30: case 33:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);           break;
   case 0: case 21:
   case 3: case 18:
   case 7: case 10: case 14: case 17: case 25: case 28: case 32: case 35:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y, NULL);     break;
   case 1: case  2: case  5: case 20: case 22: case 23:
   case 6: case 13: case 16: case 27: case 31: case 34:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);           break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   switch (m_iDirection) {
   case 4: case  8: case 11: case 19: case 26: case 29:
   case 9: case 12: case 15: case 24: case 30: case 33:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 0: case 21:
   case 3: case 18:
   case 7: case 10: case 14: case 17: case 25: case 28: case 32: case 35:
   case 1: case  2: case  5: case 20: case 22: case 23:
   case 6: case 13: case 16: case 27: case 31: case 34:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   switch (m_iDirection) {
   case 0: case 21:
   case 3: case 18:
   case 7: case 10: case 14: case 17: case 25: case 28: case 32: case 35:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      ::BitBltMasked(
         m_GContext.m_hDCDst, GetOuterRect(),
         m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
         m_pRegion, 4, GetOuterRect().pointLT());
      break;
   case 1: case  2: case  5: case 20: case 22: case 23:
   case 4: case  8: case 11: case 19: case 26: case 29:
   case 6: case 13: case 16: case 27: case 31: case 34:
   case 9: case 12: case 15: case 24: case 30: case 33:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      ::BitBltMasked(
         m_GContext.m_hDCDst, GetOuterRect(),
         m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
         m_pRegion, 3, GetOuterRect().pointLT());
      break;
   }
}
