////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Players.cpp" -    
// Copyright (C) 2003-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Players.h"
#include <TmSchema.h>
#include <algorithm>
#include "./Dialog/Administration.h"
#include "./OldVersion.h"

////////////////////////////////////////////////////////////////////////////////
//                      implemantation CPlayers
////////////////////////////////////////////////////////////////////////////////

CPlayers::CPlayers() {
   this->LoadFile();
}

CPlayers::~CPlayers() {
   this->SaveFile();
}

int CPlayers::FindName(LPCTSTR szPlayerName) const {
   for (int i=0; i<m_Body.m_Players.size(); i++) {
      if (m_Body.m_Players[i].m_strPlayerName == szPlayerName)
         return i;
   }
   return -1;
};

size_t CPlayers::AddNewRecord(LPCTSTR szPlayerName, LPCTSTR szPassword) {
   int index = FindName(szPlayerName);
   if (index >= 0) return index; //    
   nsPlayers::CRecord rec;
   rec.m_strPlayerName = szPlayerName;
   rec.m_strPassword   = szPassword;
   m_Body.m_Players.push_back(rec);
   return m_Body.m_Players.size()-1;
};

bool CPlayers::RemovePlayer(int index) {
   if (!IndexCheck(index))
      return false;
   m_Body.m_Players.erase(std::find(m_Body.m_Players.begin(), m_Body.m_Players.end(), m_Body.m_Players[index]));
   return true;
}

bool CPlayers::ModifyData(LPCTSTR szPlayerName, const EMosaic mosaic, const ESkillLevel skill, bool bVictory, DWORD dwOpenField, DWORD dwPlayTime, DWORD dwClickCount) {
   if (skill == eSkillLevelCustom) return false;
   int index = FindName(szPlayerName);
   if (index < 0) {
      if (lstrcmp(szPlayerName, CDialogAdministration::DEFAULT_PLAYER_NAME()) == 0) {
         index = AddNewRecord(szPlayerName, _T(""));
         SetAvatar(index, _T("?ICO#Pause3"));
      } else {
         return false;
      }
   }
   nsPlayers::CSubRecord &subRec = m_Body.m_Players[index].m_Record[mosaic][skill];
   subRec.m_dwGameNumber++;
   subRec.m_dwGameWin    += bVictory ? 1:0;
   subRec.m_dwOpenField  += dwOpenField;
   subRec.m_dwPlayTime   += dwPlayTime;
   subRec.m_dwClickCount += dwClickCount;
   return true;
}

BOOL CPlayers::Convert(HANDLE hFile) {
   CString strError;
   ::SetFilePointer(hFile, 0, NULL, FILE_BEGIN);

   //   
   DWORD dwNOBR = 0;
   CHAR szVersion[chDIMOF(ID_VERSIONINFO_VERSION3_v210)];
   if (!::ReadFile(hFile, szVersion, sizeof(ID_VERSIONINFO_VERSION3_v210), &dwNOBR, NULL)) {
      strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_READ);
   } else {
      if (sizeof(ID_VERSIONINFO_VERSION3_v210) != dwNOBR) {
         strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_READ); //     
      } else {
         if (lstrcmpA(ID_VERSIONINFO_VERSION3_v210, szVersion)) {
            strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_VERSION);
         } else {
            //   (  -   )
            const DWORD dwSizeData = ::GetFileSize(hFile, NULL)-dwNOBR; //    
            nsVer210::CSttstcFile StcFile;
            for (int i=0; i<dwSizeData/sizeof(nsVer210::CSttstcFile); i++) {
               if (!ReadFile(hFile, &StcFile, sizeof(nsVer210::CSttstcFile), &dwNOBR, NULL) ||
                   (sizeof(nsVer210::CSttstcFile) != dwNOBR) 
                  )
               {
                  strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_READ); //  
                  break;
               } else {
                  DWORD dwVerifyByteCRC = 0;
                  for (int j=0; j<sizeof(nsVer210::CSttstcRecord)/sizeof(DWORD); j++) {
                     *((DWORD*)(&StcFile.m_DataStc)+j) ^= *((DWORD*)(&StcFile.m_DataBad)+j); //  
                     dwVerifyByteCRC                   ^= *((DWORD*)(&StcFile.m_DataStc)+j); //  CRC
                     dwVerifyByteCRC                   ^= *((DWORD*)(&StcFile.m_DataBad)+j); //  CRC
                  }
                  if (dwVerifyByteCRC != StcFile.m_dwCRC) { // ce CRC
                     strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_READ); // CRC  - 
                     break;
                  }
                  const size_t k = m_Body.m_Players.size();
                  m_Body.m_Players.push_back(nsPlayers::CRecord()); //   
                  //        
                  m_Body.m_Players[k].m_strPlayerName = StcFile.m_DataStc.m_szPlayerName;
                  m_Body.m_Players[k].m_strPassword   = StcFile.m_DataStc.m_szPassword;
#define REPLACE_MOSAIC_SKILLLEVEL(mosaicName, eSkillLevel) \
                  m_Body.m_Players[k].m_Record[mosaicName][eSkillLevel].m_dwGameNumber = StcFile.m_DataStc.m_Record[nsVer210::mosaicName][nsVer210::eSkillLevel].m_dwGameNumber; \
                  m_Body.m_Players[k].m_Record[mosaicName][eSkillLevel].m_dwGameWin    = StcFile.m_DataStc.m_Record[nsVer210::mosaicName][nsVer210::eSkillLevel].m_dwGameWin   ; \
                  m_Body.m_Players[k].m_Record[mosaicName][eSkillLevel].m_dwOpenField  = StcFile.m_DataStc.m_Record[nsVer210::mosaicName][nsVer210::eSkillLevel].m_dwOpenField ; \
                  m_Body.m_Players[k].m_Record[mosaicName][eSkillLevel].m_dwPlayTime   = StcFile.m_DataStc.m_Record[nsVer210::mosaicName][nsVer210::eSkillLevel].m_dwPlayTime  ; \
                  m_Body.m_Players[k].m_Record[mosaicName][eSkillLevel].m_dwClickCount = StcFile.m_DataStc.m_Record[nsVer210::mosaicName][nsVer210::eSkillLevel].m_dwClickCount
#define REPLACE_MOSAIC(mosaicName) \
                  REPLACE_MOSAIC_SKILLLEVEL(mosaicName, eSkillLevelBeginner    ); \
                  REPLACE_MOSAIC_SKILLLEVEL(mosaicName, eSkillLevelAmateur     ); \
                  REPLACE_MOSAIC_SKILLLEVEL(mosaicName, eSkillLevelProfessional); \
                  REPLACE_MOSAIC_SKILLLEVEL(mosaicName, eSkillLevelCrazy       )

                  REPLACE_MOSAIC(eMosaicTriangle1   );
                  REPLACE_MOSAIC(eMosaicTriangle2   );
                  REPLACE_MOSAIC(eMosaicTriangle3   );
                  REPLACE_MOSAIC(eMosaicTriangle4   );
                  REPLACE_MOSAIC(eMosaicSquare1     );
                  REPLACE_MOSAIC(eMosaicSquare2     );
                  REPLACE_MOSAIC(eMosaicParquet1    );
                  REPLACE_MOSAIC(eMosaicParquet2    );
                  REPLACE_MOSAIC(eMosaicTrapezoid1  );
                  REPLACE_MOSAIC(eMosaicTrapezoid2  );
                  REPLACE_MOSAIC(eMosaicTrapezoid3  );
                  REPLACE_MOSAIC(eMosaicRhombus1    );
                  REPLACE_MOSAIC(eMosaicQuadrangle1 );
                  REPLACE_MOSAIC(eMosaicPentagonT24 );
                  REPLACE_MOSAIC(eMosaicPentagonT5  );
                  REPLACE_MOSAIC(eMosaicPentagonT10 );
                  REPLACE_MOSAIC(eMosaicHexagon1    );
                  REPLACE_MOSAIC(eMosaicTrSq1       );
                  REPLACE_MOSAIC(eMosaicTrSq2       );
                  REPLACE_MOSAIC(eMosaicSqTrHex     );
               }
            }
            if (dwSizeData%sizeof(nsVer210::CSttstcFile)) {
               strError = CLang::GetString(IDS__PLAYERSandSTATISTIC__ERROR_READ); //   ???
            }
         }
      }
   }
   return strError.IsEmpty();
}

