////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "StorageMines.cpp"
//   ,     
// Copyright (C) 2005-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "StorageMines.h"

CStorageMines::VECTOR_COORD_ITERATOR CStorageMines::Find(const COORD &Coord) {
   size_t size = GetSize();
   for (VECTOR_COORD_ITERATOR i=m_CellMines.begin(); i!=m_CellMines.end(); i++) {
      if (*i == Coord)
         return i;
   }
   return m_CellMines.end();
}

BOOL CStorageMines::Del(const COORD &Coord) {
   VECTOR_COORD_ITERATOR i = Find(Coord);
   if (i != m_CellMines.end()) {
      m_CellMines.erase(i);
      return true;
   }
#ifdef _DEBUG
   else
      ::MessageBox(NULL, _T("CStorageMines::Del()"), _T("Error"), MB_OK | MB_ICONERROR);
#endif
   return false;
}

BOOL CStorageMines::Add(const COORD &Coord) {
   if (Find(Coord) == m_CellMines.end()) {
      m_CellMines.push_back(Coord);
      return true;
   }
#ifdef _DEBUG
   else
      ::MessageBox(NULL, _T("CStorageMines::Add()"), _T("Error"), MB_OK | MB_ICONERROR);
#endif
   return false;
}

BOOL CStorageMines::SerializeRead(HANDLE hFile) {
   Reset();
   BOOL bRes = FALSE;
   do {
      DWORD dwSizeToEOF = ::GetFileSize(hFile, NULL) - ::SetFilePointer(hFile, 0, NULL, FILE_CURRENT);
      if (dwSizeToEOF == 0) break;
      COORD c = {0,0};
      DWORD dwNOBR = 0;
      bRes = ::ReadFile(hFile, &c, sizeof(c), &dwNOBR, NULL) && (sizeof(c) == dwNOBR);
      if (bRes)
         bRes = Add(c);
   } while(bRes);
   return bRes;
}

BOOL CStorageMines::SerializeWrite(HANDLE hFile) const {
   BOOL bRes = TRUE;
   for (int i=0; bRes && (i<GetSize()); i++) {
      DWORD dwNOBW = 0;
      COORD c = m_CellMines[i];
      bRes = ::WriteFile(hFile, &c, sizeof(c), &dwNOBW, NULL) && (sizeof(c) == dwNOBW);
   }
   return bRes;
}
